import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_js/quickjs/ffi.dart';
import 'package:win_text_editor/framework/models/tab_model.dart';
import 'package:win_text_editor/modules/module_router.dart';
import 'package:win_text_editor/shared/base/base_content_controller.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';

class TabItemsController with ChangeNotifier {
  final List<AppTab> _tabs = [];
  final Map<String, BaseContentController> _contentControllers = {}; // 保存各Tab的Controller
  String? _activeTabId;

  List<AppTab> get tabs => _tabs;
  String? get activeTabId => _activeTabId;

  AppTab? get activeTab {
    if (_activeTabId == null) return null;
    try {
      return _tabs.firstWhere((tab) => tab.id == _activeTabId);
    } catch (e) {
      Logger().error("找不到活动选项卡: $_activeTabId", source: 'EditorProvider');
      return null;
    }
  }

  Future<void> addTab(
    String id, {
    String title = '未命名',
    String? type,
    IconData? icon,
    String content = '',
  }) async {
    //创建tab组件
    final newTab = AppTab(id: id, title: title, type: type, icon: icon, content: content);

    _tabs.add(newTab);

    // 创建provider，管理状态
    final controller = createContentController(newTab);
    if (controller != null) {
      _contentControllers[id] = controller;
    }
    _activeTabId = id;
    notifyListeners();
  }

  AppTab? getTabById(String tabId) {
    try {
      return _tabs.firstWhere((tab) => tab.id == tabId);
    } catch (e) {
      Logger().error("找不到选项卡: $tabId", source: 'EditorProvider');
      return null;
    }
  }

  BaseContentController? createContentController(AppTab tab) {
    return ModuleRouter.createControllerForTab(tab);
  }

  BaseContentController? get activeContentController {
    if (_activeTabId == null) {
      return null;
    }
    final controller = _contentControllers[_activeTabId];
    if (controller == null) {
      Logger().error("没有活动的内容控制器,activeTabId:$_activeTabId", source: 'TabItemsController');
      return null;
    }
    // ignore: unnecessary_type_check
    if (controller is! BaseContentController) {
      Logger().error("活动内容控制器不是BaseContentController类型", source: 'TabItemsController');
      return null;
    }
    return controller;
  }

  void closeTab(String tabId) {
    final controller = _contentControllers[tabId];
    controller?.dispose(); // 先释放Controller
    _contentControllers.remove(tabId); // 移除Controller引用
    _tabs.removeWhere((tab) => tab.id == tabId); // 再移除Tab

    if (_activeTabId == tabId) {
      _activeTabId = _tabs.isNotEmpty ? _tabs.last.id : null;
    }
    notifyListeners();
  }

  void registerController(String tabId, BaseContentController controller) {
    _contentControllers[tabId] = controller;
  }

  T? getController<T extends ChangeNotifier>(String tabId) {
    return _contentControllers[tabId] as T?;
  }

  void disposeController(String tabId) {
    _contentControllers[tabId]?.dispose();
    _contentControllers.remove(tabId);
  }

  void setActiveTab(String tabId) {
    if (_activeTabId == tabId) return; // 如果已经是活动Tab则不通知
    _activeTabId = tabId;
    notifyListeners();
  }

  void handleFolderDoubleTap(String folderPath) {
    activeContentController?.onOpenFolder(folderPath);
  }

  void handleFileDoubleTap(String filePath, dynamic appendArg) {
    Logger().debug('双击事件参数:filePath:$filePath，appendArg：$appendArg');
    final fileName = filePath.split(Platform.pathSeparator).last;
    if (fileName == "component.xml") {
      openOrActivateTab("标准组件", RouterKey.uftComponent, Icons.extension);
    } else {
      final fileExtension = _getFileExtension(fileName);
      switch (fileExtension) {
        case 'uftstructure':
          openOrActivateTab("内存表", RouterKey.memoryTable, Icons.list);
          break;
        case 'uftfunction':
        case 'uftatomfunction':
        case 'uftatomservice':
        case 'uftfactorfunction':
        case 'uftfactorservice':
          openOrActivateTab("函数调用", RouterKey.callFunction, Icons.functions);
          break;
        default:
          Logger().error("没有活动的内容控制器", source: 'TabItemsController');
      }
    }

    activeContentController?.onOpenFile(filePath, appendArg: appendArg);
  }

  bool hasController(String tabId) {
    return _contentControllers.containsKey(tabId);
  }

  // 新增方法
  Future<void> openOrActivateTab(String title, String type, IconData icon) async {
    try {
      final existingTab = _tabs.firstWhereOrNull((tab) => tab.type == type);
      Logger().debug("获取活动Tab页${existingTab?.title}");

      if (existingTab != null) {
        setActiveTab(existingTab.id);
      } else {
        final tabId = DateTime.now().millisecondsSinceEpoch.toString();
        await addTab(tabId, title: title, type: type, icon: icon, content: "");
      }
    } catch (e) {
      Logger().error("打开或激活标签页失败: $e", source: 'TabItemsController');
    }
  }

  static String _getFileExtension(String filePath) {
    final parts = filePath.split('.');
    return parts.length > 1 ? parts.last.toLowerCase() : '';
  }
}
