import 'dart:io';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/framework/models/file_node.dart';
import 'package:win_text_editor/framework/services/fast_xml_parser.dart';
import 'package:xml/xml.dart';

class FileService {
  static const _specialExtensions = [
    '.uftfunction',
    '.uftservice',
    '.uftatomfunction',
    '.uftatomservice',
    '.uftfactorfunction',
    '.uftfactorservice',
  ];
  static const Map<String, String> _uftFloders = {
    '.settings': '项目设置',
    'metadata': '元数据',
    'tools': '工具资源',
    'uftatom': 'UFT原子',
    'uftbusiness': 'UFT业务逻辑',
    'uftfactor': 'UFT因子',
    'uftstructure': 'UFT对象',
  };
  static const _hiddenFiles = ['.classpath', '.project', '.respath', 'project.xml', 'module.xml'];

  static Future<String?> getSpecialFileName(String filePath) async {
    final extension = filePath.substring(filePath.lastIndexOf('.'));
    if (!_specialExtensions.contains(extension)) {
      return null;
    }

    try {
      final result = await FastXmlParser.parse(filePath);
      return ('[${result['objectId']}]${result['chineseName']}');
    } catch (e) {
      Logger().debug('Error reading special file: $e');
    }
    return null;
  }

  /// 延时加载目录内容（不递归）
  static Future<List<FileNode>> listDirectory(String path, {int parentDepth = 0}) async {
    final dir = Directory(path);
    final List<FileSystemEntity> entities = await dir.list().toList();
    final List<FileNode> nodes = [];
    // final stopwatch = Stopwatch()..start();

    for (final entity in entities) {
      final pathName = entity.path.split(Platform.pathSeparator).last;
      if (_hiddenFiles.contains(pathName)) continue;

      final isDirectory = await FileSystemEntity.isDirectory(entity.path);
      final displayName =
          isDirectory
              ? await getModuleDisplayName(entity.path)
              : await getSpecialFileName(entity.path);

      nodes.add(
        FileNode(
          name: displayName ?? pathName,
          path: entity.path,
          isDirectory: isDirectory,
          depth: parentDepth + 1,
        ),
      );
    }

    // stopwatch.stop();
    // Logger().debug('执行耗时: ${stopwatch.elapsedMilliseconds} 毫秒 (ms)');

    return nodes;
  }

  static Future<String?> getModuleDisplayName(String dirPath) async {
    try {
      final floderName = dirPath.split(Platform.pathSeparator).last;
      if (_uftFloders.containsKey(floderName)) return _uftFloders[floderName];

      final moduleFile = File('$dirPath${Platform.pathSeparator}module.xml');
      if (await moduleFile.exists()) {
        final content = await moduleFile.readAsString();
        final xmlDoc = XmlDocument.parse(content);
        final infoNode = xmlDoc.findAllElements('info').firstOrNull;
        return infoNode?.getAttribute('cname');
      }
    } catch (e) {
      Logger().debug('Error reading module.xml: $e');
    }
    return null;
  }

  /// 递归构建完整文件树（原方法保留备用）
  static Future<List<FileNode>> buildFileTree(String rootPath) async {
    final rootDirectory = Directory(rootPath);
    final List<FileNode> nodes = [];

    if (await rootDirectory.exists()) {
      final entities = rootDirectory.listSync();

      for (final entity in entities) {
        final pathName = entity.path.split(Platform.pathSeparator).last;
        if (_hiddenFiles.contains(pathName)) continue;
        final node = FileNode(name: pathName, path: entity.path, isDirectory: entity is Directory);

        if (entity is Directory) {
          node.children.addAll(await buildFileTree(entity.path));
        }

        nodes.add(node);
      }
    }

    return nodes;
  }

  static Future<String> readFile(String filePath) async {
    return await File(filePath).readAsString();
  }

  static Future<void> writeFile(String filePath, String content) async {
    await File(filePath).writeAsString(content);
  }
}
