// memory_table_service.dart
import 'dart:io';

import 'package:collection/collection.dart';
import 'package:win_text_editor/modules/call_function/models/call_function.dart';
import 'package:win_text_editor/modules/uft_component/models/uft_component.dart';
import 'package:win_text_editor/modules/uft_component/services/uft_component_service.dart';
import 'package:win_text_editor/shared/data/std_fields_cache.dart';
import 'package:win_text_editor/shared/models/std_filed.dart';
import 'package:xml/xml.dart' as xml;
import 'package:win_text_editor/framework/controllers/logger.dart';

class CallFunctionService {
  static const Map<String, String> _functionRootTypeMap = {
    'uftfunction': 'business:Function',
    'uftservice': 'business:Service',
    'uftatomfunction': 'business:Function',
    'uftatomservice': 'business:Service',
    'uftfactorfunction': 'business:FactorFunction',
    'uftfactorservice': 'business:FactorService',
  };

  static Future<CallFunction> parseXmlFile(String filePath) async {
    try {
      // 1. Check file extension
      final extendFileName = filePath.toLowerCase().split('.').last;
      final String? rootNodeType = _functionRootTypeMap[extendFileName];
      if (rootNodeType == null) {
        Logger().error('文件扩展名不正确:$filePath');
        throw const FormatException("文件扩展名不正确!");
      }

      // 2. 查找 metadata 目录和 stdfield.stfield 文件
      await StdFieldsCache.loadForFile(filePath);

      // 3. Read and parse structure file content
      final file = File(filePath);
      final content = await file.readAsString();

      final document = xml.XmlDocument.parse(content);
      final rootNode = document.findAllElements(rootNodeType).firstOrNull;

      if (rootNode == null) {
        throw const FormatException("文件格式错误：缺少支持的根节点");
      }

      // 4. Get basic info
      final chineseName = rootNode.getAttribute('chineseName') ?? '';
      final objectId = rootNode.getAttribute('objectId') ?? '';

      List<UftComponent> componentList = [];

      List<UftComponent> outComponentList = [];

      // 5. Process inputParameters (fields)
      final inputParameters = document.findAllElements(
        extendFileName == "uftfactorservice" ? 'internalParams' : 'inputParameters',
      );
      final inputFields = await parserFields(filePath, inputParameters, componentList);

      final factorParam =
          extendFileName == "uftfactorservice"
              ? inputFields
                  .firstWhereOrNull((field) => field.type == CallFunction.componentType)
                  ?.name
              : '';

      // 6. Process outputParameters
      final outputParameters = document.findAllElements('outputParameters');
      final outputFields = await parserFields(filePath, outputParameters, outComponentList);

      return CallFunction(
        functionType: extendFileName,
        chineseName: chineseName,
        functionNo: objectId,
        inputParameters: inputFields,
        outputParameters: outputFields,
        componentList: componentList,
        outComponentList: outComponentList,
        factorParam: factorParam,
      );
    } on xml.XmlParserException catch (e) {
      Logger().error("XML解析错误: ${e.message}");
      rethrow;
    } catch (e) {
      Logger().error("处理文件时发生错误: $e");
      rethrow;
    }
  }

  static Future<List<Field>> parserFields(
    String filePath,
    Iterable<xml.XmlElement> parameters,
    List<UftComponent> componentList,
  ) async {
    final fields = <Field>[];
    int index = 0;

    for (final parameter in parameters) {
      index++;
      final id = parameter.getAttribute('id') ?? '';
      final paramType = parameter.getAttribute('paramType') ?? 'FIELD';

      if (paramType == CallFunction.componentType) {
        var component = await UftComponentService.getUftComponent(filePath, id);
        if (component != null) {
          componentList.add(component);
        }
        fields.add(
          Field(
            index.toString(),
            id,
            component?.chineseName ?? '', // 使用组件的中文名
            paramType,
          ),
        );
      } else {
        final stdField = StdFieldsCache.getData(id);
        fields.add(
          Field(index.toString(), id, stdField?.chineseName ?? '', stdField?.dateType ?? ''),
        );
      }
    }

    return fields;
  }
}
