import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/framework/controllers/tab_items_controller.dart';
import 'package:win_text_editor/modules/code_creater/controllers/code_creater_controller.dart';
import 'package:win_text_editor/modules/code_creater/widgets/node_table.dart';
import 'package:win_text_editor/shared/components/code_generation_components.dart';

class CodeCreaterView extends StatefulWidget {
  final String tabId;
  const CodeCreaterView({super.key, required this.tabId});

  @override
  State<CodeCreaterView> createState() => _CodeCreaterViewState();
}

class _CodeCreaterViewState extends State<CodeCreaterView> {
  late final CodeCreaterController _controller;
  bool _isControllerFromTabManager = false;
  final TextEditingController _codeController = TextEditingController();
  final List<String> operations = ['逻辑服务', '逻辑函数', '原子服务', '原子函数'];

  get tabManager => Provider.of<TabItemsController>(context, listen: false);

  @override
  void initState() {
    super.initState();

    final controllerFromManager = tabManager.getController(widget.tabId);
    if (controllerFromManager != null) {
      _controller = controllerFromManager;
      _isControllerFromTabManager = true;
    } else {
      _controller = CodeCreaterController();
      _isControllerFromTabManager = false;
      tabManager.registerController(widget.tabId, _controller);
    }

    _controller.addListener(_handleControllerUpdate);
    _controller.addListener(_updateDisplay);
  }

  void _handleControllerUpdate() {
    setState(() {});
  }

  @override
  void dispose() {
    _controller.removeListener(_handleControllerUpdate);
    _controller.removeListener(_updateDisplay);
    if (!_isControllerFromTabManager) {
      _controller.dispose();
    }
    _codeController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      children: [
        // 左侧表格 (40%)
        Expanded(
          flex: 4,
          child: Padding(
            padding: const EdgeInsets.all(8.0),
            child: NodeTable(
              key: ValueKey(_controller.members.length), // 确保唯一绑定
              members: _controller.members,
              onMoveMember: _moveMember,
              onDeleteMember: _deleteMember,
              onDeleteAll: _deleteAll,
              onMoveToTop: _moveToTop,
              onMoveToBottom: _moveToBottom,
              onActionTypeChanged:
                  (index, newAction) => _controller.handleActionTypeChange(index, newAction),
            ),
          ),
        ),

        // 右侧代码生成区 (60%)
        Expanded(
          flex: 6,
          child: Padding(
            padding: const EdgeInsets.all(4.0),
            child: CodeGenerationSection(
              title: '生成代码',
              codeController: _codeController,
              onNodeDropped: (node) => _controller.onDropOutlineNode(node),
              child: OperationRadioSection(
                operations: operations,
                selectedOperation: _controller.selectedOperation,
                onOperationSelected: _selectOperation,
              ),
            ),
          ),
        ),
      ],
    );
  }

  void _moveMember(int index, int direction) {
    _move(index, index + direction);
  }

  void _move(int from, int to) async {
    if (from < 0 ||
        from >= _controller.members.length ||
        to < 0 ||
        to >= _controller.members.length) {
      return; // 越界检查
    }

    final member = _controller.members[from];
    setState(() {
      _controller.members.removeAt(from);
    });

    await Future.delayed(const Duration(milliseconds: 50)); // 让 UI 响应删除操作

    setState(() {
      _controller.members.insert(to, member); // 再插入
      _updateDisplay();
    });
  }

  void _moveToTop(int index) {
    _move(index, 0);
  }

  void _moveToBottom(int index) {
    _move(index, _controller.members.length - 1);
  }

  void _deleteMember(int index) {
    setState(() {
      final memberName = _controller.members[index].name;
      //同时删除遍历结束节点
      _controller.members.removeWhere((member) => member.name == '${memberName}_END');

      _controller.members.removeAt(index);
      _updateDisplay();
    });
  }

  void _deleteAll() {
    setState(() {
      _controller.members.clear();
      _updateDisplay();
    });
  }

  void _selectOperation(String? operation) {
    if (operation == null || operation.isEmpty) {
      return;
    }
    setState(() {
      _controller.selectedOperation = operation;
      _updateDisplay();
    });
  }

  Future<void> _updateDisplay() async {
    _codeController.text = await _controller.genCodeString();
  }
}
