import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:pluto_grid/pluto_grid.dart';
import 'package:win_text_editor/framework/common/constants.dart';
import 'package:win_text_editor/modules/code_creater/services/code_create_service.dart';
import 'package:win_text_editor/modules/outline/models/outline_node.dart';
import 'package:win_text_editor/shared/components/my_pluto_column.dart';
import 'package:win_text_editor/shared/components/my_pluto_configuration.dart';
import 'package:win_text_editor/shared/components/my_pluto_dropdown_column.dart';

class NodeTable extends StatefulWidget {
  final List<OutlineNode> members;
  final Function(int, int) onMoveMember;
  final Function(int) onDeleteMember;
  final Function() onDeleteAll;
  final Function(int) onMoveToTop;
  final Function(int) onMoveToBottom;
  final Function(int, String)? onActionTypeChanged; // 新增回调函数

  const NodeTable({
    super.key,
    required this.members,
    required this.onMoveMember,
    required this.onDeleteMember,
    required this.onDeleteAll,
    required this.onMoveToTop,
    required this.onMoveToBottom,
    this.onActionTypeChanged, // 新增参数
  });

  @override
  State<NodeTable> createState() => NodeTableState();
}

class _NodeTypeData {
  final String originalType;
  String currentSelection;

  _NodeTypeData(this.originalType, this.currentSelection);
}

class NodeTableState extends State<NodeTable> {
  PlutoGridStateManager? _stateManager;
  int? _selectedRowIndex;
  final ScrollController _scrollController = ScrollController();

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        // Header with controls
        _buildHeader(),
        // Table
        Expanded(
          child: ScrollConfiguration(
            behavior: ScrollConfiguration.of(context).copyWith(
              dragDevices: {
                PointerDeviceKind.touch,
                PointerDeviceKind.mouse,
                PointerDeviceKind.stylus,
                PointerDeviceKind.trackpad,
              },
            ),
            child: Scrollbar(
              controller: _scrollController,
              thickness: 12.0,
              thumbVisibility: true,
              interactive: true,
              child: SingleChildScrollView(
                controller: _scrollController,
                scrollDirection: Axis.horizontal,
                child: SizedBox(
                  width: MediaQuery.of(context).size.width * 0.4,
                  child: PlutoGrid(
                    key: widget.key,
                    configuration: MyPlutoGridConfiguration(),
                    columns: _buildColumns(),
                    mode: PlutoGridMode.normal,
                    rows: widget.members.isEmpty ? [] : _buildRows(widget.members),
                    noRowsWidget: const Center(child: Text('')),
                    onLoaded: (PlutoGridOnLoadedEvent event) {
                      _stateManager = event.stateManager;
                      _stateManager?.setSelectingMode(PlutoGridSelectingMode.row);
                      _stateManager?.addListener(_handleRowSelection);
                    },
                    onChanged: (PlutoGridOnChangedEvent event) {
                      _handleCellChanged(event);
                    },
                    onRowDoubleTap: (event) {
                      _selectedRowIndex = event.rowIdx;
                    },
                    rowColorCallback: (rowContext) {
                      // 当行被选中时设置背景色
                      return rowContext.stateManager.currentRow?.key == rowContext.row.key
                          ? Colors.blue[200]! // 选中行颜色
                          : Colors.white; // 默认行颜色（不能返回 null）
                    },
                  ),
                ),
              ),
            ),
          ),
        ),
      ],
    );
  }

  Widget _buildHeader() {
    final canMoveUp = _selectedRowIndex != null && _selectedRowIndex! > 0;
    final canMoveDown = _selectedRowIndex != null && _selectedRowIndex! < widget.members.length - 1;
    final canDelete = _selectedRowIndex != null;
    final canDeleteAll = widget.members.isNotEmpty;

    return Padding(
      padding: const EdgeInsets.all(2.0),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          const Text('已添加节点', style: TextStyle(fontWeight: FontWeight.bold)),
          Row(
            children: [
              IconButton(
                icon: Icon(
                  Icons.arrow_upward,
                  size: 14,
                  color: canMoveUp ? Colors.blue : Colors.grey,
                ),
                onPressed: canMoveUp ? () => widget.onMoveMember(_selectedRowIndex!, -1) : null,
                tooltip: '上移一行',
              ),
              IconButton(
                icon: Icon(
                  Icons.vertical_align_top,
                  size: 14,
                  color: canMoveUp ? Colors.blue : Colors.grey,
                ),
                onPressed: canMoveUp ? () => widget.onMoveToTop(_selectedRowIndex!) : null,
                tooltip: '上移到顶',
              ),
              IconButton(
                icon: Icon(
                  Icons.arrow_downward,
                  size: 14,
                  color: canMoveDown ? Colors.blue : Colors.grey,
                ),
                onPressed: canMoveDown ? () => widget.onMoveMember(_selectedRowIndex!, 1) : null,
                tooltip: '下移一行',
              ),
              IconButton(
                icon: Icon(
                  Icons.vertical_align_bottom,
                  size: 14,
                  color: canMoveDown ? Colors.blue : Colors.grey,
                ),
                onPressed: canMoveDown ? () => widget.onMoveToBottom(_selectedRowIndex!) : null,
                tooltip: '下移到底',
              ),
              IconButton(
                icon: Icon(Icons.delete, size: 14, color: canDelete ? Colors.red : Colors.grey),
                onPressed: canDelete ? () => widget.onDeleteMember(_selectedRowIndex!) : null,
                tooltip: '删除行',
              ),
              IconButton(
                icon: Icon(
                  Icons.delete_forever,
                  size: 14,
                  color: canDeleteAll ? Colors.red : Colors.grey,
                ),
                onPressed: canDeleteAll ? _showDeleteAllConfirmation : null,
                tooltip: '全部删除',
              ),
            ],
          ),
        ],
      ),
    );
  }

  void _showDeleteAllConfirmation() {
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
          title: const Text('确认删除'),
          content: const Text('确定要删除所有节点吗？此操作不可撤销。'),
          actions: [
            TextButton(onPressed: () => Navigator.of(context).pop(false), child: const Text('取消')),
            TextButton(
              onPressed: () {
                Navigator.of(context).pop(true);
                _deleteAllMembers();
              },
              child: const Text('删除', style: TextStyle(color: Colors.red)),
            ),
          ],
        );
      },
    );
  }

  // 添加删除所有成员的方法
  void _deleteAllMembers() {
    setState(() {
      _selectedRowIndex = null;
    });
    widget.onDeleteAll();
  }

  void _handleRowSelection() {
    if (_stateManager?.currentRow == null) return;
    setState(() {
      _selectedRowIndex = _stateManager?.currentRow?.sortIdx;
    });
  }

  void _handleCellChanged(PlutoGridOnChangedEvent event) {
    if (event.column?.field == 'type' && event.rowIdx != null) {
      final rowIndex = event.rowIdx!;
      if (rowIndex >= 0 && rowIndex < widget.members.length) {
        final newValue = event.value?.toString() ?? '';
        setState(() {
          widget.members[rowIndex].selectedAction = newValue;
        });

        // 通知controller
        if (widget.onActionTypeChanged != null) {
          widget.onActionTypeChanged!(rowIndex, newValue);
        }
      }
    }
  }

  final Map<Key, _NodeTypeData> _nodeTypeData = {};

  List<PlutoColumn> _buildColumns() {
    return [
      MyPlutoColumn(
        title: '#',
        field: 'rowNum',
        width: 40,
        enableRowChecked: false, // 允许整行选中
      ),
      MyPlutoColumn(
        title: '成员',
        field: 'content',
        width: 200,
        enableRowChecked: false, // 允许整行选中
      ),
      MyPlutoDropdownColumn(
        title: '操作类型',
        field: 'type',
        width: 120,
        enableRowChecked: false, // 允许整行选中
        optionsBuilder: (rowKey) {
          final data = _nodeTypeData[rowKey as ValueKey];
          return _getOptions(data?.originalType ?? '');
        },
      ),
    ];
  }

  List<PlutoRow> _buildRows(List<OutlineNode> members) {
    return members
        .asMap()
        .map((index, member) {
          final rowKey = ValueKey('${member.hashCode}_$index');
          final options = _getOptions(member.value);
          // Use selectedAction if it exists, otherwise use the first option
          final initialValue =
              member.selectedAction?.isNotEmpty == true
                  ? member.selectedAction!
                  : options.isNotEmpty
                  ? options.first
                  : '';

          _nodeTypeData[rowKey] = _NodeTypeData(member.value, initialValue);

          return MapEntry(
            index,
            PlutoRow(
              key: rowKey,
              cells: {
                'rowNum': PlutoCell(value: index + 1),
                'content': PlutoCell(value: member.title),
                'type': PlutoCell(value: initialValue),
              },
            ),
          );
        })
        .values
        .toList();
  }

  void refreshTable() {
    if (_stateManager != null) {
      _stateManager!.refRows.clear();
      _stateManager!.refRows.addAll(_buildRows(widget.members));
      _stateManager!.notifyListeners();
    }
  }

  @override
  void dispose() {
    _stateManager?.removeListener(_handleRowSelection);
    _scrollController.dispose();
    super.dispose();
  }

  List<String> _getOptions(String s) {
    return CodeCreateService.getNodeActions(s);
  }
}
