// content_search_controller.dart

import 'dart:async';
import 'dart:io';

import 'package:file_picker/file_picker.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/modules/content_search/models/search_mode.dart';
import 'package:win_text_editor/modules/content_search/models/search_result.dart';
import 'package:win_text_editor/modules/content_search/services/count_search_service.dart';
import 'package:win_text_editor/modules/content_search/services/locate_search_service.dart';
import 'package:win_text_editor/modules/outline/models/outline_node.dart';
import 'package:win_text_editor/shared/base/base_content_controller.dart';
import '../services/custom_search_service.dart';

class ContentSearchController extends BaseContentController {
  String _searchQuery = '';
  String _searchDirectory = '';
  String _fileType = '*.*';
  String _jumpFiles = '';
  bool _caseSensitive = false;
  bool _wholeWord = false;
  bool _useRegex = false;
  bool _customRule = false;
  SearchMode _searchMode = SearchMode.locate;
  final List<SearchResult> _results = [];
  final List<SearchResult> _allResults = [];
  bool _onlyZeroCounts = false;
  bool _isSearching = false;
  bool _shouldStop = false;

  // Getters
  String get searchQuery => _searchQuery;
  String get searchDirectory => _searchDirectory;
  String get fileType => _fileType;
  String get jumpFiles => _jumpFiles;
  bool get caseSensitive => _caseSensitive;
  bool get wholeWord => _wholeWord;
  bool get useRegex => _useRegex;
  bool get customRule => _customRule;
  SearchMode get searchMode => _searchMode;
  List<SearchResult> get results => _results;
  bool get onlyZeroCounts => _onlyZeroCounts;

  double _progress = 0;
  double get progress => _progress;

  bool get isSearching => _isSearching;

  // 添加进度更新方法
  void _updateProgress(double value) {
    final newProgress = value.clamp(0.0, 100.0);
    if (newProgress <= _progress) return;
    _progress = newProgress;
    notifyListeners();
  }

  // 添加停止搜索方法
  void stopSearch() {
    _shouldStop = true;
    _isSearching = false;
    notifyListeners();
  }

  set customRule(bool value) {
    _customRule = value;
    notifyListeners();
  }

  void setInitialSearchQuery(String query) {
    if (_searchQuery != query) {
      _searchQuery = query;
      notifyListeners();
    }
  }

  // Setters with notifyListeners
  Timer? _searchDebounce;
  set searchQuery(String value) {
    _searchDebounce?.cancel();
    _searchDebounce = Timer(const Duration(milliseconds: 500), () {
      _searchQuery = value;
      notifyListeners();
    });
  }

  set searchDirectory(String value) {
    _searchDirectory = value;
    notifyListeners();
  }

  set fileType(String value) {
    _fileType = value;
    notifyListeners();
  }

  set jumpFiles(String value) {
    _jumpFiles = value;
    notifyListeners();
  }

  set caseSensitive(bool value) {
    _caseSensitive = value;
    notifyListeners();
  }

  set wholeWord(bool value) {
    _wholeWord = value;
    notifyListeners();
  }

  set useRegex(bool value) {
    _useRegex = value;
    notifyListeners();
  }

  set searchMode(SearchMode value) {
    _searchMode = value;
    notifyListeners();
  }

  void clearResults() {
    results.clear();
    _allResults.clear();
    notifyListeners();
  }

  set onlyZeroCounts(bool value) {
    _onlyZeroCounts = value;
    _filterResults();
    notifyListeners();
  }

  void _filterResults() {
    _results.clear();
    if (_searchMode == SearchMode.count) {
      _results.addAll(
        _allResults.where((r) => _onlyZeroCounts ? r.lineNumber == 0 : r.lineNumber > 0),
      );
    } else {
      _results.addAll(_allResults);
    }
  }

  // 在 ContentSearchController 类中添加
  void removeResultByFilePath(String filePath) {
    results.removeWhere((result) => result.filePath == filePath);
    notifyListeners(); // 如果控制器是 ChangeNotifier
  }

  Future<void> startSearch() async {
    _shouldStop = false;
    _isSearching = true;
    _progress = 0;
    _results.clear();
    _allResults.clear();
    notifyListeners();

    // 校验搜索内容
    if (searchQuery.isEmpty) {
      Logger().info("搜索内容不能为空");
      return;
    }
    String first50Chars = searchQuery.length > 50 ? searchQuery.substring(0, 50) : searchQuery;
    Logger().debug("开始搜索: $first50Chars...");

    if (searchDirectory.isEmpty || !Directory(searchDirectory).existsSync()) {
      Logger().info("搜索目录不能为空");
      return;
    }

    try {
      if (customRule) {
        final validationResult = CustomSearchService.validateJsRule(searchQuery);
        if (validationResult.isError) {
          Logger().error('JavaScript 语法错误: ${validationResult.rawResult.toString()}');
          return;
        }

        _allResults.addAll(
          await CustomSearchService.performCustomSearch(
            directory: searchDirectory,
            fileType: fileType,
            jsFunction: searchQuery,
            searchMode: searchMode,
            onProgress: _updateProgress,
            shouldStop: () => _shouldStop,
          ),
        );
      } else {
        if (searchMode == SearchMode.locate) {
          _allResults.addAll(
            await LocateSearchService.performSearch(
              directory: searchDirectory,
              query: searchQuery,
              fileType: fileType,
              caseSensitive: caseSensitive,
              wholeWord: wholeWord,
              useRegex: useRegex,
              onProgress: _updateProgress,
              shouldStop: () => _shouldStop,
            ),
          );
        } else {
          final counts = await CountSearchService.performSearch(
            directory: searchDirectory,
            query: searchQuery,
            fileType: fileType,
            jumpFiles: jumpFiles,
            caseSensitive: caseSensitive,
            wholeWord: wholeWord,
            useRegex: useRegex,
            onProgress: _updateProgress,
            shouldStop: () => _shouldStop,
          );

          counts.forEach((keyword, count) {
            _allResults.add(
              SearchResult(
                filePath: keyword,
                lineNumber: count,
                lineContent: '',
                matches: [],
                queryTerm: keyword,
              ),
            );
          });
        }
      }
    } catch (e) {
      Logger().error("搜索出错: $e");
    } finally {
      _isSearching = false;
      _progress = 100;
      _shouldStop = false;
      _filterResults();
      notifyListeners();
    }
  }

  Future<void> pickDirectory() async {
    final dir = await FilePicker.platform.getDirectoryPath();
    if (dir != null) {
      searchDirectory = dir;
    }
  }

  @override
  void onOpenFile(String filePath, {dynamic appendArg}) {
    //searchDirectory = filePath;
    //notifyListeners();
  }

  @override
  void onOpenFolder(String folderPath) {
    searchDirectory = folderPath;
  }

  @override
  void onDropOutlineNode(OutlineNode node) {
    // TODO: implement onDropOutlineNode
  }
}
