// lib/app/modules/content_search/services/base_search_service.dart
import 'dart:io';
import 'package:collection/collection.dart';
import 'package:flutter/material.dart';
import 'package:win_text_editor/shared/utils/file_utils.dart';
import 'package:docx_to_text/docx_to_text.dart';
import 'dart:convert';
import 'package:flutter_charset_detector/flutter_charset_detector.dart';

typedef ProgressCallback = void Function(double progress);
typedef FileProcessor = Future<void> Function(File file, String content);

abstract class BaseSearchService {
  /// 处理文件遍历和进度回调（核心复用逻辑）
  static Future<void> processFiles({
    required String directory,
    required String fileType,
    required FileProcessor onFile,
    ProgressCallback? onProgress,
    bool Function()? shouldStop,
  }) async {
    final dir = Directory(directory);
    final totalFiles = await countFilesInDirectory(dir, fileType, shouldStop);
    if (totalFiles == 0) return;

    onProgress?.call(1);

    int processedFiles = 0;
    int oldProgress = 1;

    await for (final entity in dir.list(recursive: true)) {
      if (shouldStop?.call() == true) return;
      if (entity is! File || !FileUtils.matchesFileType(entity.path, fileType)) continue;

      // 1. 过滤二进制文件（通过文件头或扩展名）
      if (fileType != 'docx' && await _isBinaryFile(entity)) continue;

      processedFiles++;
      final progress = (processedFiles / totalFiles) * 99 + 1;
      if (progress - oldProgress >= 1) {
        oldProgress = progress.floor();
        onProgress?.call(progress);
      }

      try {
        // 2. 自动识别文本编码
        final content = await _readFileWithEncoding(entity);
        await onFile(entity, content);
      } catch (e) {
        // 3. 尝试解析docx等文档
        if (fileType == 'docx') {
          final docxContent = await _extractTextFromDocx(entity);
          if (docxContent != null) {
            await onFile(entity, docxContent);
          }
        }
        continue; // 跳过无法解析的文件
      }
    }

    onProgress?.call(100);
  }

  static Future<String> _readFileWithEncoding(File file) async {
    try {
      final bytes = await file.readAsBytes();

      // 检测文件编码（返回编码名称，如 'UTF-8'）
      DecodingResult result = await CharsetDetector.autoDecode(bytes);
      final encodingName = result.charset.toLowerCase();

      // 转换为 Dart 的 Encoding 对象
      final encoding = _getEncodingFromName(encodingName);
      return encoding.decode(bytes);
    } catch (e) {
      // 回退到UTF-8解码（兼容性处理）
      return utf8.decode(await file.readAsBytes(), allowMalformed: true);
    }
  }

  /// 将编码名称映射为 Dart 的 Encoding 对象
  static Encoding _getEncodingFromName(String name) {
    return Encoding.getByName(name) ?? latin1;
  }

  static Future<String?> _extractTextFromDocx(File file) async {
    try {
      final bytes = await file.readAsBytes();
      return docxToText(bytes);
    } catch (e) {
      return null;
    }
  }

  static Future<bool> _isBinaryFile(File file, {int sampleSize = 512}) async {
    if (isBinaryFileByExtension(file.path)) return true;

    final bytes = await file.openRead().take(sampleSize).expand((chunk) => chunk).toList();

    if (await isBinaryFileByHeader(bytes)) return true;

    // 再检查内容
    return await isLikelyBinary(bytes);
  }

  static bool isBinaryFileByExtension(String filePath) {
    final binaryExtensions = [
      '.png', '.jpg', '.jpeg', '.gif', '.bmp', // 图片
      '.mp4', '.avi', '.mov', '.mkv', // 视频
      '.zip', '.rar', '.7z', '.tar', '.gz', // 压缩包
      '.exe', '.dll', '.so', '.a', // 可执行文件
    ];
    return binaryExtensions.any((ext) => filePath.toLowerCase().endsWith(ext));
  }

  static Future<bool> isBinaryFileByHeader(List<int> bytes) async {
    // 常见二进制文件头
    final magicNumbers = {
      'PNG': [0x89, 0x50, 0x4E, 0x47],
      'JPEG': [0xFF, 0xD8, 0xFF],
      'GIF': [0x47, 0x49, 0x46],
      'ZIP': [0x50, 0x4B, 0x03, 0x04],
      'MP4': [0x00, 0x00, 0x00, 0x20, 0x66, 0x74, 0x79, 0x70],
    };

    return magicNumbers.values.any(
      (header) =>
          bytes.length >= header.length &&
          const ListEquality().equals(bytes.sublist(0, header.length), header),
    );
  }

  static Future<bool> isLikelyBinary(List<int> bytes) async {
    int controlChars = 0;
    for (final byte in bytes) {
      // 统计不可打印ASCII控制字符（排除换行/制表符）
      if (byte < 32 && byte != 9 && byte != 10 && byte != 13) {
        controlChars++;
        // 发现少量控制字符即可判定为二进制
        if (controlChars > 5) return true; // 阈值可根据需求调整
      }
    }
    return false;
  }

  /// 以下为静态工具方法（被所有服务共享）
  static List<String> splitQuery(String query) {
    return query
        .split('\n') // 先按换行符拆分
        .expand((line) => line.split(',').map((q) => q.trim())) // 再按逗号拆分并去空格
        .where((q) => q.isNotEmpty) // 过滤空字符串
        .toList();
  }

  static RegExp buildSearchPattern({
    required String query,
    required bool caseSensitive,
    required bool wholeWord,
    required bool useRegex,
  }) {
    String pattern;
    if (useRegex) {
      pattern = query;
    } else {
      pattern = RegExp.escape(query);
      if (wholeWord) {
        pattern = '\\b$pattern\\b';
      }
    }
    return RegExp(pattern, caseSensitive: caseSensitive, multiLine: true);
  }

  static Future<int> countFilesInDirectory(
    Directory dir,
    String fileType,
    bool Function()? shouldStop,
  ) async {
    int totalFiles = 0;
    await for (final entity in dir.list(recursive: true)) {
      if (shouldStop?.call() == true) return totalFiles;
      if (entity is File && FileUtils.matchesFileType(entity.path, fileType)) {
        totalFiles++;
      }
    }
    return totalFiles;
  }
}
