import 'dart:async';
import 'dart:io';
import 'dart:isolate';
import 'dart:math';
import 'package:path/path.dart' as path;
import 'package:win_text_editor/modules/content_search/services/base_search_service.dart';
import 'package:win_text_editor/shared/utils/file_utils.dart';

class CountSearchService {
  static const _maxConcurrentIsolates = 8; // 根据CPU核心数调整
  static const _queriesPerBatch = 500; // 每批搜索词数量

  static List<List<T>> _chunkList<T>(List<T> list, int chunkSize) {
    return List.generate(
      (list.length / chunkSize).ceil(),
      (i) => list.sublist(
        i * chunkSize,
        i * chunkSize + chunkSize > list.length ? list.length : i * chunkSize + chunkSize,
      ),
    );
  }

  static Future<Map<String, int>> performSearch({
    required String directory,
    required String query,
    required String fileType,
    required String jumpFiles,
    required bool caseSensitive,
    required bool wholeWord,
    required bool useRegex,
    ProgressCallback? onProgress,
    bool Function()? shouldStop,
  }) async {
    if (shouldStop?.call() == true) return {};

    final counts = <String, int>{};
    final allQueries = BaseSearchService.splitQuery(query);

    // 分割搜索词为批次
    final queryBatches = _chunkList(allQueries, _queriesPerBatch);

    // 获取所有目标文件路径
    final filePaths = await _collectFilePaths(directory, fileType, jumpFiles, shouldStop);
    if (filePaths.isEmpty) return counts;

    // 启动Isolate池
    final resultPort = ReceivePort();
    final stopPort = ReceivePort();
    final completer = Completer<Map<String, int>>();
    int activeIsolates = 0;

    // 进度处理
    int processedFiles = 0;
    void updateProgress() {
      final progress = (processedFiles / filePaths.length) * 100;
      onProgress?.call(progress);
    }

    final initIsolates = min(2, _maxConcurrentIsolates); // 首批启动2个
    for (int i = 0; i < initIsolates; i++) {
      _spawnIsolateWorker(
        filePaths.sublist(i, i + 1), // 每个Isolate处理部分文件
        queryBatches,
        resultPort.sendPort,
        stopPort.sendPort,
        caseSensitive,
        wholeWord,
        useRegex,
      );
      activeIsolates++;
    }

    // 分发任务到Isolate
    Future.delayed(const Duration(milliseconds: 300), () {
      for (int i = activeIsolates; i < min(_maxConcurrentIsolates, filePaths.length); i++) {
        _spawnIsolateWorker(
          filePaths.sublist(i, i + 1), // 每个Isolate处理部分文件
          queryBatches,
          resultPort.sendPort,
          stopPort.sendPort,
          caseSensitive,
          wholeWord,
          useRegex,
        );
        activeIsolates++;
      }
    });

    // 监听结果
    resultPort.listen((msg) {
      if (msg is Map<String, int>) {
        msg.forEach((query, count) {
          counts.update(query, (v) => v + count, ifAbsent: () => count);
        });
        processedFiles++;
        updateProgress();

        // 继续分配新任务
        if (activeIsolates < filePaths.length) {
          _spawnIsolateWorker(
            [filePaths[activeIsolates]],
            queryBatches,
            resultPort.sendPort,
            stopPort.sendPort,
            caseSensitive,
            wholeWord,
            useRegex,
          );
          activeIsolates++;
        } else if (processedFiles == filePaths.length) {
          completer.complete(counts);
        }
      }
    });

    // 处理停止请求
    stopPort.listen((_) => completer.complete(counts));

    return completer.future.whenComplete(() {
      resultPort.close();
      stopPort.close();
    });
  }

  static Future<List<String>> _collectFilePaths(
    String directory,
    String fileType,
    String jumpFiles,
    bool Function()? shouldStop,
  ) async {
    final dir = Directory(directory);
    final paths = <String>[];
    final jumpFileList = BaseSearchService.splitQuery(jumpFiles.toLowerCase());
    await for (final entity in dir.list(recursive: true)) {
      if (shouldStop?.call() == true) break;
      if (entity is File &&
          !jumpFileList.contains(path.basename(entity.path).toLowerCase()) &&
          FileUtils.matchesFileType(entity.path, fileType)) {
        paths.add(entity.path);
      }
    }
    return paths;
  }

  static void _spawnIsolateWorker(
    List<String> filePaths,
    List<List<String>> queryBatches,
    SendPort resultPort,
    SendPort stopPort,
    bool caseSensitive,
    bool wholeWord,
    bool useRegex,
  ) async {
    await Isolate.spawn(
      _isolateEntry,
      _IsolateData(
        filePaths,
        queryBatches,
        resultPort,
        stopPort,
        caseSensitive,
        wholeWord,
        useRegex,
      ),
    );
  }

  static void _isolateEntry(_IsolateData data) {
    final localCounts = <String, int>{};
    for (final filePath in data.filePaths) {
      try {
        final content = File(filePath).readAsStringSync();
        for (final batch in data.queryBatches) {
          // 并行处理每个搜索词批次
          final batchResults = ParallelTask.runSync(() {
            final batchCounts = <String, int>{};
            for (final query in batch) {
              final pattern = BaseSearchService.buildSearchPattern(
                query: query,
                caseSensitive: data.caseSensitive,
                wholeWord: data.wholeWord,
                useRegex: data.useRegex,
              );
              batchCounts[query] = pattern.allMatches(content).length;
            }
            return batchCounts;
          });

          batchResults.forEach((query, count) {
            localCounts.update(query, (v) => v + count, ifAbsent: () => count);
          });
        }
      } catch (e) {
        print('Isolate error: $e');
      }
    }
    data.resultPort.send(localCounts);
  }
}

// Isolate通信数据结构
class _IsolateData {
  final List<String> filePaths;
  final List<List<String>> queryBatches;
  final SendPort resultPort;
  final SendPort stopPort;
  final bool caseSensitive;
  final bool wholeWord;
  final bool useRegex;

  _IsolateData(
    this.filePaths,
    this.queryBatches,
    this.resultPort,
    this.stopPort,
    this.caseSensitive,
    this.wholeWord,
    this.useRegex,
  );
}

// 模拟并行任务处理
class ParallelTask {
  static T runSync<T>(T Function() task) => task();
}
