// lib/app/modules/content_search/content_search_service.dart

import 'dart:convert';
import 'dart:io';
import 'package:flutter_js/flutter_js.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/modules/content_search/models/search_mode.dart';
import 'package:win_text_editor/modules/content_search/models/search_result.dart';
import 'package:win_text_editor/shared/utils/file_utils.dart';

typedef ProgressCallback = void Function(double progress);

class CustomSearchService {
  /// 新增方法：执行自定义 JavaScript 规则搜索
  static Future<List<SearchResult>> performCustomSearch({
    required String directory,
    required String fileType,
    required String jsFunction,
    required SearchMode searchMode,
    ProgressCallback? onProgress,
    bool Function()? shouldStop,
  }) async {
    final results = <SearchResult>[];
    int count = 0;
    final dir = Directory(directory);
    final jsRuntime = getJavascriptRuntime();

    try {
      // 定义 JavaScript 函数
      final jsCode = 'function match(content){$jsFunction};';

      jsRuntime.evaluate(jsCode);

      int totalFiles = 0;
      await for (final entity in dir.list(recursive: true)) {
        if (shouldStop?.call() == true) return results;
        if (entity is File && FileUtils.matchesFileType(entity.path, fileType)) {
          totalFiles++;
        }
      }

      onProgress?.call(1);

      int processedFiles = 0;
      int calledProcessedFiles = 1;

      await for (final entity in dir.list(recursive: true)) {
        if (shouldStop?.call() == true) return results;
        if (entity is File && FileUtils.matchesFileType(entity.path, fileType)) {
          try {
            final lines = await entity.readAsLines();
            for (int i = 0; i < lines.length; i++) {
              if (shouldStop?.call() == true) return results;
              final line = lines[i].trim();
              if (line.length < 3) continue; // 跳过短行

              final result = jsRuntime.evaluate('match(${jsonEncode(line)});');
              if (result.isError) {
                throw Exception('JS Error: ${result.stringResult}');
              }

              if (result.stringResult == 'true') {
                if (searchMode == SearchMode.locate) {
                  results.add(
                    SearchResult(
                      filePath: entity.path,
                      lineNumber: i + 1,
                      lineContent: line,
                      matches: [],
                      queryTerm: "Custom Rule",
                    ),
                  );
                } else {
                  count++;
                }
              }
            }
          } catch (e) {
            Logger().error('Error in file ${entity.path}: $e');
          }

          processedFiles++;
          final progress = (processedFiles / totalFiles) * 99 + 1;
          if (processedFiles - calledProcessedFiles >= 1) {
            calledProcessedFiles = processedFiles;
            onProgress?.call(progress);
          }
        }
      }

      // 处理计数模式结果
      if (searchMode == SearchMode.count) {
        results.add(
          SearchResult(
            filePath: "Custom Rule",
            lineNumber: count,
            lineContent: '',
            matches: [],
            queryTerm: "Custom Rule",
          ),
        );
      }
    } finally {
      onProgress?.call(100);
      jsRuntime.dispose();
    }

    return results;
  }

  /// 验证 JavaScript 自定义规则的语法
  static JsEvalResult validateJsRule(String jsFunction) {
    final jsRuntime = getJavascriptRuntime();
    try {
      final jsCode = 'function match(content) {$jsFunction};';
      // 执行 JS 代码进行语法检查
      final result = jsRuntime.evaluate(jsCode);
      if (result.isError) {
        return result;
      }

      // 测试函数是否返回布尔值
      final test = jsRuntime.evaluate("match('test');");
      if (test.stringResult != 'true' && test.stringResult != 'false') {
        return JsEvalResult('Custom rule must return boolean (true/false)', null, isError: true);
      }

      return test;
    } finally {
      jsRuntime.dispose();
    }
  }
}
