// lib/app/modules/content_search/services/locate_search_service.dart
import 'dart:io';
import 'package:win_text_editor/modules/content_search/models/match_result.dart';

import '../models/search_result.dart';
import 'base_search_service.dart';

class LocateSearchService {
  static Future<List<SearchResult>> performSearch({
    required String directory,
    required String query,
    required String fileType,
    required bool caseSensitive,
    required bool wholeWord,
    required bool useRegex,
    ProgressCallback? onProgress,
    bool Function()? shouldStop,
  }) async {
    final results = <SearchResult>[];
    final queries = BaseSearchService.splitQuery(query);

    await BaseSearchService.processFiles(
      directory: directory,
      fileType: fileType,
      onProgress: onProgress,
      shouldStop: shouldStop,
      onFile: (file, content) async {
        for (final q in queries) {
          final pattern = BaseSearchService.buildSearchPattern(
            query: q,
            caseSensitive: caseSensitive,
            wholeWord: wholeWord,
            useRegex: useRegex,
          );

          final matches = pattern.allMatches(content);
          if (matches.isNotEmpty) {
            results.addAll(_convertMatchesToResults(file, content, matches, q));
          }
        }
      },
    );

    return results;
  }

  /// 将匹配结果转换为 SearchResult 列表（按行拆分）
  static List<SearchResult> _convertMatchesToResults(
    File file,
    String content,
    Iterable<RegExpMatch> matches,
    String queryTerm,
  ) {
    final results = <SearchResult>[];
    final lines = content.split('\n');

    // 创建一个映射表：行号 -> 该行的所有匹配项
    final lineMatches = <int, List<RegExpMatch>>{};
    for (final match in matches) {
      // 计算匹配所在的行号
      final lineNumber = _getLineNumber(lines, match.start);
      lineMatches.putIfAbsent(lineNumber, () => []).add(match);
    }

    // 为每行匹配生成结果
    lineMatches.forEach((lineNumber, matchesInLine) {
      final lineContent = lines[lineNumber - 1]; // 行号从1开始
      final lineResults =
          matchesInLine.map((m) {
            // 计算匹配在行内的相对位置
            final lineStartPos = _getLineStartPos(lines, lineNumber - 1);
            final startInLine = m.start - lineStartPos;
            final endInLine = m.end - lineStartPos;

            return MatchResult(start: startInLine, end: endInLine);
          }).toList();

      results.add(
        SearchResult(
          filePath: file.path,
          lineNumber: lineNumber,
          lineContent: lineContent,
          matches: lineResults,
          queryTerm: queryTerm,
        ),
      );
    });

    return results;
  }

  /// 辅助方法：根据字符位置计算行号
  static int _getLineNumber(List<String> lines, int charPos) {
    int currentPos = 0;
    for (int i = 0; i < lines.length; i++) {
      currentPos += lines[i].length + 1; // +1 换行符
      if (charPos < currentPos) {
        return i + 1; // 行号从1开始
      }
    }
    return lines.length; // 默认返回最后一行
  }

  /// 辅助方法：获取指定行的起始字符位置
  static int _getLineStartPos(List<String> lines, int lineIndex) {
    int pos = 0;
    for (int i = 0; i < lineIndex; i++) {
      pos += lines[i].length + 1; // +1 换行符
    }
    return pos;
  }
}
