import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:provider/provider.dart';
import 'package:syncfusion_flutter_datagrid/datagrid.dart';
import 'package:path/path.dart' as path;
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/modules/content_search/controllers/content_search_controller.dart';
import 'package:file_picker/file_picker.dart';
import 'package:win_text_editor/modules/content_search/models/count_result.dart';
import 'dart:io';

import 'package:win_text_editor/modules/content_search/models/search_mode.dart';
import 'package:win_text_editor/modules/content_search/models/search_result.dart';
import 'package:win_text_editor/shared/base/my_sf_data_grid.dart';
import 'package:win_text_editor/shared/base/my_sf_data_source.dart';
import 'package:win_text_editor/shared/components/my_grid_column.dart';
// import 'package:recycle_bin/recycle_bin.dart';

class ResultsView extends StatelessWidget {
  const ResultsView({super.key});

  @override
  Widget build(BuildContext context) {
    final controller = context.watch<ContentSearchController>();

    // 监听搜索模式的变化，当搜索模式改变时，清空结果列表
    final previousSearchMode = context.select<ContentSearchController, SearchMode>(
      (c) => c.searchMode,
    );
    if (previousSearchMode != controller.searchMode) {
      controller.clearResults();
    }

    return Card(
      child:
          controller.searchMode == SearchMode.locate
              ? _buildLocateGrid(controller, context)
              : _buildCountGrid(controller, context),
    );
  }

  Widget _buildLocateGrid(ContentSearchController controller, BuildContext context) {
    return MySfDataGrid<SearchResult>(
      source: LocateDataSource(controller, context),
      columns: [
        ShortGridColumn(columnName: 'index', label: '序号'),
        MyGridColumn(columnName: 'file', label: '文件(行号)', maximumWidth: 400),
        MyGridColumn(columnName: 'content', label: '内容'),
        ShortGridColumn(columnName: 'action', label: '操作', width: 90),
      ],
      selectable: false,
    );
  }

  Widget _buildCountGrid(ContentSearchController controller, BuildContext context) {
    return MySfDataGrid<CountResult>(
      source: CountDataSource(controller, context),
      columns: [
        ShortGridColumn(columnName: 'index', label: '序号'),
        MyGridColumn(columnName: 'keyword', label: '关键词', minimumWidth: 300),
        MyGridColumn(columnName: 'matchCount', label: '匹配数量'),
      ],
      selectable: false,
    );
  }
}

class LocateDataSource extends MySfDataSource<SearchResult> {
  final ContentSearchController controller;
  final BuildContext context;

  LocateDataSource(this.controller, this.context)
    : super(controller.results, onSelectionChanged: (index, isSelected) {});

  @override
  List<DataGridRow> get rows =>
      controller.results.asMap().entries.map((result) {
        final index = result.key + 1;
        return DataGridRow(
          cells: [
            DataGridCell(columnName: 'index', value: index),
            DataGridCell(
              columnName: 'file',
              value: '${path.basename(result.value.filePath)}(${result.value.lineNumber})',
            ),
            DataGridCell(
              columnName: 'content',
              value: result.value, // 传递整个结果对象用于高亮显示
            ),
            DataGridCell(
              columnName: 'action',
              value: result.value.filePath, // Store file path for delete action
            ),
          ],
        );
      }).toList();

  @override
  DataGridRowAdapter buildRow(DataGridRow row) {
    final cells = row.getCells();
    final result = cells[2].value as SearchResult;

    return DataGridRowAdapter(
      cells: [
        Container(
          alignment: Alignment.center,
          padding: const EdgeInsets.symmetric(horizontal: 8),
          child: Text(cells[0].value.toString()),
        ),
        // 文件名单元格
        Container(
          alignment: Alignment.centerLeft,
          padding: const EdgeInsets.symmetric(horizontal: 8),
          child: Text(cells[1].value.toString(), overflow: TextOverflow.ellipsis, maxLines: 1),
        ),
        // 内容单元格（带高亮）
        Container(
          alignment: Alignment.centerLeft,
          padding: const EdgeInsets.symmetric(horizontal: 8),
          child: SingleChildScrollView(
            scrollDirection: Axis.horizontal,
            child: _buildHighlightedText(result.lineContent, result.matches),
          ),
        ),
        Container(
          alignment: Alignment.center,
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              IconButton(
                icon: const Icon(Icons.open_in_new, size: 18, color: Colors.blue),
                onPressed: () => _openFile(result.filePath),
              ),
              IconButton(
                icon: const Icon(Icons.delete_forever, size: 18, color: Colors.red),
                onPressed: () => _showDeleteConfirmation(result.filePath),
              ),
            ],
          ),
        ),
      ],
    );
  }

  Future<void> _openFile(String filePath) async {
    try {
      final file = File(filePath);
      if (await file.exists()) {
        // Use Process.run to open the file with the default system handler
        if (Platform.isWindows) {
          await Process.run('start', ['""', filePath], runInShell: true);
        } else if (Platform.isMacOS) {
          await Process.run('open', [filePath], runInShell: true);
        } else if (Platform.isLinux) {
          await Process.run('xdg-open', [filePath], runInShell: true);
        }
      } else {
        if (context.mounted) {
          ScaffoldMessenger.of(context).showSnackBar(SnackBar(content: Text('文件不存在: $filePath')));
        }
      }
    } catch (e) {
      if (context.mounted) {
        ScaffoldMessenger.of(
          context,
        ).showSnackBar(SnackBar(content: Text('打开文件失败: ${e.toString()}')));
      }
    }
  }

  Future<void> _showDeleteConfirmation(String filePath) async {
    bool confirmed = false;

    await showDialog<bool>(
      context: context,
      builder: (context) {
        // 捕获键盘事件
        return Shortcuts(
          shortcuts: const {
            // 绑定回车键
            SingleActivator(LogicalKeyboardKey.enter): _ConfirmAction(),
          },
          child: Actions(
            actions: {
              _ConfirmAction: CallbackAction<_ConfirmAction>(
                onInvoke: (_) {
                  confirmed = true;
                  Navigator.pop(context, true);
                  return null;
                },
              ),
            },
            child: Focus(
              autofocus: true, // 自动获取焦点
              child: AlertDialog(
                title: const Text('确认删除'),
                content: Column(
                  mainAxisSize: MainAxisSize.min,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    const Text('将所选文件删除吗？'),
                    const SizedBox(height: 8),
                    Text(filePath, style: const TextStyle(fontSize: 12, color: Colors.grey)),
                  ],
                ),
                actions: [
                  TextButton(
                    onPressed: () => Navigator.pop(context, false),
                    child: const Text('取消'),
                  ),
                  TextButton(
                    onPressed: () {
                      confirmed = true;
                      Navigator.pop(context, true);
                    },
                    child: const Text('确认'),
                  ),
                ],
              ),
            ),
          ),
        );
      },
    );

    if (confirmed && context.mounted) {
      try {
        await File(filePath).delete();
        controller.removeResultByFilePath(filePath);
        notifyListeners();

        if (context.mounted) {
          Logger().info('已删除文件: $filePath');
        }
      } catch (e) {
        if (context.mounted) {
          Logger().error('删除失败: ${e.toString()}');
        }
      }
    }
  }

  Widget _buildHighlightedText(String text, matches) {
    final spans = <TextSpan>[];
    int lastEnd = 0;

    for (final match in matches) {
      if (match.start > lastEnd) {
        spans.add(
          TextSpan(
            text: text.substring(lastEnd, match.start),
            style: const TextStyle(color: Colors.black),
          ),
        );
      }
      spans.add(
        TextSpan(
          text: text.substring(match.start, match.end),
          style: const TextStyle(color: Colors.blue, fontWeight: FontWeight.bold),
        ),
      );
      lastEnd = match.end;
    }
    if (lastEnd < text.length) {
      spans.add(
        TextSpan(text: text.substring(lastEnd), style: const TextStyle(color: Colors.black)),
      );
    }

    return RichText(text: TextSpan(children: spans), overflow: TextOverflow.clip);
  }
}

class CountDataSource extends MySfDataSource<CountResult> {
  final ContentSearchController controller;
  final BuildContext context;
  late final List<MapEntry<String, int>> _counts;

  CountDataSource(this.controller, this.context)
    : super([], onSelectionChanged: (index, isSelected) {}) {
    final counts = <String, int>{};
    for (var result in controller.results) {
      counts[result.filePath] = (counts[result.filePath] ?? 0) + result.lineNumber;
    }
    _counts = counts.entries.toList();
  }

  @override
  List<DataGridRow> get rows =>
      _counts.asMap().entries.map((entry) {
        final index = entry.key + 1;
        return DataGridRow(
          cells: [
            DataGridCell(columnName: 'index', value: index),
            DataGridCell(columnName: 'keyword', value: entry.value.key),
            DataGridCell(columnName: 'count', value: '${entry.value.value}'),
          ],
        );
      }).toList();

  @override
  DataGridRowAdapter buildRow(DataGridRow row) {
    return DataGridRowAdapter(
      cells:
          row.getCells().map((cell) {
            return Container(
              alignment: cell.columnName == 'count' ? Alignment.centerRight : Alignment.centerLeft,
              padding: const EdgeInsets.symmetric(horizontal: 8),
              child: Text(cell.value.toString(), overflow: TextOverflow.ellipsis, maxLines: 1),
            );
          }).toList(),
    );
  }
}

class _ConfirmAction extends Intent {
  const _ConfirmAction();
}
