import 'dart:async';

import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:win_text_editor/shared/components/my_checkbox.dart';
import 'package:win_text_editor/shared/components/text_editor.dart';
import 'package:win_text_editor/modules/content_search/controllers/content_search_controller.dart';
import 'package:win_text_editor/modules/content_search/models/search_mode.dart';

class SearchSettings extends StatefulWidget {
  const SearchSettings({super.key});

  @override
  State<SearchSettings> createState() => _SearchSettingsState();
}

class _SearchSettingsState extends State<SearchSettings> {
  Duration _elapsedTime = Duration.zero;
  final Stopwatch _stopwatch = Stopwatch();
  Timer? _updateTimer;

  @override
  void dispose() {
    _updateTimer?.cancel();
    _stopwatch.stop();
    super.dispose();
  }

  void _startTimer() {
    _elapsedTime = Duration.zero;
    _stopwatch
      ..reset()
      ..start();

    // 每100毫秒更新一次UI，保持流畅显示
    _updateTimer = Timer.periodic(const Duration(milliseconds: 100), (timer) {
      setState(() {
        _elapsedTime = _stopwatch.elapsed;
      });
    });
  }

  void _stopTimer() {
    _updateTimer?.cancel();
    _updateTimer = null;
    _stopwatch.stop();
    // 确保获取最终准确时间
    _elapsedTime = _stopwatch.elapsed;
  }

  String _formatDuration(Duration duration) {
    String twoDigits(int n) => n.toString().padLeft(2, '0');
    String twoDigitMinutes = twoDigits(duration.inMinutes.remainder(60));
    String twoDigitSeconds = twoDigits(duration.inSeconds.remainder(60));
    String twoDigitMillis = twoDigits(duration.inMilliseconds.remainder(1000) ~/ 10);

    return "${twoDigits(duration.inHours)}:$twoDigitMinutes:$twoDigitSeconds.$twoDigitMillis";
  }

  @override
  Widget build(BuildContext context) {
    final controller = context.watch<ContentSearchController>();

    // 监听搜索状态变化来控制计时器
    if (controller.isSearching && _updateTimer == null) {
      _startTimer();
    } else if (!controller.isSearching && _updateTimer != null) {
      _stopTimer();
    }

    return Card(
      child: Padding(
        padding: const EdgeInsets.all(8.0),
        child: Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            // 搜索内容框 (保持原样)
            Expanded(
              child: SizedBox(
                height: 360,
                child: TextEditor(
                  tabId: 'search_content_${controller.hashCode}',
                  title: '搜索内容[列表以换行或半角逗号分隔]',
                  initialContent: controller.searchQuery, // 绑定到控制器的状态
                  onContentChanged: (content) {
                    controller.searchQuery = content; // 实时同步内容到控制器
                  },
                ),
              ),
            ),
            const SizedBox(width: 8),
            // 设置按钮区域
            SizedBox(
              width: 310,
              child: Container(
                height: 360,
                decoration: BoxDecoration(
                  border: Border.all(color: Colors.grey),
                  borderRadius: BorderRadius.circular(4),
                ),
                padding: const EdgeInsets.all(8.0),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    // 搜索方式 - 两选项排在一行
                    const Text('搜索方式:', style: TextStyle(fontSize: 12)),
                    const SizedBox(height: 4),
                    Row(
                      children: [
                        Expanded(
                          child: Row(
                            children: [
                              Transform.scale(
                                scale: 0.75,
                                child: Radio<SearchMode>(
                                  value: SearchMode.locate,
                                  groupValue: controller.searchMode,
                                  onChanged: (value) {
                                    if (value != null) {
                                      controller.clearResults(); // 清除搜索结果
                                      controller.searchMode = value;
                                    }
                                  },
                                ),
                              ),
                              const Text('定位', style: TextStyle(fontSize: 12)),
                            ],
                          ),
                        ),
                        Expanded(
                          child: Row(
                            children: [
                              Transform.scale(
                                scale: 0.75,
                                child: Radio<SearchMode>(
                                  value: SearchMode.count,
                                  groupValue: controller.searchMode,
                                  onChanged: (value) {
                                    if (value != null) {
                                      controller.clearResults(); // 清除搜索结果
                                      controller.searchMode = value;
                                    }
                                  },
                                ),
                              ),
                              const Text('计数', style: TextStyle(fontSize: 12)),
                            ],
                          ),
                        ),
                      ],
                    ),
                    Visibility(
                      visible: controller.searchMode == SearchMode.count,
                      child: Column(
                        children: [
                          MyCheckbox(
                            title: '显示未匹配列表',
                            value: controller.onlyZeroCounts,
                            onChanged:
                                controller.searchMode == SearchMode.count
                                    ? (value) {
                                      controller.onlyZeroCounts = value!;
                                    }
                                    : null,
                          ),
                          const SizedBox(height: 4),
                        ],
                      ),
                    ),
                    const SizedBox(height: 4),
                    // 匹配规则
                    const Text('匹配规则:', style: TextStyle(fontSize: 12)),
                    const SizedBox(height: 4),
                    // 第一行复选框
                    Row(
                      children: [
                        Expanded(
                          child: MyCheckbox(
                            title: '大小写敏感',
                            value: controller.caseSensitive,
                            onChanged: (value) {
                              controller.caseSensitive = value!;
                              if (value) {
                                controller.customRule = false;
                                controller.useRegex = false;
                              }
                            },
                          ),
                        ),
                        Expanded(
                          child: MyCheckbox(
                            title: '全字匹配',
                            value: controller.wholeWord,
                            onChanged: (value) {
                              controller.wholeWord = value!;
                              if (value) {
                                controller.customRule = false;
                                controller.useRegex = false;
                              }
                            },
                          ),
                        ),
                      ],
                    ),
                    Row(
                      children: [
                        Expanded(
                          child: MyCheckbox(
                            title: '正则匹配',
                            value: controller.useRegex,
                            onChanged: (value) {
                              controller.useRegex = value!;
                              if (value) {
                                controller.caseSensitive = false;
                                controller.wholeWord = false;
                                controller.customRule = false;
                              }
                            },
                          ),
                        ),
                        Expanded(
                          child: MyCheckbox(
                            title: '自定义规则',
                            value: controller.customRule,
                            onChanged: (value) {
                              controller.customRule = value!;
                              if (value) {
                                controller.caseSensitive = false;
                                controller.wholeWord = false;
                                controller.useRegex = false;
                              }
                            },
                          ),
                        ),
                      ],
                    ),
                    const SizedBox(height: 8),
                    // 仅在自定义规则选中时显示的JS函数说明
                    Visibility(
                      visible: controller.customRule,
                      child: const Column(
                        children: [
                          Text.rich(
                            TextSpan(
                              text:
                                  "在搜索内容输入框中输入js函数体:\n"
                                  "function boolean match(content) {//content为单行文本\n"
                                  "}//返回是否匹配",
                              style: TextStyle(fontSize: 12, fontFamily: 'monospace'),
                            ),
                          ),
                          SizedBox(height: 8),
                        ],
                      ),
                    ),
                    // 开始搜索按钮
                    Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        Expanded(
                          child: ElevatedButton.icon(
                            icon: const Icon(Icons.search, size: 20),
                            label: const Text('开始搜索'),
                            onPressed:
                                controller.isSearching ? null : () => controller.startSearch(),
                          ),
                        ),
                        const SizedBox(width: 8),
                        Expanded(
                          child: ElevatedButton.icon(
                            icon: const Icon(Icons.stop, color: Colors.red),
                            label: const Text('停止', style: TextStyle(color: Colors.red)),
                            onPressed:
                                controller.isSearching
                                    ? () {
                                      controller.stopSearch();
                                      _stopTimer();
                                    }
                                    : null,
                          ),
                        ),
                      ],
                    ),
                    const SizedBox(height: 8),
                    Text(
                      ' 搜索计时：[${_formatDuration(_elapsedTime)}]',
                      style: const TextStyle(fontSize: 12),
                    ),
                  ],
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }
}
