import 'dart:io';

import 'package:file_picker/file_picker.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/modules/outline/models/outline_node.dart';
import 'package:win_text_editor/shared/base/base_content_controller.dart';

class DataCompareController extends BaseContentController {
  List<String> leftColumns = [];
  List<String> rightColumns = [];
  List<Map<String, dynamic>> leftData = [];
  List<Map<String, dynamic>> rightData = [];

  // 比对结果状态
  List<Map<String, dynamic>> comparedData = [];

  Future<void> importCsv(bool isLeftTable, String csvContent) async {
    try {
      // 1. 确定分隔符
      final lines = csvContent.split('\n');
      if (lines.isEmpty) return;

      String delimiter = lines[0].contains('\t') ? '\t' : ',';

      // 2. 解析列标题（从第一行第二列开始）
      final headers = lines[0].split(delimiter).skip(1).toList();

      // 3. 解析数据
      List<Map<String, dynamic>> data = [];
      for (int i = 1; i < lines.length; i++) {
        if (lines[i].trim().isEmpty) continue;

        final values = lines[i].split(delimiter);
        if (values.isEmpty) continue;

        // 创建行数据：序号、主键和各列值
        final row = {
          'serial': i.toString(),
          'key': values[0],
          ...Map.fromIterables(
            headers,
            List.generate(headers.length, (j) => j + 1 < values.length ? values[j + 1] : ''),
          ),
        };
        data.add(row);
      }

      if (isLeftTable) {
        leftColumns = headers;
        leftData = data;
      } else {
        rightColumns = headers;
        rightData = data;
      }

      _compareData();
      notifyListeners();
    } catch (e) {
      Logger().error('${isLeftTable ? '左表' : '右表'}导入失败: $e');
      rethrow;
    }
  }

  void _compareData() {
    comparedData = [];

    // 1. 处理空数据情况
    if (leftData.isEmpty && rightData.isEmpty) return;

    // 单边数据直接全部标记为不匹配
    if (leftData.isEmpty || rightData.isEmpty) {
      final source = leftData.isEmpty ? rightData : leftData;
      comparedData =
          source
              .map((row) => {...row, 'match_status': 'no_match', 'is_left': leftData.isNotEmpty})
              .toList();
      return;
    }

    // 3. 双边数据比对
    final rightMap = {for (var r in rightData) r['key']: r};

    // 先处理匹配的行
    for (var leftRow in leftData) {
      final rightRow = rightMap[leftRow['key']];

      if (rightRow != null) {
        bool isFullMatch = leftColumns.every((col) => leftRow[col] == rightRow[col]);

        comparedData.add({
          ...leftRow,
          'match_status': isFullMatch ? 'full_match' : 'key_match',
          'is_left': true,
          'right_data': rightRow,
        });
        rightMap.remove(leftRow['key']);
      } else {
        comparedData.add({...leftRow, 'match_status': 'no_match', 'is_left': true});
      }
    }

    // 添加仅右表有的数据
    comparedData.addAll(
      rightMap.values.map((row) => ({...row, 'match_status': 'no_match', 'is_left': false})),
    );

    // 4. 排序：匹配的在前
    comparedData.sort((a, b) {
      const order = {'full_match': 0, 'key_match': 1, 'no_match': 2};
      return order[a['match_status']]!.compareTo(order[b['match_status']]!);
    });
  }

  void exportLeftTable(String matchType) async {
    try {
      final filteredData =
          comparedData.where((row) {
            if (!row['is_left']) return false;
            return row['match_status'] == matchType;
          }).toList();

      if (filteredData.isEmpty) {
        Logger().info('没有匹配类型的左表数据可导出');
        return;
      }

      // 生成CSV内容
      final csvContent = _generateCsvContent(filteredData, leftColumns);
      final fileName = 'left_table_${_getMatchTypeName(matchType)}.csv';
      await _saveCsvFile(fileName, csvContent);

      Logger().info('左表(${_getMatchTypeName(matchType)})导出成功');
    } catch (e) {
      Logger().error('左表导出失败: $e');
      rethrow;
    }
  }

  void exportRightTable(String matchType) async {
    try {
      final List<Map<String, dynamic>> filteredData = [];

      // 全匹配直接取左表数据
      if (matchType == 'full_match') {
        filteredData.addAll(
          comparedData.where((row) {
            return row['match_status'] == "full_match";
          }).toList(),
        );
      } else if (matchType == 'key_match') {
        //取出左表数据，再从左表数据的right_data中取出右表数据
        filteredData.addAll(
          comparedData
              .where((row) => row['match_status'] == "key_match")
              .map((row) {
                // 确保right_data存在且是Map类型
                if (row['right_data'] is Map<String, dynamic>) {
                  return row['right_data'] as Map<String, dynamic>;
                } else {
                  return <String, dynamic>{}; // 返回空Map作为保护
                }
              })
              .where((row) => row.isNotEmpty) // 过滤掉无效数据
              .toList(),
        );
      } else {
        filteredData.addAll(
          comparedData.where((row) {
            if (row['is_left']) return false;
            return row['match_status'] == matchType;
          }).toList(),
        );
      }

      if (filteredData.isEmpty) {
        Logger().info('没有匹配类型的右表数据可导出');
        return;
      }

      // 生成CSV内容
      final csvContent = _generateCsvContent(filteredData, rightColumns);
      final fileName = 'right_table_${_getMatchTypeName(matchType)}.csv';
      await _saveCsvFile(fileName, csvContent);

      Logger().info('右表(${_getMatchTypeName(matchType)})导出成功');
    } catch (e) {
      Logger().error('右表导出失败: $e');
      rethrow;
    }
  }

  // 辅助方法：生成CSV内容
  String _generateCsvContent(List<Map<String, dynamic>> data, List<String> columns) {
    final buffer = StringBuffer();

    // 写入标题行 (只保留主键和各列名)
    buffer.write('主键'); // 不再包含"序号"
    for (var column in columns) {
      buffer.write('\t$column');
    }
    buffer.writeln();

    // 写入数据行
    for (var row in data) {
      buffer.write('${row['key']}'); // 不再包含serial
      for (var column in columns) {
        buffer.write('\t${row[column] ?? ''}');
      }
      buffer.writeln();
    }

    return buffer.toString();
  }

  // 辅助方法：保存CSV文件
  Future<void> _saveCsvFile(String fileName, String content) async {
    final filePath = await FilePicker.platform.saveFile(
      dialogTitle: '保存对比结果',
      fileName: fileName,
      type: FileType.custom,
      allowedExtensions: ['csv'],
    );

    if (filePath != null) {
      final file = File(filePath);
      await file.writeAsString(content);
    }
  }

  // 辅助方法：获取匹配类型名称
  String _getMatchTypeName(String matchType) {
    switch (matchType) {
      case 'full_match':
        return '整行匹配';
      case 'key_match':
        return '主键匹配';
      case 'no_match':
        return '不匹配';
      default:
        return '未知类型';
    }
  }

  @override
  void onOpenFile(String filePath, {dynamic appendArg}) {
    // TODO: implement onOpenFile
  }

  @override
  void onOpenFolder(String folderPath) {
    // TODO: implement onOpenFolder
  }

  @override
  void onDropOutlineNode(OutlineNode node) {
    // TODO: implement onDropOutlineNode
  }
}
