import 'dart:io';

import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:syncfusion_flutter_datagrid/datagrid.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/framework/controllers/tab_items_controller.dart';
import 'package:win_text_editor/modules/data_compare/controllers/data_compare_controller.dart';
import 'package:win_text_editor/modules/data_compare/widgets/data_compare_data_source.dart';
import 'package:win_text_editor/shared/utils/file_utils.dart';
import 'data_compare_grid.dart'; // 新增导入

class DataCompareView extends StatefulWidget {
  final String tabId;
  const DataCompareView({super.key, required this.tabId});

  @override
  State<DataCompareView> createState() => _DataCompareViewState();
}

class _DataCompareViewState extends State<DataCompareView> {
  late final DataCompareController _controller;
  late final DataGridController _dataGridController;
  bool _isControllerFromTabManager = false;

  late final DataCompareDataSource _dataSource;

  get tabManager => Provider.of<TabItemsController>(context, listen: false);

  @override
  void initState() {
    super.initState();
    _dataGridController = DataGridController();

    final controllerFromManager = tabManager.getController(widget.tabId);
    if (controllerFromManager != null) {
      _controller = controllerFromManager;
      _isControllerFromTabManager = true;
    } else {
      _controller = DataCompareController();
      _isControllerFromTabManager = false;
      tabManager.registerController(widget.tabId, _controller);
    }

    // 初始化DataSource
    _dataSource = DataCompareDataSource(_controller);
  }

  @override
  void dispose() {
    _dataGridController.dispose();

    if (!_isControllerFromTabManager) {
      _controller.dispose();
    }

    super.dispose();
  }

  Future<void> _importLeftTable() async {
    try {
      FilePickerResult? result = await FilePicker.platform.pickFiles(
        type: FileType.custom,
        allowedExtensions: ['csv'],
      );

      if (result != null) {
        final file = File(result.files.single.path!);
        final content = await FileUtils.readFileWithAutoEncoding(file);
        await _controller.importCsv(true, content);
      }
    } catch (e) {
      Logger().error('导入失败: ${e.toString()}');
    }
  }

  Future<void> _importRightTable() async {
    try {
      FilePickerResult? result = await FilePicker.platform.pickFiles(
        type: FileType.custom,
        allowedExtensions: ['csv'],
      );

      if (result != null) {
        final file = File(result.files.single.path!);
        final content = await FileUtils.readFileWithAutoEncoding(file);

        await _controller.importCsv(false, content);
      }
    } catch (e) {
      Logger().error('右表导入失败: ${e.toString()}');
    }
  }

  // 修改后的导出按钮方法
  Widget _buildExportButton(bool isLeftTable) {
    return Builder(
      builder: (btnContext) {
        return ElevatedButton(
          onPressed: () {
            final RenderBox renderBox = btnContext.findRenderObject() as RenderBox;
            final buttonSize = renderBox.size;
            final offset = renderBox.localToGlobal(Offset.zero);

            _showExportMenu(
              isLeftTable,
              RelativeRect.fromLTRB(
                offset.dx,
                offset.dy + buttonSize.height,
                offset.dx + buttonSize.width,
                offset.dy + buttonSize.height,
              ),
            );
          },
          style: ElevatedButton.styleFrom(
            backgroundColor: isLeftTable ? Colors.green[50] : Colors.blue[50],
            shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(8)),
          ),
          child: Row(
            children: [
              const Icon(Icons.output, size: 18),
              const SizedBox(width: 4),
              Text('导出${isLeftTable ? '左表' : '右表'}'),
            ],
          ),
        );
      },
    );
  }

  // 修改后的显示菜单方法
  void _showExportMenu(bool isLeftTable, RelativeRect position) {
    showMenu<String>(
      context: context,
      position: position,
      items: [
        const PopupMenuItem(
          value: 'full_match',
          child: Row(
            children: [
              Icon(Icons.double_arrow, color: Colors.green),
              SizedBox(width: 8),
              Text('整行匹配'),
            ],
          ),
        ),
        const PopupMenuItem(
          value: 'key_match',
          child: Row(
            children: [
              Icon(Icons.arrow_forward_ios, color: Colors.blue),
              SizedBox(width: 8),
              Text('主键匹配'),
            ],
          ),
        ),
        const PopupMenuItem(
          value: 'no_match',
          child: Row(
            children: [Icon(Icons.close, color: Colors.red), SizedBox(width: 8), Text('不匹配')],
          ),
        ),
      ],
    ).then((value) {
      if (value != null) {
        if (isLeftTable) {
          _controller.exportLeftTable(value);
        } else {
          _controller.exportRightTable(value);
        }
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider.value(
      value: _controller,
      child: Consumer<DataCompareController>(
        builder: (context, controller, child) {
          return Padding(
            padding: const EdgeInsets.only(left: 4.0, right: 4.0),
            child: Column(
              children: [
                Container(
                  height: 50,
                  padding: const EdgeInsets.symmetric(horizontal: 8),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      Row(
                        children: [
                          ElevatedButton(
                            onPressed: _importLeftTable,
                            style: ElevatedButton.styleFrom(
                              backgroundColor: Colors.green[50],
                              shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(8)),
                            ),
                            child: const Text('导入左表(CSV)'),
                          ),
                          const SizedBox(width: 8),
                          _buildExportButton(true), // 使用新的导出按钮
                        ],
                      ),
                      Row(
                        children: [
                          ElevatedButton(
                            onPressed: _importRightTable,
                            style: ElevatedButton.styleFrom(
                              backgroundColor: Colors.blue[50],
                              shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(8)),
                            ),
                            child: const Text('导入右表(CSV)'),
                          ),
                          const SizedBox(width: 8),
                          _buildExportButton(false), // 使用新的导出按钮
                        ],
                      ),
                    ],
                  ),
                ),
                Expanded(child: DataCompareGrid(dataSource: _dataSource, controller: controller)),
              ],
            ),
          );
        },
      ),
    );
  }
}
