// xml_extract_service.dart
import 'dart:io';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/shared/utils/file_utils.dart';
import 'package:xml/xml.dart' as xml;
import 'package:win_text_editor/modules/data_extract/models/search_result.dart';
import 'package:win_text_editor/modules/data_extract/models/xml_rule.dart';

class XmlExtractService {
  Future<List<SearchResult>> extractFromDirectory({
    required String directory,
    required String fileType,
    required XmlRule rule,
  }) async {
    final results = <SearchResult>[];
    final dir = Directory(directory);
    int rowNum = 1;

    await for (var entity in dir.list(recursive: true)) {
      if (entity is File && FileUtils.matchesFileType(entity.path, fileType)) {
        try {
          final fileContent = await entity.readAsString();
          final document = xml.XmlDocument.parse(fileContent);
          final fileResults = _extractWithRule(document, rule, entity.path);

          // 如果是"仅提取重复项"模式，确保只保留matchCount>1的结果
          if (rule.isDuplicatesOnly) {
            results.addAll(fileResults.where((r) => r.matchCount > 1));
          } else {
            results.addAll(fileResults);
          }
        } catch (e) {
          Logger().error('XmlExtractService.extractFromDirectory方法执行出错: $e');
          results.add(
            SearchResult(
              rowNum: rowNum++,
              filePath: entity.path,
              content: 'Error: $e',
              matchCount: 1,
            ),
          );
        }
      }
    }

    // 更新行号
    for (int i = 0; i < results.length; i++) {
      results[i] = results[i].copyWith(rowNum: i + 1);
    }

    return results;
  }

  List<SearchResult> _extractWithRule(xml.XmlDocument document, XmlRule rule, String filePath) {
    final List<xml.XmlElement> nodes = document.findAllElements(rule.nodePath).toList();

    if (rule.isFirstOccurrence && nodes.isNotEmpty) {
      final attr = nodes.first.getAttribute(rule.attributeName);
      return attr != null
          ? [SearchResult(rowNum: 0, filePath: filePath, content: attr, matchCount: 1)]
          : [];
    } else if (rule.isDuplicatesOnly) {
      return _findDuplicateAttributes(nodes, rule.attributeName, filePath);
    } else {
      final attributeCounts = <String, int>{};
      // Count occurrences of each attribute value
      for (final node in nodes) {
        final attr = node.getAttribute(rule.attributeName);
        if (attr != null) {
          attributeCounts[attr] = (attributeCounts[attr] ?? 0) + 1;
        }
      }

      return nodes
          .map((node) => node.getAttribute(rule.attributeName))
          .where((attr) => attr != null)
          .map(
            (attr) => SearchResult(
              rowNum: 0, // Will be updated later
              filePath: filePath,
              content: attr!,
              matchCount: attributeCounts[attr]!,
            ),
          )
          .toList();
    }
  }

  List<SearchResult> _findDuplicateAttributes(
    List<xml.XmlElement> nodes,
    String attributeName,
    String filePath,
  ) {
    final attributeCounts = <String, int>{};
    final attributeFirstOccurrence = <String, xml.XmlElement>{};

    // 第一次遍历：统计每个属性值的出现次数，并记录第一次出现的节点
    for (final node in nodes) {
      final attr = node.getAttribute(attributeName);
      if (attr != null) {
        attributeCounts[attr] = (attributeCounts[attr] ?? 0) + 1;
        attributeFirstOccurrence.putIfAbsent(attr, () => node);
      }
    }

    // 只保留出现次数>1的属性值
    final duplicateAttributes =
        attributeCounts.entries.where((entry) => entry.value > 1).map((entry) {
          final attr = entry.key;
          return SearchResult(
            rowNum: 0, // 稍后会更新
            filePath: filePath,
            content: attr,
            matchCount: entry.value,
          );
        }).toList();

    return duplicateAttributes;
  }
}
