import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:win_text_editor/modules/data_extract/controllers/data_extract_controller.dart';
import 'package:win_text_editor/modules/data_extract/models/xml_rule.dart';

class ConditionSetting extends StatefulWidget {
  const ConditionSetting({super.key});

  @override
  State<ConditionSetting> createState() => _ConditionSettingState();
}

class _ConditionSettingState extends State<ConditionSetting> {
  final _nodePathController = TextEditingController();
  final _attributeNameController = TextEditingController();
  final _namespacePrefixController = TextEditingController();
  int _extractionOption = 0; // 0: all, 1: first occurrence, 2: duplicates only
  bool _isExtracting = false;

  @override
  void dispose() {
    _nodePathController.dispose();
    _attributeNameController.dispose();
    _namespacePrefixController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final controller = context.watch<DataExtractController>();

    return Card(
      child: Padding(
        padding: const EdgeInsets.all(16.0),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: [
            const Text('数据提取规则设置：', style: TextStyle(fontSize: 14, fontWeight: FontWeight.bold)),
            const SizedBox(height: 16),

            // 规则输入表单
            TextField(
              controller: _nodePathController,
              decoration: const InputDecoration(
                labelText: '节点名称',
                hintText: '如: business:Service',
                border: OutlineInputBorder(),
              ),
            ),
            const SizedBox(height: 12),
            TextField(
              controller: _attributeNameController,
              decoration: const InputDecoration(
                labelText: '属性名称',
                hintText: '如: chineseName 或 name',
                border: OutlineInputBorder(),
              ),
            ),

            const SizedBox(height: 12),

            // Radio buttons for extraction options
            Row(
              children: [
                Radio<int>(
                  value: 1,
                  groupValue: _extractionOption,
                  onChanged: (value) {
                    setState(() {
                      _extractionOption = value ?? 0;
                    });
                  },
                ),
                const Text('仅提取第一个匹配项'),

                const SizedBox(width: 16),

                Radio<int>(
                  value: 2,
                  groupValue: _extractionOption,
                  onChanged: (value) {
                    setState(() {
                      _extractionOption = value ?? 0;
                    });
                  },
                ),
                const Text('仅提取重复项'),
              ],
            ),

            const SizedBox(height: 16),
            // 操作按钮行
            Row(
              children: [
                Expanded(
                  child: ElevatedButton.icon(
                    icon: const Icon(Icons.play_arrow),
                    label: const Text('开始'),
                    onPressed: _isExtracting ? null : () => _startExtraction(controller),
                  ),
                ),
                const SizedBox(width: 12),
                Expanded(
                  child: ElevatedButton.icon(
                    icon: const Icon(Icons.stop, color: Colors.red),
                    label: const Text('停止', style: TextStyle(color: Colors.red)),
                    onPressed: _isExtracting ? _stopExtraction : null,
                  ),
                ),
              ],
            ),
          ],
        ),
      ),
    );
  }

  void _setRule() {
    final nodePath = _nodePathController.text.trim();
    final attributeName = _attributeNameController.text.trim();

    if (nodePath.isNotEmpty && attributeName.isNotEmpty) {
      final controller = Provider.of<DataExtractController>(context, listen: false);
      controller.setRule(
        XmlRule(
          nodePath: nodePath,
          attributeName: attributeName,
          isFirstOccurrence: _extractionOption == 1,
          isDuplicatesOnly: _extractionOption == 2,
          namespacePrefix:
              _namespacePrefixController.text.trim().isNotEmpty
                  ? _namespacePrefixController.text.trim()
                  : null,
        ),
      );
    }
  }

  Future<void> _startExtraction(DataExtractController controller) async {
    if (controller.searchDirectory.isEmpty) {
      ScaffoldMessenger.of(context).showSnackBar(const SnackBar(content: Text('请先选择搜索目录')));
      return;
    }

    _setRule();

    setState(() => _isExtracting = true);
    try {
      await controller.executeExtraction();
    } finally {
      if (mounted) {
        setState(() => _isExtracting = false);
      }
    }
  }

  void _stopExtraction() {
    // 这里需要确保控制器中有取消提取的逻辑
    final controller = Provider.of<DataExtractController>(context, listen: false);
    // 假设控制器中有cancelExtraction方法
    controller.cancelExtraction();
    setState(() => _isExtracting = false);
  }
}
