import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:syncfusion_flutter_datagrid/datagrid.dart';
import 'package:path/path.dart' as path;

import 'package:file_picker/file_picker.dart';
import 'dart:io';

import 'package:win_text_editor/modules/data_extract/controllers/data_extract_controller.dart';
import 'package:win_text_editor/shared/components/my_grid_column.dart';

class ResultsView extends StatelessWidget {
  const ResultsView({super.key});

  @override
  Widget build(BuildContext context) {
    final controller = context.watch<DataExtractController>();

    return Card(
      child: GestureDetector(
        onSecondaryTapDown: (details) {
          _showContextMenu(context, details.globalPosition, controller);
        },
        child: _buildLocateGrid(controller),
      ),
    );
  }

  Future<void> _showContextMenu(
    BuildContext context,
    Offset position,
    DataExtractController controller,
  ) async {
    // 获取渲染对象以正确定位菜单
    final renderBox = context.findRenderObject() as RenderBox;
    final localPosition = renderBox.globalToLocal(position);

    // 显示菜单并等待选择结果
    final result = await showMenu<String>(
      context: context,
      position: RelativeRect.fromLTRB(
        position.dx,
        position.dy,
        position.dx + renderBox.size.width - localPosition.dx,
        position.dy + renderBox.size.height - localPosition.dy,
      ),
      items: [
        const PopupMenuItem<String>(value: 'exportFileName', child: Text('导出文件名(csv)')),
        const PopupMenuItem<String>(value: 'exportContent', child: Text('导出内容(csv)')),
        const PopupMenuItem<String>(value: 'exportAll', child: Text('导出全部(csv)')),
      ],
    );

    // 处理菜单选择结果
    if (result != null && result.startsWith('export') && context.mounted) {
      try {
        await _exportToCsv(controller, result);
      } catch (e) {
        if (context.mounted) {
          ScaffoldMessenger.of(
            context,
          ).showSnackBar(SnackBar(content: Text('导出失败: ${e.toString()}')));
        }
      }
    }
  }

  Future<void> _exportToCsv(DataExtractController controller, String? exportType) async {
    String csvData = '';
    csvData =
        exportType == 'exportFileName'
            ? '文件名称\n'
            : (exportType == 'exportContent' ? '内容\n' : '文件名称\t内容\n');
    for (var result in controller.results) {
      switch (exportType) {
        case 'exportFileName':
          csvData += '${path.basename(result.filePath)}\n';
          break;
        case 'exportContent':
          csvData += '${result.content}\n';
          break;
        default:
          csvData += '${path.basename(result.filePath)}\t${result.content}\n';
      }
    }

    final filePath = await FilePicker.platform.saveFile(
      dialogTitle: '保存导出结果',
      fileName: 'search_results.csv',
      type: FileType.custom,
      allowedExtensions: ['csv'],
    );

    if (filePath != null) {
      final file = File(filePath);
      await file.writeAsString(csvData);
    }
  }

  Widget _buildLocateGrid(DataExtractController controller) {
    return SfDataGrid(
      rowHeight: 32,
      headerRowHeight: 32,
      source: LocateDataSource(controller),
      columns: [
        ShortGridColumn(columnName: 'rowNum', label: '序号'),
        MyGridColumn(columnName: 'file', label: '文件名称', minimumWidth: 300),
        MyGridColumn(columnName: 'content', label: '内容'),
      ShortGridColumn(columnName: 'matchCount', label: '匹配次数'),
      ],
      selectionMode: SelectionMode.multiple,
      navigationMode: GridNavigationMode.cell,
      gridLinesVisibility: GridLinesVisibility.both,
      headerGridLinesVisibility: GridLinesVisibility.both,
      allowSorting: false,
      allowFiltering: false,
      columnWidthMode: ColumnWidthMode.fill,
      isScrollbarAlwaysShown: true,
      allowColumnsResizing: true, // 关键开关
      columnResizeMode: ColumnResizeMode.onResizeEnd,
    );
  }
}

class LocateDataSource extends DataGridSource {
  final DataExtractController controller;

  LocateDataSource(this.controller);

  @override
  List<DataGridRow> get rows =>
      controller.results.map((result) {
        return DataGridRow(
          cells: [
            DataGridCell(columnName: 'rowNum', value: result.rowNum),
            DataGridCell(columnName: 'file', value: path.basename(result.filePath)),
            DataGridCell(columnName: 'content', value: result.content),
            DataGridCell(columnName: 'matchCount', value: result.matchCount),
          ],
        );
      }).toList();

  @override
  DataGridRowAdapter? buildRow(DataGridRow row) {
    return DataGridRowAdapter(
      cells: row.getCells().map<Widget>((cell) {
            return Container(
          alignment: cell.columnName == 'matchCount' 
              ? Alignment.center 
              : Alignment.centerLeft,
              padding: const EdgeInsets.symmetric(horizontal: 8),
              child: Text(cell.value.toString(), overflow: TextOverflow.ellipsis),
            );
          }).toList(),
    );
  }
}
