import 'package:flutter/material.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/modules/data_format/services/mustache_service.dart';
import 'package:win_text_editor/modules/outline/models/outline_node.dart';
import 'package:win_text_editor/shared/base/base_content_controller.dart';
import 'grid_view_controller.dart';

class DataFormatController extends BaseContentController {
  final GridViewController gridController;
  String _templateText = '';

  // 使用ValueNotifier来管理结果文本
  final ValueNotifier<String> _resultTextNotifier = ValueNotifier('');

  // 暴露notifier给外部访问
  ValueNotifier<String> get resultTextNotifier => _resultTextNotifier;

  //---------------初始化方法----

  DataFormatController() : gridController = GridViewController() {
    _setupCrossControllerCommunication();
  }

  //设置跨控制器状态协同
  void _setupCrossControllerCommunication() {}

  // Set template text from the editor
  void setTemplateText(String text) {
    _templateText = text;
  }

  // Apply the template to grid data
  void applyTemplate() {
    if (_templateText.isEmpty) {
      _resultTextNotifier.value = '错误：模板文本不能为空';
      Logger().error(_resultTextNotifier.value);
      return;
    }

    // Validate template syntax
    if (!MustacheService.validateTemplate(_templateText)) {
      _resultTextNotifier.value = '错误：模板语法不正确';
      Logger().error(_resultTextNotifier.value);
      return;
    }

    // Get variables from template
    final templateVars = MustacheService.getTemplateVariables(_templateText);
    if (templateVars.isEmpty) {
      _resultTextNotifier.value = '错误：模板中没有找到变量';
      Logger().error(_resultTextNotifier.value);
      return;
    }

    // Get grid data
    final gridData = gridController.csvData;
    if (gridData.isEmpty || gridData.length < 2) {
      _resultTextNotifier.value = '错误：没有CSV数据';
      Logger().error(_resultTextNotifier.value);
      return;
    }

    final headers = gridData.first;
    final dataRows = gridData.sublist(1);

    // Check if all template variables exist in headers
    final missingVars = templateVars.where((varName) => !headers.contains(varName)).toList();
    if (missingVars.isNotEmpty) {
      _resultTextNotifier.value = '错误:没有找到模板变量： ${missingVars.join(', ')}';
      Logger().error(_resultTextNotifier.value);
      return;
    }

    // Process each row
    final resultBuffer = StringBuffer();
    for (final row in dataRows) {
      final rowData = <String, dynamic>{};
      for (var i = 0; i < headers.length; i++) {
        if (i < row.length) {
          rowData[headers[i]] = row[i];
        } else {
          rowData[headers[i]] = '';
        }
      }

      try {
        final rendered = MustacheService.applyTemplate(_templateText, rowData);
        resultBuffer.writeln(rendered);
      } catch (e) {
        resultBuffer.writeln('Error processing row: $e');
      }
    }

    _resultTextNotifier.value = resultBuffer.toString();
  }

  @override
  void onOpenFile(String filePath, {dynamic appendArg}) {}

  @override
  void onOpenFolder(String folderPath) {
    // 不支持打开文件夹
  }

  @override
  void dispose() {
    _resultTextNotifier.dispose(); // 记得在dispose时释放资源
    gridController.dispose();
    super.dispose();
  }

  @override
  void onDropOutlineNode(OutlineNode node) {
    // TODO: implement onDropOutlineNode
  }
}
