import 'package:flutter/material.dart';
import 'package:syncfusion_flutter_datagrid/datagrid.dart';
import 'package:file_picker/file_picker.dart';
import 'dart:io';
import 'package:csv/csv.dart';
import 'package:win_text_editor/modules/data_format/controllers/grid_view_controller.dart';
import 'package:win_text_editor/shared/components/my_grid_column.dart';

class DataGridView extends StatefulWidget {
  final GridViewController controller;

  const DataGridView({super.key, required this.controller});

  @override
  State<DataGridView> createState() => _DataGridViewState();
}

class _DataGridViewState extends State<DataGridView> {
  final TextEditingController _filePathController = TextEditingController();
  List<List<dynamic>> _csvData = [];
  String? _delimiter;
  final ScrollController _horizontalScrollController = ScrollController();

  @override
  void dispose() {
    _filePathController.dispose();
    _horizontalScrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        // 文件选择区域
        Padding(
          padding: const EdgeInsets.all(8.0),
          child: Row(
            children: [
              Expanded(
                child: TextField(
                  controller: _filePathController,
                  decoration: const InputDecoration(
                    labelText: 'CSV文件路径',
                    border: OutlineInputBorder(),
                  ),
                  readOnly: true,
                ),
              ),
              const SizedBox(width: 8),
              ElevatedButton(onPressed: _pickAndLoadCsvFile, child: const Text('选择文件')),
            ],
          ),
        ),
        // 数据表格区域
        Expanded(
          child: Row(
            children: [
              _csvData.isEmpty ? const Center(child: Text('请选择CSV文件')) : _buildScrollableDataGrid(),
            ],
          ),
        ),
      ],
    );
  }

  Widget _buildScrollableDataGrid() {
    return Align(
      // 添加 Align 组件使整个表格左对齐
      alignment: Alignment.topLeft, // 设置为左上对齐
      child: Scrollbar(
        controller: _horizontalScrollController,
        thumbVisibility: true,
        trackVisibility: true,
        child: SingleChildScrollView(
          controller: _horizontalScrollController,
          scrollDirection: Axis.horizontal,
          child: SizedBox(width: _calculateTableWidth(), child: _buildCsvDataGrid()),
        ),
      ),
    );
  }

  double _calculateTableWidth() {
    if (_csvData.isEmpty || _csvData.length < 2) return 0;
    final columnCount = _csvData.first.length;
    return columnCount * 150.0; // 每列宽度150
  }

  Future<void> _pickAndLoadCsvFile() async {
    try {
      final result = await FilePicker.platform.pickFiles(
        type: FileType.custom,
        allowedExtensions: ['csv'],
      );

      if (result != null) {
        final file = File(result.files.single.path!);
        _filePathController.text = file.path;
        final content = await file.readAsString();

        // 检测分隔符（第一行是标题行）
        final firstLine = content.split('\n').first.trim();
        _delimiter = firstLine.contains('\t') ? '\t' : ',';

        // 解析CSV - 使用更健壮的解析方式
        final csvTable = const CsvToListConverter(
          shouldParseNumbers: false,
          allowInvalid: false,
          eol: '\n',
        ).convert(content, fieldDelimiter: _delimiter);

        // 清理数据并添加序号列
        final cleanedData =
            csvTable
                .where(
                  (row) => row.isNotEmpty && row.any((cell) => cell.toString().trim().isNotEmpty),
                )
                .toList(); // 先转换为List

        // 添加序号列
        final dataWithIndex =
            cleanedData.asMap().entries.map((entry) {
              final index = entry.key;
              final row = entry.value;
              // 如果是标题行(第一行)，添加"序号"列标题
              if (index == 0) {
                return ['序号', ...row.map((cell) => cell.toString().trim()).toList()];
              }
              // 数据行添加序号(从1开始)
              return [index, ...row.map((cell) => cell.toString().trim()).toList()];
            }).toList();

        setState(() {
          _csvData = dataWithIndex;
        });

        widget.controller.setCsvData(dataWithIndex);
      }
    } catch (e) {
      ScaffoldMessenger.of(context).showSnackBar(SnackBar(content: Text('加载CSV文件失败: $e')));
    }
  }

  Widget _buildCsvDataGrid() {
    if (_csvData.isEmpty || _csvData.length < 2) {
      return const Center(child: Text('没有有效数据或数据格式不正确'));
    }

    final headers = _csvData.first;
    final dataRows = _csvData.sublist(1);

    final columns = [
      // 添加序号列
      MyGridColumn(columnName: '序号', label: '序号', minimumWidth: 60),

      // 原有列
      ...headers.sublist(1).map<GridColumn>((header) {
        return MyGridColumn(
          columnName: header.toString(),
          label: header.toString(),
          minimumWidth: 150,
        );
      }).toList(),
    ];

    final dataSource = _CsvDataSource(headers: headers, rows: dataRows);

    return SfDataGrid(
      rowHeight: 32,
      headerRowHeight: 32,
      source: dataSource,
      columns: columns,
      gridLinesVisibility: GridLinesVisibility.both,
      headerGridLinesVisibility: GridLinesVisibility.both,
      columnWidthMode: ColumnWidthMode.fill,
      allowColumnsResizing: true,
      columnResizeMode: ColumnResizeMode.onResizeEnd,
    );
  }
}

class _CsvDataSource extends DataGridSource {
  final List<dynamic> headers;
  final List<List<dynamic>> _rows;

  _CsvDataSource({required this.headers, required List<List<dynamic>> rows}) : _rows = rows;

  @override
  List<DataGridRow> get rows {
    return _rows.map((row) {
      return DataGridRow(
        cells:
            headers.asMap().entries.map((entry) {
              final columnIndex = entry.key;
              final columnName = entry.value.toString();
              final cellValue = columnIndex < row.length ? row[columnIndex].toString() : '';
              return DataGridCell(columnName: columnName, value: cellValue);
            }).toList(),
      );
    }).toList();
  }

  @override
  DataGridRowAdapter? buildRow(DataGridRow row) {
    return DataGridRowAdapter(
      cells:
          row.getCells().map<Widget>((dataGridCell) {
            return Container(
              padding: const EdgeInsets.symmetric(horizontal: 8.0),
              alignment: Alignment.centerLeft,
              child: Text(
                dataGridCell.value.toString(),
                overflow: TextOverflow.ellipsis,
                maxLines: 1,
              ),
            );
          }).toList(),
    );
  }
}
