import 'package:syncfusion_flutter_datagrid/datagrid.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/framework/services/macro_template_service.dart';
import 'package:win_text_editor/modules/outline/models/code_partner.dart';
import 'package:win_text_editor/modules/outline/models/outline_node.dart';
import 'package:win_text_editor/shared/models/std_filed.dart';
import 'package:win_text_editor/shared/uft_std_fields/field_data_source.dart';
import 'package:win_text_editor/modules/memory_table/controllers/index_data_source.dart';
import 'package:win_text_editor/modules/memory_table/models/memory_table.dart';
import 'package:win_text_editor/modules/memory_table/services/memory_table_service.dart';
import 'package:win_text_editor/shared/base/base_content_controller.dart';

class MemoryTableController extends BaseContentController {
  String? _errorMessage;
  String tableName = "";
  String objectId = "";
  String chineseName = "";

  late DataGridSource fieldsSource;
  late DataGridSource indexesSource;
  final MacroTemplateService templateService = MacroTemplateService();

  // 新增：维护MemoryTable对象
  late MemoryTable _memoryTable;

  MemoryTableController() {
    // 初始化空数据
    final initialFields = [Field('1', '', '', '', false), Field('2', '', '', '', false)];

    final initialIndexes = [Index('', false, '', '', false)];

    fieldsSource = FieldsDataSource(
      initialFields,
      onSelectionChanged: (index, isSelected) {
        updateFieldSelection(index, isSelected);
      },
    );

    indexesSource = IndexesDataSource(
      initialIndexes,
      onSelectionChanged: (index, isSelected) {
        updateIndexSelection(index, isSelected);
      },
    );

    // 初始化MemoryTable
    _memoryTable = MemoryTable(tableName: '', columns: initialFields, indexes: initialIndexes);
  }

  String? get errorMessage => _errorMessage;

  // 新增：获取当前MemoryTable
  MemoryTable get memoryTable => _memoryTable;

  void initTemplateService() {
    if (!templateService.inited) {
      templateService.init();
    }
  }

  String? genCodeString(List<String> macroList) {
    initTemplateService();
    return templateService.renderTemplate(macroList, _memoryTable.toMap());
  }

  @override
  Future<void> onOpenFile(String filePath, {dynamic appendArg}) async {
    Logger().info("Opening file: $filePath");
    try {
      final tableData = await MemoryTableService.parseStructureFile(filePath);

      // Update controller state
      tableName = tableData.tableName;
      chineseName = tableData.chineseName;
      objectId = tableData.objectId;

      // Update data sources
      (fieldsSource as FieldsDataSource).updateData(tableData.fields);
      (indexesSource as IndexesDataSource).updateData(tableData.indexes);

      // 更新MemoryTable对象
      _memoryTable = MemoryTable(
        tableName: tableName,
        columns: tableData.fields,
        indexes: tableData.indexes,
      );

      // Clear any previous error
      _errorMessage = null;

      // Notify UI to update
      notifyListeners();
    } catch (e) {
      _errorMessage = e.toString();
      notifyListeners();
      Logger().error("Error opening file: $e");
    }
  }

  // 新增：更新字段选择状态
  void updateFieldSelection(int index, bool isSelected) {
    final fields = (fieldsSource as FieldsDataSource).data;
    if (index >= 0 && index < fields.length) {
      fields[index].isSelected = isSelected;
      fieldsSource.notifyListeners();

      // 同步更新MemoryTable
      _memoryTable.columns[index].isSelected = isSelected;
      notifyListeners();
    }
  }

  // 新增：更新索引选择状态
  void updateIndexSelection(int index, bool isSelected) {
    final indexes = (indexesSource as IndexesDataSource).data;
    if (index >= 0 && index < indexes.length) {
      indexes[index].isSelected = isSelected;
      indexesSource.notifyListeners();

      // 同步更新MemoryTable
      _memoryTable.indexes[index].isSelected = isSelected;
      notifyListeners();
    }
  }

  @override
  void onOpenFolder(String folderPath) {
    // 不支持打开文件夹
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void onDropOutlineNode(OutlineNode node) {
    _memoryTable.codePartners.add(CodePartner.fromOutlineNode(node));
    notifyListeners();
  }
}
