// memory_table_service.dart
import 'dart:io';

import 'package:win_text_editor/modules/memory_table/models/memory_table.dart';
import 'package:win_text_editor/shared/data/std_fields_cache.dart';
import 'package:win_text_editor/shared/models/std_filed.dart';
import 'package:xml/xml.dart' as xml;
import 'package:path/path.dart' as path;
import 'package:win_text_editor/framework/controllers/logger.dart';

class MemoryTableService {
  static Future<TableData> parseStructureFile(String filePath) async {
    try {
      // 1. Check file extension
      if (!filePath.toLowerCase().endsWith('.uftstructure')) {
        throw const FormatException("文件扩展名必须是.uftstructure");
      }

      // 2. 查找 metadata 目录和 stdfield.stfield 文件
      await StdFieldsCache.loadForFile(filePath);

      // 3. Read and parse structure file content
      final file = File(filePath);
      final content = await file.readAsString();

      final document = xml.XmlDocument.parse(content);
      final structureNode = document.findAllElements('structure:Structure').firstOrNull;

      if (structureNode == null) {
        throw const FormatException("文件格式错误：缺少structure:Structure节点");
      }

      // 4. Get basic info
      final fileNameWithoutExt = path.basenameWithoutExtension(filePath);
      final chineseName = structureNode.getAttribute('chineseName') ?? '';
      final objectId = structureNode.getAttribute('objectId') ?? '';

      // 5. Process properties (fields)
      final properties = document.findAllElements('properties');
      final fields = <Field>[];
      int index = 1;

      for (final property in properties) {
        final id = property.getAttribute('id') ?? '';
        // 尝试从缓存获取标准字段信息
        final stdField = StdFieldsCache.getData(id);
        fields.add(
          Field(
            (index++).toString(), // 序号
            id, // 名称
            stdField?.chineseName ?? '', // 中文名
            stdField?.dateType ?? '', // 类型
          ),
        );
      }

      // 6. Process indexes
      final indexes = document.findAllElements('indexs');
      final indexList = <Index>[];

      for (final indexNode in indexes) {
        final name = indexNode.getAttribute('name') ?? '';
        final containerType = indexNode.getAttribute('containerType');
        final isPrimary = containerType == null;
        final rule = containerType ?? '';

        // Get all index fields
        final items = indexNode.findElements('items');
        final fieldsList =
            items
                .map((item) => item.getAttribute('attrname') ?? '')
                .where((f) => f.isNotEmpty)
                .toList();
        final indexFields = fieldsList.join(',');

        indexList.add(
          Index(
            name, // 索引名称
            isPrimary, // 是否主键
            indexFields, // 索引字段
            rule, // 规则
          ),
        );
      }

      return TableData(
        tableName: fileNameWithoutExt,
        chineseName: chineseName,
        objectId: objectId,
        fields: fields.isNotEmpty ? fields : [],
        indexes: indexList.isNotEmpty ? indexList : [],
      );
    } on xml.XmlParserException catch (e) {
      Logger().error("XML解析错误: ${e.message}");
      rethrow;
    } catch (e) {
      Logger().error("处理文件时发生错误: $e");
      rethrow;
    }
  }
}

class TableData {
  final String tableName;
  final String chineseName;
  final String objectId;
  final List<Field> fields;
  final List<Index> indexes;

  TableData({
    required this.tableName,
    required this.chineseName,
    required this.objectId,
    required this.fields,
    required this.indexes,
  });
}
