import 'package:flutter/material.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/modules/outline/models/outline_node.dart';
import 'package:win_text_editor/modules/outline/services/outline_service.dart';

class OutlineProvider with ChangeNotifier {
  List<OutlineNode> _outlineNode = [];
  bool _isLoading = true;
  String? _currentRootPath; // 跟踪当前根路径
  static OutlineNode rootNode = OutlineNode(
    name: "所有",
    title: "所有",
    value: 'All',
    isDirectory: true,
    isRoot: true,
    isExpanded: true,
  );

  bool get isLoading => _isLoading;
  bool get hasRoot => _outlineNode.isNotEmpty && _outlineNode[0].isRoot;

  // 移除构造函数的_initOutlineTree调用
  OutlineProvider();

  String? get rootPath => _currentRootPath;

  // 新增方法：手动设置根路径
  Future<void> setRootPath(String path) async {
    if (path == _currentRootPath) {
      return;
    }
    _currentRootPath = path;
    _loadRootDirectory();
  }

  // outline_provider.dart 中的 _loadRootDirectory 方法
  Future<void> _loadRootDirectory() async {
    if (_currentRootPath == null) return;

    _isLoading = true;
    notifyListeners();

    try {
      // 验证目录结构
      final isValid = await OutlineService.validateDirectoryStructure(_currentRootPath!);

      if (!isValid) {
        Logger().error('目录结构不完整，请检查必需的文件和目录');
        _outlineNode = []; // 清空现有节点
      } else {
        // 获取分词结果节点
        final wordNodes = await OutlineService.getWordNodes(_currentRootPath!);
        Logger().info('获取到 ${wordNodes.length} 个分词结果');
        _outlineNode = wordNodes;
      }
    } catch (e) {
      Logger().error('加载根目录时出错: $e');
      _outlineNode = [];
    } finally {
      _isLoading = false;
      notifyListeners();
    }
  }

  Future<void> loadDirectoryContents(OutlineNode dirNode, {bool refresh = false}) async {
    if (dirNode.children.isNotEmpty) {
      return;
    }
    _isLoading = true;
    notifyListeners();

    if (_currentRootPath != null) {
      await OutlineService.loadChildren(_currentRootPath!, dirNode);
      dirNode.isExpanded = true;
    }
    _isLoading = false;
    notifyListeners();
  }

  Future<void> refreshOutlineTree() async {
    _isLoading = true;
    notifyListeners();
    rootNode.isExpanded = false;
    rootNode.children.clear();
    final wordNodes = await OutlineService.getWordNodes(_currentRootPath!);
    rootNode.title = "所有";
    rootNode.children = wordNodes;
    rootNode.isExpanded = true;
    _isLoading = false;
    notifyListeners();
  }

  // 根据搜索查询过滤节点
  List<OutlineNode> applyFilter(String searchQuery) {
    if (searchQuery.isEmpty) {
      if (rootNode.children.isEmpty) rootNode.children = _outlineNode;
      return [rootNode];
    }

    OutlineService.applySearchFilter(rootNode, searchQuery);
    return [rootNode];
  }
}
