import 'dart:io';

import 'package:path/path.dart' as path;
import 'package:win_text_editor/framework/common/constants.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/modules/outline/models/outline_node.dart';
import 'package:win_text_editor/modules/outline/services/component_service.dart';
import 'package:xml/xml.dart';

class FunctionsService {
  //服务层索引
  static Map<String, List<String>> uftbusinessMap = {};

  //原子层索引
  static Map<String, List<String>> uftatomserviceMap = {};
  static Map<String, List<String>> uftatomFunctionMap = {};

  static Future<void> initFunctionsMap(String rootPath) async {
    await _initUftMap('uftbusiness', rootPath, uftbusinessMap, ['.uftfunction']);

    await _initUftMap('uftatom', rootPath, uftatomserviceMap, ['.uftatomservice']);
    await _initUftMap('uftatom', rootPath, uftatomFunctionMap, ['.uftatomfunction']);
  }

  static Future<void> _initUftMap(
    String dirName,
    String rootPath,
    Map<String, List<String>> targetMap,
    List<String> fileExtensions,
  ) async {
    final dir = Directory('$rootPath\\$dirName');
    if (!await dir.exists()) {
      Logger().error('$dirName目录不存在');
      return;
    }

    targetMap.clear();

    try {
      final files =
          await dir
              .list(recursive: true)
              .where((entity) => fileExtensions.any((ext) => entity.path.endsWith(ext)))
              .cast<File>()
              .toList();

      await Future.wait(files.map((file) => _processUftFile(file, targetMap)));
    } catch (e) {
      Logger().error('加载$dirName对象失败: $e');
      rethrow;
    }
  }

  static Future<void> _processUftFile(File file, Map<String, List<String>> targetMap) async {
    try {
      final content = await file.readAsString();
      final document = XmlDocument.parse(content);
      final businessNode = document.rootElement;

      final chineseName = businessNode.getAttribute('chineseName') ?? '未命名';
      targetMap[chineseName] = [file.path];

      final parameterNodes = <XmlElement>[
        ...businessNode.findElements('inputParameters'),
        ...businessNode.findElements('outputParameters'),
        ...businessNode.findElements('internalParams'),
      ];

      for (final item in parameterNodes) {
        final id = item.getAttribute('id');
        if (id == null) continue;

        if (item.getAttribute('paramType') == null ||
            item.getAttribute('paramType') != 'COMPONENT') {
          targetMap[chineseName]?.add(id);
        } else {
          targetMap[chineseName]?.addAll(ComponentService.componentFieldMap[id]?.sublist(1) ?? []);
        }
      }
    } catch (e) {
      Logger().error('解析文件 ${file.path} 失败: $e');
    }
  }

  //加载原子服务层
  static void loadAtomService(String? fieldName, OutlineNode parentNode) {
    if (fieldName == null || fieldName.isEmpty) return;

    uftatomserviceMap.forEach((key, value) {
      if (value.contains(fieldName)) {
        parentNode.children.add(
          OutlineNode(
            name: path.basenameWithoutExtension(value[0]), // 文件名
            title: key,
            value: Constants.atomService,
            isDirectory: false, // 这些是叶子节点
            depth: 4,
            path: value[0],
          ),
        );
      }
    });
  }

  static Future<void> loadAtomFunction(String? fieldName, OutlineNode parentNode) async {
    if (fieldName == null || fieldName.isEmpty) return;

    uftatomFunctionMap.forEach((key, value) {
      if (value.contains(fieldName)) {
        parentNode.children.add(
          OutlineNode(
            name: path.basenameWithoutExtension(value[0]), // 文件名
            title: key,
            value: Constants.atomFunction,
            isDirectory: false, // 这些是叶子节点
            depth: 4,
            path: value[0],
          ),
        );
      }
    });
  }

  //加载业务逻辑层
  static void loadBusiness(String? fieldName, OutlineNode parentNode) {
    if (fieldName == null || fieldName.isEmpty) return;

    uftbusinessMap.forEach((key, value) {
      final functionName = path.basenameWithoutExtension(value[0]);
      if (value.contains(fieldName)) {
        parentNode.children.add(
          OutlineNode(
            name: functionName,
            title: key,
            value: Constants.business,
            isDirectory: false, // 这些是叶子节点
            depth: 4,
            path: value[0], // 文件名
          ),
        );
      }
    });
  }

  static List<OutlineNode> searchBusiness(String searchQuery) {
    final List<OutlineNode> nodes = [];
    uftbusinessMap.forEach((key, value) {
      final functionName = path.basenameWithoutExtension(value[0]);
      if (key.contains(searchQuery)) {
        nodes.add(
          OutlineNode(
            name: functionName,
            title: key,
            value: Constants.business,
            isDirectory: false, // 这些是叶子节点
            depth: 4,
            path: value[0], // 文件名
          ),
        );
      }
    });

    return nodes;
  }

  static List<OutlineNode> searchAtomServices(String searchQuery) {
    final List<OutlineNode> nodes = [];
    uftatomserviceMap.forEach((key, value) {
      final functionName = path.basenameWithoutExtension(value[0]);
      if (key.contains(searchQuery)) {
        nodes.add(
          OutlineNode(
            name: functionName,
            title: key,
            value: Constants.atomService,
            isDirectory: false, // 这些是叶子节点
            depth: 4,
            path: value[0], // 文件名
          ),
        );
      }
    });

    return nodes;
  }

  static List<OutlineNode> searchAtomFunctions(String searchQuery) {
    final List<OutlineNode> nodes = [];
    uftatomFunctionMap.forEach((key, value) {
      if (key.contains(searchQuery)) {
        nodes.add(
          OutlineNode(
            name: path.basenameWithoutExtension(value[0]),
            title: key,
            value: Constants.atomFunction,
            isDirectory: false, // 这些是叶子节点
            depth: 4,
            path: value[0], // 文件名
          ),
        );
      }
    });

    return nodes;
  }
}
