// outline_service.dart
import 'dart:io';
import 'package:flutter/services.dart';
import 'package:jieba_flutter/analysis/jieba_segmenter.dart';
import 'package:win_text_editor/framework/common/constants.dart';
import 'package:win_text_editor/modules/outline/models/outline_node.dart';
import 'package:win_text_editor/modules/outline/services/component_service.dart';
import 'package:win_text_editor/modules/outline/services/functions_service.dart';
import 'package:win_text_editor/modules/outline/services/std_field_service.dart';
import 'package:win_text_editor/modules/outline/services/uft_object_service.dart';
import 'package:xml/xml.dart';
import 'package:yaml/yaml.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';

class OutlineService {
  // 静态常量
  // ignore: constant_identifier_names
  static const List<String> REQUIRED_FILES = [
    'metadata/stdfield.stdfield',
    'metadata/stdobj.xml',
    'metadata/component.xml',
  ];

  // ignore: constant_identifier_names
  static const List<String> REQUIRED_DIRS = ['uftstructure', 'uftatom', 'uftbusiness', 'uftfactor'];

  // ignore: constant_identifier_names
  static const List<String> FILTERED_WORD_CLASSES = ['v', 'a', 'ad', 'f', 'd', 't', 'r'];

  //黑名单
  static List<String> _blackList = [];

  static OutlineNode? _searchNode;
  //结巴初始化标记
  static bool _isJiebaInitialized = false;
  static JiebaSegmenter? _segmenter;

  // 私有方法：初始化分词器
  static Future<void> _initJieba() async {
    if (!_isJiebaInitialized) {
      try {
        await JiebaSegmenter.init();
        _segmenter = JiebaSegmenter();
        _isJiebaInitialized = true;
      } catch (e) {
        Logger().error('初始化分词器失败: $e');
        throw Exception('分词器初始化失败');
      }
    }
  }

  // 私有方法：初始化黑名单
  static Future<void> _initBlackList() async {
    if (_blackList.isNotEmpty) return;

    try {
      final yamlString = await rootBundle.loadString('assets/config/words_classes.yaml');
      final yamlMap = loadYaml(yamlString);
      final blackListItems = (yamlMap['outline_name_black_list'] as List?)?.cast<String>() ?? [];

      _blackList =
          blackListItems
              .expand<String>(
                (item) =>
                    item.split(',').map((word) => word.trim()).where((word) => word.isNotEmpty),
              )
              .toList();

      Logger().info('加载黑名单成功: ${_blackList.length}个词');
    } catch (e) {
      Logger().error('加载黑名单失败: $e');
      _blackList = [];
    }
  }

  // 私有方法：初始化词性字典（优化版）
  static Future<Map<String, String>> _initWordClassDict() async {
    Map<String, String> wordClassDict = {};
    try {
      Logger().info('开始加载词性字典...');
      final dictContent = await rootBundle.loadString('assets/dict.txt');

      wordClassDict = Map.fromEntries(
        dictContent.split('\n').where((line) => line.trim().isNotEmpty).map((line) {
          final parts = line.trim().split(RegExp(r'\s+'));
          return (parts.length >= 3) ? MapEntry(parts[0], parts[2]) : null;
        }).whereType<MapEntry<String, String>>(),
      );
    } catch (e) {
      Logger().error('加载词性字典失败: $e');
    }
    return wordClassDict;
  }

  // 私有方法：解析stdfield文件
  static Future<List<String>> _parseChineseNames(String filePath) async {
    try {
      final content = await File(filePath).readAsString();
      return XmlDocument.parse(content)
          .findAllElements('items')
          .map((e) => e.getAttribute('chineseName') ?? '')
          .where((name) => name.isNotEmpty)
          .toList();
    } catch (e) {
      Logger().error('解析stdfield文件失败: $e');
      return [];
    }
  }

  // 私有方法：分词并统计词频
  static Future<Map<String, int>> _analyzeWords(List<String> chineseNames) async {
    await _initJieba();
    final wordFrequency = <String, int>{};

    Logger().info('开始分词，共有 ${chineseNames.length} 个中文名称');

    for (final name in chineseNames) {
      final tokens = _segmenter!.process(name, SegMode.SEARCH);
      for (final token in tokens) {
        final word = token.word.trim();
        if (word.length > 1 && !_blackList.contains(word)) {
          wordFrequency[word] = (wordFrequency[word] ?? 0) + 1;
        }
      }
    }

    Logger().info('分词完成，共找到 ${wordFrequency.length} 个有效词语');
    return wordFrequency;
  }

  // 公开方法：加载子节点
  static Future<void> loadChildren(String rootPath, OutlineNode dirNode) async {
    try {
      if (dirNode.depth == 1) {
        await StdFieldService.loadStdfields(rootPath, dirNode);
      } else {
        Logger().error("节点层次不支持: ${dirNode.depth}");
      }
    } catch (e) {
      Logger().error('加载子节点失败: $e');
      rethrow;
    }
  }

  // 公开方法：获取分词结果节点
  static Future<List<OutlineNode>> getWordNodes(String rootPath) async {
    if (rootPath.isEmpty) {
      throw Exception('根路径不能为空');
    }

    final wordClassDict = await _initWordClassDict();

    //初始化数据
    await Future.wait([
      _initBlackList(),
      ComponentService.initComponentFieldMap(rootPath),
      StdFieldService.initStdFieldMap(rootPath),
      UftObjectService.initUftObjectMap(rootPath),
      FunctionsService.initFunctionsMap(rootPath),
    ]);

    final stdfieldPath = '$rootPath/metadata/stdfield.stdfield';
    if (!await File(stdfieldPath).exists()) {
      throw Exception('stdfield文件不存在');
    }

    final chineseNames = await _parseChineseNames(stdfieldPath);
    if (chineseNames.isEmpty) {
      throw Exception('未找到有效的中文名称');
    }

    final wordFrequency = await _analyzeWords(chineseNames);
    final sortedWords = wordFrequency.entries.toList()..sort((a, b) => b.value.compareTo(a.value));

    return sortedWords
        .map(
          (entry) => OutlineNode(
            name: entry.key,
            value: '',
            title: '${entry.key}(${entry.value})',
            frequency: entry.value,
            isDirectory: true,
            depth: 1,
            wordClass: wordClassDict[entry.key],
          ),
        )
        .where((node) => !FILTERED_WORD_CLASSES.contains(node.wordClass))
        .toList();
  }

  // 公开方法：验证目录结构
  static Future<bool> validateDirectoryStructure(String rootPath) async {
    try {
      // 检查文件
      final fileChecks = await Future.wait(
        REQUIRED_FILES.map((path) => File('$rootPath/$path').exists()),
      );

      if (fileChecks.any((exists) => !exists)) {
        Logger().error('缺少必要文件');
        return false;
      }

      // 检查目录
      final dirChecks = await Future.wait(
        REQUIRED_DIRS.map((path) => Directory('$rootPath/$path').exists()),
      );

      if (dirChecks.any((exists) => !exists)) {
        Logger().error('缺少必要目录');
        return false;
      }

      return true;
    } catch (e) {
      Logger().error('验证目录结构时出错: $e');
      return false;
    }
  }

  static String _oldSearchQuery = "";

  static void applySearchFilter(OutlineNode root, String searchQuery) {
    if (_searchNode != null && _oldSearchQuery == searchQuery) {
      return;
    }

    _oldSearchQuery = searchQuery;

    //创建搜索词第一层节点
    _searchNode = OutlineNode(
      name: searchQuery,
      title: searchQuery,
      value: 'Search',
      isDirectory: true,
      isExpanded: true,
      depth: 1,
    );

    //搜索标准字段
    final List<OutlineNode> stdFieldNodes = StdFieldService.searchStdFields(searchQuery);
    _searchNode?.children.addAll(stdFieldNodes);

    final virtualNode = OutlineNode(
      name: "virtualNode",
      title: "【非字段匹配】",
      value: 'virtualNode',
      isDirectory: true,
      depth: 2,
      isExpanded: true,
    );

    //搜索UFT对象
    final List<OutlineNode> uftObjectNodes = UftObjectService.searchUftObjects(searchQuery);
    if (uftObjectNodes.isNotEmpty) {
      final uftObjectActionNode = OutlineNode(
        name: Constants.uftTable,
        title: 'UFT对象(${uftObjectNodes.length})',
        value: 'virtualNode',
        isDirectory: true,
        depth: 3,
      );
      uftObjectActionNode.children.addAll(uftObjectNodes);
      virtualNode.children.add(uftObjectActionNode);
    }

    //搜索组件
    final List<OutlineNode> componentNodes = ComponentService.searchComponents(searchQuery);
    if (componentNodes.isNotEmpty) {
      final componentActionNode = OutlineNode(
        name: Constants.component,
        title: '标准组件(${componentNodes.length})',
        value: 'virtualNode',
        isDirectory: true,
        depth: 3,
      );
      componentActionNode.children.addAll(componentNodes);
      virtualNode.children.add(componentActionNode);
    }

    //搜索函数
    final List<OutlineNode> businessNodes = FunctionsService.searchBusiness(searchQuery);
    if (businessNodes.isNotEmpty) {
      final businessActionNode = OutlineNode(
        name: Constants.business,
        title: "业务函数(${businessNodes.length})",
        value: 'virtualNode',
        isDirectory: true,
        depth: 3,
      );
      businessActionNode.children.addAll(businessNodes);
      virtualNode.children.add(businessActionNode);
    }

    //搜索函数
    final List<OutlineNode> atomServiceNodes = FunctionsService.searchAtomServices(searchQuery);
    if (atomServiceNodes.isNotEmpty) {
      final atomActionNode = OutlineNode(
        name: Constants.atomService,
        title: '原子服务(${atomServiceNodes.length})',
        value: 'virtualNode',
        isDirectory: true,
        depth: 3,
      );
      atomActionNode.children.addAll(atomServiceNodes);
      virtualNode.children.add(atomActionNode);
    }

    final List<OutlineNode> atomFunctionNodes = FunctionsService.searchAtomFunctions(searchQuery);
    if (atomFunctionNodes.isNotEmpty) {
      final atomActionNode = OutlineNode(
        name: Constants.atomFunction,
        title: '原子函数(${atomFunctionNodes.length})',
        value: 'virtualNode',
        isDirectory: true,
        depth: 3,
      );
      atomActionNode.children.addAll(atomFunctionNodes);
      virtualNode.children.add(atomActionNode);
    }

    if (virtualNode.children.isNotEmpty) {
      _searchNode?.children.add(virtualNode);
    }

    root.children.clear();
    root.isExpanded = false;

    //如果没有子节点，返回空列表
    if (_searchNode!.children.isNotEmpty) {
      root.title = "搜索结果";
      root.children.add(_searchNode!);
      root.isExpanded = true;
    } else {
      root.title = "无结果";
    }
  }
}
