import 'dart:io';

import 'package:win_text_editor/framework/common/constants.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/modules/outline/models/outline_node.dart';
import 'package:win_text_editor/modules/outline/services/component_service.dart';
import 'package:win_text_editor/modules/outline/services/functions_service.dart';
import 'package:win_text_editor/modules/outline/services/uft_object_service.dart';
import 'package:xml/xml.dart';

class StdFieldService {
  // ignore: constant_identifier_names
  static const Map<String, String> FIELD_ACTIONS = {
    Constants.uftTable: "UFT对象",
    Constants.component: "标准组件",
    Constants.business: "业务函数",
    Constants.atomService: "原子服务",
    Constants.atomFunction: "原子函数",
  };
  //字段索引
  static Map<String, String> stdfieldMap = {};

  //初始化标准字段
  static Future<void> initStdFieldMap(String rootPath) async {
    final stdfieldFile = File('$rootPath/metadata/stdfield.stdfield');
    if (!await stdfieldFile.exists()) {
      throw Exception('stdfield文件不存在');
    }

    stdfieldMap.clear();

    final content = await stdfieldFile.readAsString();
    final document = XmlDocument.parse(content);

    for (final item in document.findAllElements('items')) {
      final chineseName = item.getAttribute('chineseName');
      final name = item.getAttribute('name');
      final status = item.getAttribute('status');

      if (chineseName != null && name != null && status != '删除') {
        stdfieldMap[name] = chineseName;
      }
    }
  }

  // 加载字段操作
  static void _loadFieldActions(OutlineNode dirNode) {
    switch (dirNode.name) {
      case Constants.uftTable:
        UftObjectService.loadUftObject(dirNode.value, dirNode);
        break;
      case Constants.component:
        ComponentService.loadComponent(dirNode.value, dirNode);
        break;
      case Constants.business:
        FunctionsService.loadBusiness(dirNode.value, dirNode);
        break;
      case Constants.atomService:
        FunctionsService.loadAtomService(dirNode.value, dirNode);
        break;
      case Constants.atomFunction:
        FunctionsService.loadAtomFunction(dirNode.value, dirNode);
        break;
      default:
        Logger().error("操作节点类型不支持: ${dirNode.value}");
    }
  }

  // 加载标准字段
  static Future<void> loadStdfields(String rootPath, OutlineNode dirNode) async {
    dirNode.children.clear();
    Logger().info('搜索标准字段:${dirNode.name}');

    stdfieldMap.forEach((key, value) {
      if (value.contains(dirNode.name)) {
        final fieldNode = OutlineNode(
          name: key,
          title: '$value($key)',
          value: key,
          isDirectory: true,
          depth: 2,
        );
        _createActionNodes(fieldNode);

        fieldNode.isDirectory = fieldNode.children.isNotEmpty;
        if (fieldNode.isDirectory) {
          dirNode.children.insert(0, fieldNode);
        } else {
          dirNode.children.add(fieldNode);
        }
      }
    });
  }

  // 私有方法：创建操作节点
  static void _createActionNodes(OutlineNode parentNode) {
    for (var entry in FIELD_ACTIONS.entries) {
      final actionNode = OutlineNode(
        name: entry.key,
        value: parentNode.value,
        title: entry.value,
        isDirectory: true,
        depth: 3,
      );

      //加载子节点
      _loadFieldActions(actionNode);

      if (actionNode.children.isNotEmpty) {
        actionNode.title = '${entry.value}(${actionNode.children.length})';
        parentNode.children.add(actionNode);
      }
    }
  }

  static List<OutlineNode> searchStdFields(String searchQuery) {
    final List<OutlineNode> children = [];
    stdfieldMap.forEach((key, value) {
      if (searchQuery.contains(key) ||
          searchQuery.contains(value) ||
          key.contains(searchQuery) ||
          value.contains(searchQuery)) {
        final fieldNode = OutlineNode(
          name: key,
          title: '$value($key)',
          value: key,
          isDirectory: false,
          depth: 2,
        );
        _createActionNodes(fieldNode);
        if (fieldNode.children.isNotEmpty) {
          fieldNode.isDirectory = true;
          children.insert(0, fieldNode);
        } else {
          children.add(fieldNode);
        }
      }
    });
    return children;
  }
}
