import 'dart:async';

import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:win_text_editor/framework/common/constants.dart';
import 'package:win_text_editor/framework/controllers/file_provider.dart';
import 'package:win_text_editor/framework/services/file_path_manager.dart';
import 'package:win_text_editor/modules/outline/controllers/outline_provider.dart';
import 'package:win_text_editor/modules/outline/models/outline_node.dart';
import 'package:win_text_editor/shared/components/tree_view.dart';

class OutlineExplorer extends StatefulWidget {
  final Function(String, dynamic)? onFileDoubleTap;
  final Function(String)? onFolderDoubleTap;

  const OutlineExplorer({super.key, this.onFileDoubleTap, this.onFolderDoubleTap});

  @override
  State<OutlineExplorer> createState() => _OutlineExplorerState();
}

class _OutlineExplorerState extends State<OutlineExplorer> {
  final ScrollController _scrollController = ScrollController();
  final TextEditingController _searchController = TextEditingController();
  String _searchQuery = '';
  Timer? _searchDebounceTimer;

  @override
  void dispose() {
    _scrollController.dispose();
    _searchController.dispose();
    _searchDebounceTimer?.cancel();
    super.dispose();
  }

  // 过滤节点方法 - 递归版本
  List<OutlineNode> _filterNodes(OutlineProvider outlineProvider) {
    return outlineProvider.applyFilter(_searchQuery);
  }

  @override
  Widget build(BuildContext context) {
    final outlineProvider = Provider.of<OutlineProvider>(context);
    final fileProvider = Provider.of<FileProvider>(context); // 添加这行

    // 当 fileProvider 的 rootPath 变化时，会自动触发 rebuild
    _loadLastOpenedFolder(outlineProvider, fileProvider);

    return Column(
      children: [
        // 搜索框
        Padding(
          padding: const EdgeInsets.all(4.0),
          child: SizedBox(
            height: 30, // 设置固定高度
            child: TextField(
              controller: _searchController,
              style: const TextStyle(fontSize: 12), // 输入文字大小为10
              decoration: InputDecoration(
                hintText: '搜索...',
                hintStyle: const TextStyle(fontSize: 12), // 提示文字大小为10
                prefixIcon: const Icon(
                  Icons.search,
                  size: 14, // 搜索图标大小为10
                ),
                border: OutlineInputBorder(
                  borderRadius: BorderRadius.circular(4.0),
                  borderSide: const BorderSide(
                    color: Colors.grey, // 边框颜色设置为灰色
                    width: 1.0,
                  ),
                ),
                enabledBorder: OutlineInputBorder(
                  borderRadius: BorderRadius.circular(4.0),
                  borderSide: const BorderSide(
                    color: Colors.grey, // 未聚焦时边框颜色
                    width: 1.0,
                  ),
                ),
                focusedBorder: OutlineInputBorder(
                  borderRadius: BorderRadius.circular(4.0),
                  borderSide: const BorderSide(
                    color: Colors.grey, // 聚焦时边框颜色
                    width: 1.0,
                  ),
                ),
                contentPadding: const EdgeInsets.symmetric(
                  vertical: 4.0, // 调整垂直内边距以适应小字体
                  horizontal: 6.0,
                ),
              ),
              onChanged: (value) {
                _searchDebounceTimer?.cancel();
                _searchDebounceTimer = Timer(const Duration(milliseconds: 500), () {
                  if (mounted) setState(() => _searchQuery = value);
                });
              },
              onEditingComplete: () {
                if (mounted) setState(() => _searchQuery = _searchController.text);
              },
            ),
          ),
        ),
        Expanded(
          child: Scrollbar(
            controller: _scrollController,
            thumbVisibility: true,
            child: Stack(
              children: [
                SingleChildScrollView(
                  controller: _scrollController,
                  scrollDirection: Axis.horizontal,
                  child: Container(
                    color: Colors.white,
                    child: SizedBox(
                      width: 300,
                      child: TreeView(
                        nodes: _filterNodes(outlineProvider),
                        config: const TreeViewConfig(
                          showIcons: true,
                          lazyLoad: true,
                          showRefreshButton: true,
                        ),
                        onNodeTap: (node) => _handleNodeTap(context, node as OutlineNode),
                        onNodeDoubleTap:
                            (node) => _handleNodeDoubleTap(node as OutlineNode, fileProvider),
                        onRefresh: () async {
                          await outlineProvider.refreshOutlineTree();
                          setState(() => _searchController.text = '');
                        },
                      ),
                    ),
                  ),
                ),
                if (outlineProvider.isLoading)
                  Positioned.fill(
                    child: Container(
                      color: Colors.black.withAlpha(3),
                      child: const Center(
                        child: Column(
                          mainAxisSize: MainAxisSize.min,
                          children: [
                            CircularProgressIndicator(),
                            SizedBox(height: 10),
                            Text("加载中...", style: TextStyle(color: Colors.blue)),
                          ],
                        ),
                      ),
                    ),
                  ),
              ],
            ),
          ),
        ),
      ],
    );
  }

  Future<void> _handleNodeTap(BuildContext context, OutlineNode node) async {
    final outlineProvider = Provider.of<OutlineProvider>(context, listen: false);
    if (node.isDirectory) {
      setState(() {
        node.isExpanded = !node.isExpanded; // 触发更新
      });
      await outlineProvider.loadDirectoryContents(node);
    }
  }

  void _handleNodeDoubleTap(TreeNode node, FileProvider fileProvider) {
    final outlineNode = node as OutlineNode;
    if (outlineNode.depth < 3) {
      outlineNode.isExpanded = !outlineNode.isExpanded;
      return;
    }
    switch (outlineNode.value) {
      case Constants.uftTable:
      case Constants.business:
      case Constants.atomFunction:
      case Constants.atomService:
        if (widget.onFileDoubleTap != null) {
          widget.onFileDoubleTap!(outlineNode.path!, null);
        }
        break;
      case Constants.component:
        if (widget.onFileDoubleTap != null) {
          widget.onFileDoubleTap!(outlineNode.path!, outlineNode.name);
        }
        break;
    }
  }

  Future<void> _loadLastOpenedFolder(
    OutlineProvider outlineProvider,
    FileProvider? fileProvider,
  ) async {
    // 使用 fileProvider 的路径
    if (fileProvider?.rootPath != null && fileProvider!.rootPath!.isNotEmpty) {
      outlineProvider.setRootPath(fileProvider.rootPath!);
      return;
    }

    // 最后尝试从存储中加载
    final String? lastOpenedFolder = await FilePathManager.getLastOpenedFolder();
    if (lastOpenedFolder != null) {
      outlineProvider.setRootPath(lastOpenedFolder);
    }
  }
}
