// filter_controller.dart
import 'package:flutter/foundation.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/shared/models/template_node.dart';
import '../../../shared/base/safe_notifier.dart';

class FilterController extends SafeNotifier {
  String? _selectedFilterField;
  String? _selectedFilterOperator;
  String _filterValue = '';
  bool _isFilterValid = false;
  bool _isFilterApplied = false;
  List<String> _availableFields = [];

  // 仅按钮需要全局通知的Notifier
  final ValueNotifier<bool> _filterActionNotifier = ValueNotifier(false);

  String? get selectedFilterField => _selectedFilterField;
  String? get selectedFilterOperator => _selectedFilterOperator;
  String get filterValue => _filterValue;
  bool get isFilterValid => _isFilterValid;
  bool get isFilterApplied => _isFilterApplied;
  List<String> get availableFields => _availableFields;

  ValueNotifier<bool> get filterActionNotifier => _filterActionNotifier;

  set selectedFilterOperator(String? value) {
    _selectedFilterOperator = value;
    _updateFilterValidity();
    // 仅通知本地监听器（下拉框变化）
    safeNotify();
  }

  set filterValue(String value) {
    _filterValue = value;
    _updateFilterValidity();
    // 仅通知本地监听器（输入框变化）
    safeNotify();
  }

  set selectedFilterField(String? value) {
    // 当设置的值不在可用字段中时自动清空
    if (value != null && !_availableFields.contains(value)) {
      _selectedFilterField = null;
    } else {
      _selectedFilterField = value;
    }
    _updateFilterValidity();
    safeNotify();
  }

  void updateAvailableFields(List<String> newFields) {
    _availableFields = newFields;
    // 检查当前选中值是否仍然有效
    if (_selectedFilterField != null && !newFields.contains(_selectedFilterField)) {
      _selectedFilterField = null;
    }
    _updateFilterValidity();
    safeNotify();
  }

  void _updateFilterValidity() {
    _isFilterValid = _checkFilterValidity();
  }

  bool _checkFilterValidity() {
    if (_selectedFilterField == null || _selectedFilterOperator == null || _filterValue.isEmpty) {
      return false;
    }

    final operator = _selectedFilterOperator!;
    final value = _filterValue;

    if (operator == 'between' || operator == 'in') {
      final parts = value.split(',').map((e) => e.trim()).where((e) => e.isNotEmpty).toList();
      if (parts.length < 2) {
        return false;
      }

      if (operator == 'between') {
        for (var part in parts) {
          if (double.tryParse(part) == null) {
            return false;
          }
        }
      }
    } else if (operator == '>' || operator == '<') {
      return double.tryParse(value) != null;
    }

    return true;
  }

  //进行过滤处理，供外部获取调用，提供原始列表，获取过滤结果
  List<TemplateItem> doFilter(List<TemplateItem> templateItems) {
    if (selectedFilterField == null || selectedFilterOperator == null) {
      return templateItems;
    }

    Logger().info("过滤：$selectedFilterField $selectedFilterOperator $filterValue");

    List<TemplateItem> filteredItems = [];

    // 1. 找出所有匹配的主项
    final primaryMatches = templateItems.where((item) {
      final nodeName = item.xPath.split('/').last;
      if (nodeName == selectedFilterField) {
        return _matchesCondition(item.value);
      }
      return false;
    });

    // 2. 收集这些主项的ID（同实例的所有项共享相同ID）
    final matchedRowIds = primaryMatches.map((item) => item.rowId).toSet();

    // 3. 过滤出同实例的所有项
    filteredItems.cast();

    filteredItems.addAll(
      templateItems.where((item) {
        return matchedRowIds.contains(item.rowId);
      }).toList(),
    );

    _isFilterApplied = true;
    return filteredItems;
  }

  // 检查值是否符合条件
  bool _matchesCondition(String value) {
    if (selectedFilterOperator == null || filterValue.isEmpty) {
      return false;
    }
    switch (selectedFilterOperator!) {
      case '==':
        return value == filterValue;

      case '>':
        final numValue = double.tryParse(value);
        final filterNum = double.tryParse(filterValue);
        return numValue != null && filterNum != null && numValue > filterNum;

      case '<':
        final numValue = double.tryParse(value);
        final filterNum = double.tryParse(filterValue);
        return numValue != null && filterNum != null && numValue < filterNum;

      case 'between':
        final parts =
            filterValue.split(',').map((e) => e.trim()).where((e) => e.isNotEmpty).toList();

        if (parts.length < 2) return false;

        final lower = double.tryParse(parts[0]);
        final upper = double.tryParse(parts[1]);
        final numValue = double.tryParse(value);

        return numValue != null &&
            lower != null &&
            upper != null &&
            numValue >= lower &&
            numValue <= upper;

      case 'in':
        final filterValues =
            filterValue.split(',').map((e) => e.trim()).where((e) => e.isNotEmpty).toList();

        return filterValues.contains(value);

      default:
        return false;
    }
  }

  //执行过滤
  void applyFilter() {
    _isFilterApplied = true;
    _filterActionNotifier.value = !_filterActionNotifier.value;
  }

  // 取消过滤
  void clearFilter() {
    _isFilterApplied = false;
    _filterActionNotifier.value = !_filterActionNotifier.value;
  }
}
