import 'package:file_picker/file_picker.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/modules/outline/models/outline_node.dart';
import 'package:win_text_editor/shared/models/template_node.dart';
import 'package:win_text_editor/shared/base/base_content_controller.dart';
import 'package:xml/xml.dart' as xml;
import 'dart:io';
import 'tree_view_controller.dart';
import 'filter_controller.dart';
import 'grid_view_controller.dart';

class TemplateParserController extends BaseContentController {
  final TreeViewController treeController;
  final FilterController filterController;
  final GridViewController gridController;

  static const String modeByPath = "byPath";
  static const String modeByStruct = "byStruct";

  String _filePath = '';
  String? _errorMessage;
  String statisticsMode = modeByPath;

  String get filePath => _filePath;
  String? get errorMessage => _errorMessage;

  //---------------初始化方法----

  TemplateParserController()
    : treeController = TreeViewController(),
      filterController = FilterController(),
      gridController = GridViewController() {
    _setupCrossControllerCommunication();
  }

  //设置跨控制器状态协同
  void _setupCrossControllerCommunication() {
    // 1：树选择变化时更新过滤字段选项
    treeController.addListener(() {
      filterController.updateAvailableFields(treeController.selectedNodeNames);
      // 更新Grid的节点引用
      gridController.updateTreeNodesRef(treeController.treeNodes);
    });

    // 2：过滤条件变化时更新表格数据
    filterController.filterActionNotifier.addListener(() {
      if (filterController.isFilterApplied) {
        final List<TemplateItem> filteredItems = filterController.doFilter(
          gridController.templateItems,
        );
        gridController.applyFilter(filteredItems);
      } else {
        gridController.clearFilter();
      }
    });
  }

  //----------------业务入口方法-----

  void setStatisticsMode(String? value) {
    statisticsMode = value ?? modeByPath;
    _loadTemplateData();
    notifyListeners();
  }

  //widget调用入口：打开文件
  Future<void> pickFile() async {
    final result = await FilePicker.platform.pickFiles(
      type: FileType.custom,
      allowedExtensions: ['xml', '*'],
    );
    if (result != null) {
      _filePath = result.files.single.path!;
      notifyListeners(); // 通知 Consumer 刷新
      await _loadTemplateData();
    }
  }

  //执行框架回调入口：双击左侧资源管理文件
  Future<void> setFilePath(String path) async {
    _filePath = path;
    notifyListeners(); // 通知 Consumer 刷新
    await _loadTemplateData();
  }

  List<TemplateNode> _buildTreeNodes(
    xml.XmlElement element,
    String path, {
    required int depth,
    int repeatCount = 1,
  }) {
    // 根据统计模式决定如何构建节点
    if (statisticsMode == TemplateParserController.modeByStruct) {
      return _buildTreeNodesByStructure(element.document!);
    } else {
      return _buildTreeNodesByPath(element, path, depth: depth, repeatCount: repeatCount);
    }
  }

  //--------------------私有方法---------
  // 构建树节点
  List<TemplateNode> _buildTreeNodesByPath(
    xml.XmlElement element,
    String path, {
    required int depth,
    int repeatCount = 1,
  }) {
    final node = TemplateNode(
      path: path,
      name: element.qualifiedName,
      children: [],
      depth: depth,
      isExpanded: depth < 5, // 默认展开前两层
      isRepeated: repeatCount > 1,
      repeatCount: repeatCount,
    );

    // 添加当前元素的所有属性节点
    if (element.attributes.isNotEmpty) {
      node.children.addAll(
        element.attributes.map(
          (attr) => TemplateNode(
            path: '$path/@${attr.name.local}',
            name: '@${attr.qualifiedName}',
            children: [],
            depth: depth + 1,
            isAttribute: true,
          ),
        ),
      );
    }

    // 处理子元素节点（忽略文本节点）
    final childElements = element.children.whereType<xml.XmlElement>();
    final groupedChildren = <String, List<xml.XmlElement>>{};

    // 按元素名分组
    for (var child in childElements) {
      groupedChildren.putIfAbsent(child.name.local, () => []).add(child);
    }

    // 为每个唯一子元素创建节点
    groupedChildren.forEach((name, elements) {
      String path0 = '$path/${elements.first.name.local}';
      if (elements.length == 1) {
        // 单一节点直接添加（包含其所有属性）
        node.children.addAll(_buildTreeNodes(elements.first, path0, depth: depth + 1));
      } else {
        // 多个相同节点需要合并
        node.children.addAll(
          _buildTreeNodes(elements.first, path0, depth: depth + 1, repeatCount: elements.length),
        );
      }
    });

    return [node];
  }

  List<TemplateNode> _buildTreeNodesByStructure(xml.XmlDocument document) {
    final structureMap = <String, List<xml.XmlElement>>{};

    // 收集所有节点并按结构分组
    void collectNodes(xml.XmlElement element) {
      final structureKey = _getNodeStructureKey(element);
      structureMap.putIfAbsent(structureKey, () => []).add(element);

      // 继续遍历子元素
      for (var child in element.children.whereType<xml.XmlElement>()) {
        collectNodes(child);
      }
    }

    collectNodes(document.rootElement);

    // 构建树节点
    return structureMap.entries.map((entry) {
      final elements = entry.value;
      final firstElement = elements.first;
      final nodeName = firstElement.name.local;
      final attributes = firstElement.attributes.map((attr) => attr.name.local).toList()..sort();

      // 构建父节点
      final parentNode = TemplateNode(
        path: entry.key,
        name:
            attributes.isEmpty
                ? '$nodeName(${elements.length})'
                : '$nodeName(${attributes.join("|")})(${elements.length})',
        children: [],
        depth: 0,
        isExpanded: true,
        isRepeated: false,
        repeatCount: elements.length,
      );

      // 添加属性节点（仅显示名称）
      parentNode.children.addAll(
        firstElement.attributes.map(
          (attr) => TemplateNode(
            path: '${entry.key}/@${attr.name.local}',
            name: '@${attr.qualifiedName}',
            children: [],
            depth: 1,
            isAttribute: true,
          ),
        ),
      );

      // 添加文本内容节点（如果有）
      final hasTextContent = firstElement.children.whereType<xml.XmlText>().isNotEmpty;

      if (hasTextContent) {
        parentNode.children.add(
          TemplateNode(
            path: '${entry.key}/#text',
            name: '#text',
            children: [],
            depth: 1,
            isTextNode: true,
          ),
        );
      }

      return parentNode;
    }).toList();
  }

  // 获取节点结构标识键（仅节点名和属性名）
  String _getNodeStructureKey(xml.XmlElement element) {
    final attrNames = element.attributes.map((attr) => attr.name.local).toList()..sort();

    return '${element.name.local}|${attrNames.join(",")}';
  }

  // 修改 _loadTemplateData 方法
  Future<void> _loadTemplateData() async {
    try {
      _errorMessage = null;
      final file = File(_filePath);
      final content = await file.readAsString();
      final document = xml.XmlDocument.parse(content);

      // 更新树视图
      treeController.updateTreeNodes(
        statisticsMode == modeByStruct
            ? _buildTreeNodesByStructure(document)
            : _buildTreeNodesByPath(document.rootElement, document.rootElement.localName, depth: 0),
      );

      // 更新表格数据
      gridController.updateTemplateItems(_parseAllNodeValues(document));
    } catch (e) {
      _errorMessage = 'Failed to load XML: ${e.toString()}';
      Logger().error('XML加载错误$_errorMessage');
    }
  }

  // 获取节点结构标识符（属性名排序后拼接）
  List<TemplateItem> _parseAllNodeValues(xml.XmlDocument document) {
    final items = <TemplateItem>[];
    int id = 0;

    // 按结构统计模式的处理逻辑
    if (statisticsMode == modeByStruct) {
      final structureMap = <String, List<xml.XmlElement>>{};

      // 收集所有节点并按结构分组
      void collectNodes(xml.XmlElement element) {
        final structureKey = _getNodeStructureKey(element);
        structureMap.putIfAbsent(structureKey, () => []).add(element);

        for (var child in element.children.whereType<xml.XmlElement>()) {
          collectNodes(child);
        }
      }

      collectNodes(document.rootElement);

      // 为每个结构组创建数据项
      structureMap.forEach((structureKey, elements) {
        final firstElement = elements.first;
        final attributes = firstElement.attributes.map((attr) => attr.name.local).toList()..sort();
        int index = 0;
        // 添加属性数据项
        for (final attrName in attributes) {
          index = 0;
          for (final element in elements) {
            final attr = element.getAttributeNode(attrName);
            if (attr != null) {
              index++;
              items.add(
                TemplateItem(
                  id: id++,
                  rowId: "$structureKey/@$index",
                  xPath: '$structureKey/@$attrName',
                  value: attr.value,
                ),
              );
            }
          }
        }

        // 添加文本内容数据项
        index = 0;
        for (final element in elements) {
          final textNodes = element.children.whereType<xml.XmlText>();
          if (textNodes.isNotEmpty) {
            index++;
            items.add(
              TemplateItem(
                id: id++,
                rowId: "$structureKey/$index",
                xPath: "$structureKey/#text",
                value: textNodes.first.text,
              ),
            );
          }
        }
      });

      return items;
    }

    // 按路径统计模式的原始逻辑
    void traverse(xml.XmlElement element, String currentPath, int index) {
      // 处理属性
      for (final attr in element.attributes) {
        items.add(
          TemplateItem(
            id: id++,
            rowId: "$currentPath/@$index",
            xPath: '$currentPath/@${attr.name.local}',
            value: attr.value,
          ),
        );
      }

      // 处理文本节点
      final textNodes = element.children.whereType<xml.XmlText>();
      if (textNodes.isNotEmpty) {
        items.add(
          TemplateItem(
            id: id++,
            rowId: "$currentPath/@$index",
            xPath: currentPath,
            value: textNodes.first.text,
          ),
        );
      }

      // 处理子元素
      final childElements = element.children.whereType<xml.XmlElement>();
      final groupedChildren = <String, List<xml.XmlElement>>{};

      // 按元素名分组
      for (var child in childElements) {
        groupedChildren.putIfAbsent(child.name.local, () => []).add(child);
      }

      // 处理子元素（不再需要特殊处理重复元素）
      int i = 0;
      for (final child in element.childElements) {
        traverse(child, '$currentPath/${child.name.local}', i++);
      }
    }

    traverse(document.rootElement, document.rootElement.localName, 0);
    return items;
  }

  //-----------框架回调--
  @override
  void onOpenFile(String filePath, {dynamic appendArg}) {
    setFilePath(filePath);
  }

  @override
  void onOpenFolder(String folderPath) {
    // 不支持打开文件夹
  }

  @override
  void dispose() {
    treeController.dispose();
    filterController.dispose();
    gridController.dispose();
    super.dispose();
  }

  @override
  void onDropOutlineNode(OutlineNode node) {
    // TODO: implement onDropOutlineNode
  }
}
