import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:syncfusion_flutter_datagrid/datagrid.dart';
import 'package:win_text_editor/modules/template_parser/controllers/grid_view_controller.dart';
import 'package:win_text_editor/shared/components/my_grid_column.dart';
import 'package:win_text_editor/shared/models/template_node.dart';
import 'package:file_picker/file_picker.dart';
import 'dart:io';

class TemplateGridView extends StatelessWidget {
  const TemplateGridView({super.key});

  @override
  Widget build(BuildContext context) {
    return Consumer<GridViewController>(
      builder: (context, controller, _) {
        return GestureDetector(
          onSecondaryTapDown: (details) {
            _showContextMenu(context, details.globalPosition, controller);
          },
          child: _buildGridView(controller),
        );
      },
    );
  }

  Future<void> _showContextMenu(
    BuildContext context,
    Offset position,
    GridViewController controller,
  ) async {
    final renderBox = context.findRenderObject() as RenderBox;
    final localPosition = renderBox.globalToLocal(position);

    final result = await showMenu<String>(
      context: context,
      position: RelativeRect.fromLTRB(
        position.dx,
        position.dy,
        position.dx + renderBox.size.width - localPosition.dx,
        position.dy + renderBox.size.height - localPosition.dy,
      ),
      items: [const PopupMenuItem<String>(value: 'export', child: Text('导出(csv)'))],
    );

    if (result == 'export' && context.mounted) {
      try {
        await _exportToCsv(controller);
      } catch (e) {
        if (context.mounted) {
          ScaffoldMessenger.of(
            context,
          ).showSnackBar(SnackBar(content: Text('导出失败: ${e.toString()}')));
        }
      }
    }
  }

  Future<void> _exportToCsv(GridViewController controller) async {
    final selectedNodes = controller.getSelectedNodes();
    if (selectedNodes.isEmpty) return;

    // 直接从数据源获取数据
    final dataSource = _TemplateItemDataSource(
      rows: _buildDataRows(selectedNodes, controller.displayedItems),
      selectedNodes: selectedNodes,
    );

    // 构建表头（不包含序号列）
    String csvData = selectedNodes
        .map((node) => node.isAttribute ? node.name.substring(1) : node.name)
        .join('\t');
    csvData += '\n';

    // 填充数据（跳过第一列的序号）
    for (final row in dataSource.rows) {
      // 从第1列开始（跳过第0列的序号）
      for (int i = 1; i < row.getCells().length; i++) {
        csvData += row.getCells()[i].value.toString();
        if (i < row.getCells().length - 1) csvData += '\t';
      }
      csvData += '\n';
    }

    // 保存文件
    final filePath = await FilePicker.platform.saveFile(
      dialogTitle: '保存导出结果',
      fileName: 'template_results.csv',
      type: FileType.custom,
      allowedExtensions: ['csv'],
    );

    if (filePath != null) {
      await File(filePath).writeAsString(csvData);
    }
  }

  Widget _buildGridView(GridViewController controller) {
    final selectedNodes = controller.getSelectedNodes();

    if (selectedNodes.isEmpty) {
      return const Center(child: Text('请在左侧树中选择要显示的节点（勾选复选框）'));
    }

    final allItems = controller.displayedItems;
    final rows = _buildDataRows(selectedNodes, allItems);

    final dataSource = _TemplateItemDataSource(
      rows: rows,
      selectedNodes: selectedNodes,
      onCellValueChanged: (node, columnName, newValue) {
        // 处理数据变更
        controller.updateItemValue(xPath: columnName, newValue: newValue);
      },
    );

    final columns = <GridColumn>[
      ShortGridColumn(
        columnName: 'index',
        label: '序号',
        allowEditing: false, // 序号列不可编辑
      ),
      ...selectedNodes.map((node) {
        return MyGridColumn(
          columnName: node.path,
          label: node.isAttribute ? node.name.substring(1) : node.name,
        );
      }).toList(),
    ];

    return SfDataGrid(
      rowHeight: 32,
      headerRowHeight: 32,
      source: dataSource,
      columns: columns,
      gridLinesVisibility: GridLinesVisibility.both,
      headerGridLinesVisibility: GridLinesVisibility.both,
      columnWidthMode: ColumnWidthMode.fill,
      allowColumnsResizing: true,
      allowEditing: true, // 启用编辑
      editingGestureType: EditingGestureType.tap, // 点击编辑
    );
  }

  List<Map<String, dynamic>> _buildDataRows(
    List<TemplateNode> selectedNodes,
    List<TemplateItem> allItems,
  ) {
    final instanceMap = <String, Map<String, dynamic>>{};

    // 1. 先按实例分组
    for (final item in allItems) {
      // 2. 只填充选中的列
      if (selectedNodes.any((n) => n.path == item.xPath)) {
        final instanceId = item.rowId; // 或使用其他分组逻辑
        instanceMap.putIfAbsent(instanceId, () => {'_index': instanceMap.length + 1});
        instanceMap[instanceId]![item.xPath] = item.value;
      }
    }

    // 3. 确保所有选中列都存在
    return instanceMap.values.map((row) {
      for (final node in selectedNodes) {
        row.putIfAbsent(node.path, () => row[node.path] ?? '');
      }
      return row;
    }).toList();
  }
}

class _TemplateItemDataSource extends DataGridSource {
  final List<Map<String, dynamic>> _rows;
  final List<TemplateNode> selectedNodes;
  final Function(TemplateNode, String, dynamic)? onCellValueChanged;

  _TemplateItemDataSource({
    required List<Map<String, dynamic>> rows,
    required this.selectedNodes,
    this.onCellValueChanged,
  }) : _rows = rows;

  @override
  List<DataGridRow> get rows {
    return _rows.asMap().entries.map((entry) {
      final index = entry.key;
      final rowData = entry.value;

      return DataGridRow(
        cells: [
          DataGridCell<int>(columnName: 'index', value: index + 1),
          ...selectedNodes.map((node) {
            return DataGridCell<String>(
              columnName: node.path,
              value: rowData[node.path]?.toString() ?? '',
            );
          }).toList(),
        ],
      );
    }).toList();
  }

  @override
  DataGridRowAdapter? buildRow(DataGridRow row) {
    return DataGridRowAdapter(
      cells:
          row.getCells().map<Widget>((dataGridCell) {
            return Container(
              padding: const EdgeInsets.symmetric(horizontal: 8.0),
              alignment:
                  dataGridCell.columnName == 'index' ? Alignment.center : Alignment.centerLeft,
              child: Text(dataGridCell.value.toString()),
            );
          }).toList(),
    );
  }
}
