import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:win_text_editor/framework/controllers/tab_items_controller.dart';
import 'package:win_text_editor/modules/template_parser/controllers/template_parser_controller.dart';
import 'package:win_text_editor/modules/template_parser/widgets/filter_panel.dart';
import 'package:win_text_editor/modules/template_parser/widgets/grid_view.dart';
import 'package:win_text_editor/modules/template_parser/widgets/tree_view.dart';

class TemplateParserView extends StatefulWidget {
  final String tabId;
  const TemplateParserView({super.key, required this.tabId});

  @override
  State<TemplateParserView> createState() => _TemplateParserViewState();
}

class _TemplateParserViewState extends State<TemplateParserView> {
  late final TemplateParserController _controller;

  bool _isControllerFromTabManager = false;

  get tabManager => Provider.of<TabItemsController>(context, listen: false);

  @override
  void initState() {
    super.initState();

    final controllerFromManager = tabManager.getController(widget.tabId);
    if (controllerFromManager != null) {
      _controller = controllerFromManager;
      _isControllerFromTabManager = true;
    } else {
      _controller = TemplateParserController();
      _isControllerFromTabManager = false;
      tabManager.registerController(widget.tabId, _controller);
    }
  }

  @override
  void dispose() {
    if (!_isControllerFromTabManager) {
      _controller.dispose();
    }
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: [
        ChangeNotifierProvider.value(value: _controller),
        ChangeNotifierProvider.value(value: _controller.treeController),
        ChangeNotifierProvider.value(value: _controller.filterController),
        ChangeNotifierProvider.value(value: _controller.gridController),
      ],
      child: Padding(
        padding: const EdgeInsets.all(8.0),
        child: Column(
          children: [
            _buildFilePathInput(),
            const SizedBox(height: 8),
            Expanded(child: _buildMainContent()),
          ],
        ),
      ),
    );
  }

  Widget _buildFilePathInput() {
    return Consumer<TemplateParserController>(
      builder: (context, controller, _) {
        return Row(
          children: [
            // 新增统计模式下拉框
            SizedBox(
              width: 150,
              child: DropdownButtonFormField<String>(
                value: 'byPath', // 默认值
                decoration: const InputDecoration(
                  labelText: '统计模式',
                  border: OutlineInputBorder(),
                  contentPadding: EdgeInsets.symmetric(horizontal: 8, vertical: 12),
                ),
                items: const [
                  DropdownMenuItem(value: 'byPath', child: Text('按节点路径')),
                  DropdownMenuItem(value: 'byStruct', child: Text('按节点结构')),
                ],
                onChanged: (value) {
                  controller.setStatisticsMode(value);
                },
              ),
            ),
            const SizedBox(width: 8),
            // 原有的文件路径输入框
            Expanded(
              child: TextField(
                decoration: InputDecoration(
                  labelText: 'XML File',
                  hintText: 'Select an XML file',
                  suffixIcon: IconButton(
                    icon: const Icon(Icons.folder_open),
                    onPressed: controller.pickFile,
                  ),
                  border: const OutlineInputBorder(),
                  errorText: controller.errorMessage,
                ),
                controller: TextEditingController(text: controller.filePath),
                readOnly: true,
              ),
            ),
          ],
        );
      },
    );
  }

  Widget _buildMainContent() {
    return Consumer<TemplateParserController>(
      builder: (context, controller, _) {
        if (controller.errorMessage != null) {
          return Center(child: Text(controller.errorMessage!));
        }

        return Row(
          children: [
            SizedBox(
              width: MediaQuery.of(context).size.width * 0.3,
              child: Column(
                children: [
                  const Expanded(flex: 2, child: Card(child: TemplateTreeView())),
                  const SizedBox(height: 8),
                  SizedBox(
                    height: 340,
                    child: Card(
                      child: TemplateFilterPanel(controller: controller.filterController),
                    ),
                  ),
                ],
              ),
            ),
            const SizedBox(width: 8),
            const Expanded(child: Card(child: TemplateGridView())),
          ],
        );
      },
    );
  }
}
