// memory_table_service.dart

import 'dart:io';

import 'package:collection/collection.dart';
import 'package:win_text_editor/modules/memory_table/models/memory_table.dart';
import 'package:win_text_editor/modules/uft_component/models/uft_component.dart';
import 'package:win_text_editor/shared/data/std_fields_cache.dart';
import 'package:win_text_editor/shared/models/std_filed.dart';

import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:xml/xml.dart' as xml;

class UftComponentService {
  static final Logger _logger = Logger();
  static List<UftComponent> _components = [];

  static Future<UftComponent?> getUftComponent(String filePath, String componentName) async {
    if (_components.isNotEmpty) {
      final com = _components.firstWhereOrNull((c) => c.name == componentName);
      if (com != null) return com;
    }
    return parseComponentFile(
      filePath,
    ).then((l) => l.firstWhereOrNull((c) => c.name == componentName));
  }

  static Future<List<UftComponent>> parseComponentFile(
    String filePath, {
    String? filterName,
  }) async {
    try {
      if (_components.isNotEmpty && filterName == null) {
        _components.firstWhereOrNull((c) => c.isSelected)?.isSelected = false;
        return _components;
      }
      // 1. Check file extension
      File? file =
          filePath.toLowerCase().endsWith('component.xml')
              ? File(filePath)
              : await findMetadataFile(filePath);

      if (file == null) {
        throw const FormatException("没有找到标准组件文件:component.xml");
      }

      _components.clear();

      // 2. 查找 metadata 目录和 stdfield.stfield 文件
      await StdFieldsCache.loadForFile(filePath);

      // 3. Read and parse structure file content
      final content = await file.readAsString();
      _logger.info('加载标准组件,过滤词:$filterName');
      final document = xml.XmlDocument.parse(content);
      final componentNodes = document.findAllElements('items');

      if (componentNodes.isEmpty) {
        throw const FormatException("文件格式错误：缺少items节点");
      }

      // 4. 解析组件列表
      _components = <UftComponent>[];
      int id = 0;
      for (var node in componentNodes) {
        if (node.findElements("items").isEmpty) continue;
        id++;
        final name = node.getAttribute('name') ?? '';
        final chineseName = node.getAttribute('chineseName') ?? '';

        final fields = <Field>[];
        List<Index>? indexes = [];
        int index = 1;

        // 5.解析是否有索引
        final indexNodes = node.findElements('indexs');
        if (indexNodes.isNotEmpty) {
          final attrnames = <String>[];
          for (final indexNode in indexNodes) {
            final indexName = indexNode.getAttribute('name') ?? '';

            //5.1 解析索引字段
            final indexFieldNodes = indexNode.findElements('items');
            for (final fieldNode in indexFieldNodes) {
              final fieldName = fieldNode.getAttribute('attrname') ?? '';
              if (fieldName.isEmpty) continue; // 跳过没有名称的字段
              attrnames.add(fieldName);
            }

            indexes.add(Index(indexName, true, attrnames.join(','), '', false));
            //将索引添加到字段列表中
            // fields.add(Field((index++).toString(), indexName, '索引', 'COMPONENT_INDEX'));
          }
        }

        // 6. 解析字段列表
        final childNodes = node.children;

        for (final childNode in childNodes) {
          if (childNode is xml.XmlElement) {
            if (childNode.name.local != 'items') continue; // 只处理items节点
          }
          final name = childNode.getAttribute('name') ?? '';
          if (name.isEmpty) continue; // 跳过没有名称的字段

          // 尝试从缓存获取标准字段信息
          final stdField = StdFieldsCache.getData(name);
          fields.add(
            Field(
              (index++).toString(), // 序号
              name, // 名称
              stdField?.chineseName ?? '', // 中文名
              stdField?.dateType ?? '', // 类型
            ),
          );
        }

        _components.add(
          UftComponent(
            id: id,
            name: name,
            chineseName: chineseName,
            fields: fields,
            indexes: indexes,
          ),
        );
      }

      if (filterName != null && filterName.isNotEmpty) {
        return _components.where((com) => com.name == filterName).toList();
      }
      return _components;
    } on xml.XmlParserException catch (e) {
      _logger.error("XML解析错误: ${e.message}");
      rethrow;
    } catch (e) {
      _logger.error("处理文件时发生错误: $e");
      rethrow;
    }
  }

  static Future<File?> findMetadataFile(String filePath) async {
    Directory currentDir = File(filePath).parent;
    const targetDirName = 'metadata';
    const targetFileName = 'component.xml';

    // 向上查找 metadata 目录
    while (true) {
      final metadataDir = Directory('${currentDir.path}/$targetDirName');
      if (await metadataDir.exists()) {
        final stdFieldFile = File('${metadataDir.path}/$targetFileName');
        if (await stdFieldFile.exists()) {
          return stdFieldFile;
        } else {
          Logger().error("没找到标准组件文件 $targetFileName");
          return null;
        }
      }

      // 到达根目录时停止
      if (currentDir.path == currentDir.parent.path) {
        Logger().error("没有找到元数据目录 $targetDirName");
        return null;
      }

      currentDir = currentDir.parent;
    }
  }
}
