import 'dart:async';

import 'package:file_picker/file_picker.dart';
import 'package:flutter/foundation.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/modules/outline/models/outline_node.dart';
import 'package:win_text_editor/modules/xml_search/models/search_result.dart';
import 'package:win_text_editor/modules/xml_search/services/xml_search_service.dart';
import 'package:win_text_editor/shared/base/base_content_controller.dart';

class XmlSearchController extends BaseContentController {
  String _searchDirectory = '';
  String _searchQuery = '';
  String nodePath = '';
  String attributeName = '';
  bool _isSearching = false;

  final List<SearchResult> _results = [];
  final XmlSearchService _searchService = XmlSearchService();

  List<SearchResult> get results => List.unmodifiable(_results); // 返回不可修改的副本
  String get searchDirectory => _searchDirectory;
  String get searchQuery => _searchQuery;
  bool get isSearching => _isSearching;

  Timer? _searchDebounce;

  final ValueNotifier<bool> refreshNotifier = ValueNotifier(false);

  set errorMessage(String value) {
    Logger().error('打开文件出错:$value');
  }

  set searchQuery(String value) {
    _searchDebounce?.cancel();
    _searchDebounce = Timer(const Duration(milliseconds: 500), () {
      _searchQuery = value;
      notifyListeners();
    });
  }

  void removeResult(SearchResult result) async {
    try {
      Logger().debug('删除前结果数: ${_results.length}');
      await _searchService.removeNodes(searchDirectory, nodePath, attributeName, [result]);
      _results.removeWhere((r) => r.hashCode == result.hashCode); // 使用hashCode确保正确删除
      Logger().debug('删除后结果数: ${_results.length}');
      notifyListeners();
      Future.delayed(Duration.zero, notifyListeners);
    } catch (e) {
      Logger().error('删除失败: $e');
      rethrow;
    }
  }

  Future<void> pickFile() async {
    final result = await FilePicker.platform.pickFiles(
      type: FileType.custom,
      allowedExtensions: ['xml', '*'],
    );
    if (result != null) {
      _searchDirectory = result.files.single.path!;
      notifyListeners(); // 通知 Consumer 刷新
    }
  }

  Future<void> executeSearching() async {
    Logger().info("开始搜索文件:$_searchDirectory");
    if (_searchDirectory.isEmpty ||
        _searchQuery.isEmpty ||
        nodePath.isEmpty ||
        attributeName.isEmpty) {
      Logger().error("所有条件都不能为空。");
      return;
    }

    _isSearching = true;
    notifyListeners();

    try {
      _results.clear();
      _results.addAll(
        await _searchService.searchFromDirectory(
          directory: _searchDirectory,
          nodeName: nodePath,
          attributeName: attributeName,
          queryContent: searchQuery,
        ),
      );
      notifyListeners();
      // 添加延迟二次通知
      Future.delayed(Duration.zero, notifyListeners);
    } catch (e) {
      Logger().error("搜索文件出错:$e");
    } finally {
      _isSearching = false;
      notifyListeners();
    }
  }

  void refresh() {
    notifyListeners();
  }

  set searchDirectory(String value) {
    _searchDirectory = value;
    notifyListeners();
  }

  Future<void> pickDirectory() async {
    final dir = await FilePicker.platform.getDirectoryPath();
    if (dir != null) {
      searchDirectory = dir;
    }
  }

  @override
  void onOpenFile(String filePath, {dynamic appendArg}) {
    searchDirectory = filePath;
  }

  @override
  void onOpenFolder(String folderPath) {
    //不支持
  }

  void cancelSearching() {}

  void toggleSelectAll(bool select) {
    for (var result in results) {
      result.isSelected = select;
    }
  }

  List<SearchResult> getSelectedResults() {
    return results.where((r) => r.isSelected).toList();
  }

  Future<void> batchRemoveNodes() async {
    if (_searchDirectory.isEmpty) {
      throw Exception('请先选择XML文件');
    }

    final selectedResults = getSelectedResults();
    if (selectedResults.isEmpty) {
      return;
    }

    _isSearching = true;
    notifyListeners();

    try {
      // 先删除文件中的节点
      await _searchService.removeNodes(_searchDirectory, nodePath, attributeName, selectedResults);

      // 再从结果列表中移除
      _results.removeWhere((r) => r.isSelected);
    } finally {
      _isSearching = false;
      notifyListeners();
    }
  }

  @override
  void onDropOutlineNode(OutlineNode node) {
    // TODO: implement onDropOutlineNode
  }
}
