// xml_search_service.dart
import 'dart:io';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:xml/xml.dart' as xml;
import 'package:win_text_editor/modules/xml_search/models/search_result.dart';

class XmlSearchService {
  Future<List<SearchResult>> searchFromDirectory({
    required String directory,
    required String nodeName,
    required String attributeName,
    required String queryContent,
  }) async {
    final results = <SearchResult>[];
    try {
      final searchValues =
          queryContent
              .split(RegExp(r'[\n,]'))
              .map((e) => e.trim())
              .where((e) => e.isNotEmpty)
              .toList();
      if (searchValues.isEmpty) {
        searchValues.add(queryContent);
      }

      final file = File(directory);
      final content = await file.readAsString();
      final document = xml.XmlDocument.parse(content);

      int rowNum = 1;

      // 2. Search for nodes with specified name and attribute
      for (int i = 0; i < searchValues.length; i++) {
        final nodes = document.findAllElements(nodeName);
        int index = 0;
        for (final node in nodes) {
          final attributeValue = node.getAttribute(attributeName);
          if (attributeValue != null && attributeValue == searchValues[i]) {
            results.add(
              SearchResult(rowNum: rowNum++, attributeValue: attributeValue, index: index++),
            );
          }
        }
      }
      Logger().info("共发现记录 ${rowNum - 1} 条");
    } catch (e) {
      Logger().error('xmlSearchService.searchFromDirectory方法执行出错: $e');
    }

    return results;
  }

  Future<void> removeNodes(
    String directory,
    String nodeName,
    String attributeName,
    List<SearchResult> resultList,
  ) async {
    try {
      // 1. 打开并解析 XML 文件
      final file = File(directory);
      final content = await file.readAsString();
      final document = xml.XmlDocument.parse(content);
      String newContent = content;

      // 2. 查找所有匹配的节点,反向删除（避免删除后序号为空）
      for (int i = resultList.length - 1; i >= 0; i--) {
        SearchResult result = resultList[i];
        final nodes =
            document.findAllElements(nodeName).where((node) {
              final attributeValue = node.getAttribute(attributeName);
              return attributeValue == result.attributeValue;
            }).toList();

        // 3. 检查是否存在指定序号的节点
        if (result.index >= 0 && result.index < nodes.length) {
          final nodeToRemove = nodes[result.index];

          // 4. 删除节点
          nodeToRemove.parent?.children.remove(nodeToRemove);

          // 5. 保存修改后的 XML 文件
          newContent = document.toXmlString(pretty: true);

          Logger().info(
            '成功删除节点: $nodeName[$attributeName="${result.attributeValue}"][${result.index}]',
          );
        } else {
          Logger().warning('未找到序号为 ${result.index} 的匹配节点');
        }
      }
      await file.writeAsString(newContent);
    } catch (e) {
      Logger().error('删除节点时出错: $e');
      rethrow; // 重新抛出异常以便上层处理
    }
  }
}
