import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:win_text_editor/modules/xml_search/controllers/xml_search_controller.dart';
import 'package:win_text_editor/shared/components/text_editor.dart';

class ConditionSetting extends StatefulWidget {
  const ConditionSetting({super.key});

  @override
  State<ConditionSetting> createState() => _ConditionSettingState();
}

class _ConditionSettingState extends State<ConditionSetting> {
  bool _isSearching = false;
  late TextEditingController _nodePathController;
  late TextEditingController _attributeNameController;

  @override
  void initState() {
    super.initState();
    final controller = context.read<XmlSearchController>();
    _nodePathController = TextEditingController(text: controller.nodePath);
    _attributeNameController = TextEditingController(text: controller.attributeName);
  }

  @override
  void dispose() {
    _nodePathController.dispose();
    _attributeNameController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final controller = context.watch<XmlSearchController>();

    return Card(
      child: Padding(
        padding: const EdgeInsets.all(16.0),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: [
            const Text('搜索规则设置：', style: TextStyle(fontSize: 14, fontWeight: FontWeight.bold)),
            const SizedBox(height: 16),

            // 规则输入表单
            TextField(
              controller: _nodePathController,
              decoration: const InputDecoration(
                labelText: '节点名称',
                hintText: '如: business:Service',
                border: OutlineInputBorder(),
              ),
              onChanged: (value) {
                controller.nodePath = value;
              },
            ),
            const SizedBox(height: 12),
            TextField(
              controller: _attributeNameController,
              decoration: const InputDecoration(
                labelText: '属性名称',
                hintText: '如: chineseName 或 name',
                border: OutlineInputBorder(),
              ),
              onChanged: (value) {
                controller.attributeName = value;
              },
            ),

            const SizedBox(height: 12),

            SizedBox(
              height: 360,
              child: TextEditor(
                tabId: 'search_content_${controller.hashCode}',
                title: '[列表以\\n或,分隔]',
                initialContent: controller.searchQuery, // 绑定到控制器的状态
                onContentChanged: (content) {
                  controller.searchQuery = content; // 实时同步内容到控制器
                },
              ),
            ),
            const SizedBox(height: 16),
            // 操作按钮行
            Row(
              children: [
                Expanded(
                  child: ElevatedButton.icon(
                    icon: const Icon(Icons.play_arrow),
                    label: const Text('开始'),
                    onPressed: _isSearching ? null : () => _startSearching(controller),
                  ),
                ),
                const SizedBox(width: 12),
                Expanded(
                  child: ElevatedButton.icon(
                    icon: const Icon(Icons.stop, color: Colors.red),
                    label: const Text('停止', style: TextStyle(color: Colors.red)),
                    onPressed: _isSearching ? _stopSearch : null,
                  ),
                ),
              ],
            ),
            const SizedBox(height: 12),
            ElevatedButton.icon(
              icon: const Icon(Icons.delete_forever, color: Colors.red),
              label: const Text('批量删除节点', style: TextStyle(color: Colors.red)),
              onPressed: () => _batchRemove(controller),
              style: ElevatedButton.styleFrom(
                minimumSize: const Size.fromHeight(48), // 设置按钮高度
              ),
            ),
          ],
        ),
      ),
    );
  }

  Future<void> _startSearching(XmlSearchController controller) async {
    if (controller.searchDirectory.isEmpty) {
      ScaffoldMessenger.of(context).showSnackBar(const SnackBar(content: Text('请先选择搜索文件')));
      return;
    }

    if (controller.nodePath.isEmpty) {
      ScaffoldMessenger.of(context).showSnackBar(const SnackBar(content: Text('请先设置节点名称')));
      return;
    }

    if (controller.attributeName.isEmpty) {
      ScaffoldMessenger.of(context).showSnackBar(const SnackBar(content: Text('请先设置属性名称')));
      return;
    }

    if (controller.searchQuery.isEmpty) {
      ScaffoldMessenger.of(context).showSnackBar(const SnackBar(content: Text('请先设置搜索内容')));
      return;
    }

    setState(() => _isSearching = true);
    try {
      await controller.executeSearching();
    } finally {
      if (mounted) {
        setState(() => _isSearching = false);
      }
    }
  }

  void _stopSearch() {
    // 这里需要确保控制器中有取消提取的逻辑
    final controller = Provider.of<XmlSearchController>(context, listen: false);
    // 假设控制器中有cancelExtraction方法
    controller.cancelSearching();
    setState(() => _isSearching = false);
  }

  Future<void> _batchRemove(XmlSearchController controller) async {
    final selectedResults = controller.getSelectedResults();
    if (selectedResults.isEmpty) {
      if (mounted) {
        ScaffoldMessenger.of(context).showSnackBar(const SnackBar(content: Text('请先选择要删除的节点')));
      }
      return;
    }

    bool confirm =
        await showDialog<bool>(
          context: context,
          builder:
              (context) => AlertDialog(
                title: const Text('确认批量删除'),
                content: Text('确定要删除选中的 ${selectedResults.length} 个节点吗？'),
                actions: [
                  TextButton(
                    onPressed: () => Navigator.pop(context, false),
                    child: const Text('取消'),
                  ),
                  TextButton(
                    onPressed: () => Navigator.pop(context, true),
                    child: const Text('确认', style: TextStyle(color: Colors.red)),
                  ),
                ],
              ),
        ) ??
        false;

    if (confirm && mounted) {
      setState(() => _isSearching = true);
      try {
        await controller.batchRemoveNodes();
        if (mounted) {
          ScaffoldMessenger.of(
            context,
          ).showSnackBar(SnackBar(content: Text('成功删除 ${selectedResults.length} 个节点')));
        }
      } catch (e) {
        if (mounted) {
          ScaffoldMessenger.of(
            context,
          ).showSnackBar(SnackBar(content: Text('删除失败: ${e.toString()}')));
        }
      } finally {
        if (mounted) {
          setState(() => _isSearching = false);
        }
      }
    }
  }
}
