import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:pluto_grid/pluto_grid.dart';
import 'package:path/path.dart' as path;

import 'package:file_picker/file_picker.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/modules/xml_search/models/search_result.dart';
import 'dart:io';

import 'package:win_text_editor/modules/xml_search/controllers/xml_search_controller.dart';
import 'package:win_text_editor/shared/components/my_pluto_column.dart';
import 'package:win_text_editor/shared/components/my_pluto_configuration.dart';

class ResultsView extends StatefulWidget {
  const ResultsView({super.key});

  @override
  State<ResultsView> createState() => _ResultsViewState();
}

class _ResultsViewState extends State<ResultsView> {
  late final XmlSearchController controller;
  PlutoGridStateManager? stateManager;

  @override
  void initState() {
    super.initState();
    controller = context.read<XmlSearchController>();
  }

  @override
  Widget build(BuildContext context) {
    // 直接使用Consumer而不是ValueListenableBuilder
    return Consumer<XmlSearchController>(
      builder: (context, controller, child) {
        return Card(
          child: GestureDetector(
            onSecondaryTapDown: (details) {
              _showContextMenu(context, details.globalPosition, controller);
            },
            child: _buildResultGrid(controller),
          ),
        );
      },
    );
  }

  Future<void> _showContextMenu(
    BuildContext context,
    Offset position,
    XmlSearchController controller,
  ) async {
    final renderBox = context.findRenderObject() as RenderBox;
    final localPosition = renderBox.globalToLocal(position);

    final result = await showMenu<String>(
      context: context,
      position: RelativeRect.fromLTRB(
        position.dx,
        position.dy,
        position.dx + renderBox.size.width - localPosition.dx,
        position.dy + renderBox.size.height - localPosition.dy,
      ),
      items: [const PopupMenuItem<String>(value: 'exportAll', child: Text('导出(csv)'))],
    );

    if (result != null && result.startsWith('export') && context.mounted) {
      try {
        await _exportToCsv(controller, result);
      } catch (e) {
        if (context.mounted) {
          ScaffoldMessenger.of(
            context,
          ).showSnackBar(SnackBar(content: Text('导出失败: ${e.toString()}')));
        }
      }
    }
  }

  Future<void> _exportToCsv(XmlSearchController controller, String? exportType) async {
    String csvData = '搜索值\t节点\n';
    for (var result in controller.results) {
      csvData += '${path.basename(result.attributeValue)}\t${result.index}\n';
    }

    final filePath = await FilePicker.platform.saveFile(
      dialogTitle: '保存导出结果',
      fileName: 'search_results.csv',
      type: FileType.custom,
      allowedExtensions: ['csv'],
    );

    if (filePath != null) {
      final file = File(filePath);
      await file.writeAsString(csvData);
    }
  }

  Widget _buildResultGrid(XmlSearchController controller) {
    return PlutoGrid(
      key: ValueKey(controller.results.hashCode),
      configuration: MyPlutoGridConfiguration(),
      columns: _buildColumns(),
      mode: PlutoGridMode.normal,
      rows: controller.results.isEmpty ? [] : _buildRows(controller),
      noRowsWidget: const Center(child: Text('没有搜索结果')),

      onLoaded: (PlutoGridOnLoadedEvent event) {
        stateManager ??= event.stateManager;
      },

      onRowChecked: (PlutoGridOnRowCheckedEvent e) {
        if (e.isAll) {
          controller.toggleSelectAll(e.isChecked!);
        } else {
          final result = e.row?.cells['action']!.value as SearchResult;
          result.isSelected = e.isChecked!;
        }
      },
    );
  }

  List<PlutoRow> _buildRows(XmlSearchController controller) {
    return controller.results.asMap().entries.map((entry) {
      final index = entry.key;
      final result = entry.value;
      return PlutoRow(
        key: ValueKey('${result.hashCode}_$index'), // 复合key确保唯一性
        cells: {
          'rowNum': PlutoCell(value: index + 1),
          'content': PlutoCell(value: result.attributeValue),
          'index': PlutoCell(value: result.index),
          'action': PlutoCell(value: result),
        },
      );
    }).toList();
  }

  List<PlutoColumn> _buildColumns() {
    return [
      MyPlutoColumn(title: '#', field: 'rowNum', width: 60, enableRowChecked: true),
      MyPlutoColumn(
        title: '搜索内容',
        field: 'content',
        type: PlutoColumnType.text(),
        editable: true,
        autoSize: true,
      ),
      MyPlutoColumn(title: '节点序号', field: 'index'),
      MyPlutoColumn(
        title: '操作',
        field: 'action',
        renderer: (rendererContext) {
          final result = rendererContext.row.cells['action']!.value as SearchResult;
          return Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              IconButton(
                icon: const Icon(Icons.delete_forever, size: 18, color: Colors.red),
                onPressed: () => _showDeleteConfirmation(context, rendererContext.row, result),
              ),
            ],
          );
        },
      ),
    ];
  }

  Future<void> _showDeleteConfirmation(
    BuildContext context,
    PlutoRow row,
    SearchResult result,
  ) async {
    final confirmed =
        await showDialog<bool>(
          context: context,
          builder:
              (context) => AlertDialog(
                title: const Text('确认删除'),
                content: Text('确定要删除 ${result.attributeValue} 吗？'),
                actions: [
                  TextButton(
                    onPressed: () => Navigator.pop(context, false),
                    child: const Text('取消'),
                  ),
                  TextButton(
                    onPressed: () => Navigator.pop(context, true),
                    child: const Text('删除'),
                  ),
                ],
              ),
        ) ??
        false;

    if (confirmed && context.mounted) {
      try {
        final controller = context.read<XmlSearchController>();

        // 1. 从数据源删除
        controller.removeResult(result);

        Logger().info('已删除节点: ${result.attributeValue}');

        ScaffoldMessenger.of(
          context,
        ).showSnackBar(SnackBar(content: Text('已删除: ${result.attributeValue}')));
      } catch (e) {
        Logger().error('删除失败: $e');
        if (context.mounted) {
          ScaffoldMessenger.of(
            context,
          ).showSnackBar(SnackBar(content: Text('删除失败: ${e.toString()}')));
        }
      }
    }
  }
}
