import 'package:flutter/material.dart';
import 'package:syncfusion_flutter_datagrid/datagrid.dart';
import 'package:win_text_editor/shared/base/selectable_data_source.dart';
import 'package:win_text_editor/shared/base/selectable_item.dart';
import 'package:win_text_editor/shared/base/my_sf_data_source.dart';

// ignore: must_be_immutable
class MySfDataGrid<T extends SelectableItem> extends StatelessWidget {
  final MySfDataSource<T> source;
  final Function(int index, bool isSelected)? onSelectionChanged;
  final List<GridColumn> columns;
  final DataGridController? controller;
  bool selectable = false;

  MySfDataGrid({
    super.key,
    required this.source,
    required this.columns,
    this.onSelectionChanged,
    this.controller,
    this.selectable = false,
  });

  Widget _buildCheckboxHeader(BuildContext context, SelectableDataSource<T> dataSource) {
    final allSelected =
        dataSource.items.isNotEmpty && dataSource.items.every((item) => item.isSelected);
    final someSelected =
        dataSource.items.isNotEmpty &&
        dataSource.items.any((item) => item.isSelected) &&
        !allSelected;

    return Container(
      alignment: Alignment.center,
      color: Colors.grey[200],
      child: Transform.scale(
        scale: 0.75,
        child: Checkbox(
          value: allSelected,
          tristate: someSelected,
          onChanged: (value) => dataSource.toggleAllSelection(value ?? false),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Card(
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Expanded(
            child: LayoutBuilder(
              builder: (context, constraints) {
                return SizedBox(
                  width: constraints.maxWidth,
                  child: SfDataGrid(
                    rowHeight: 32,
                    headerRowHeight: 32,
                    source: source,
                    gridLinesVisibility: GridLinesVisibility.both,
                    headerGridLinesVisibility: GridLinesVisibility.both,
                    allowColumnsResizing: true, // 启用列宽调整
                    columnWidthMode: ColumnWidthMode.fill,
                    columnResizeMode: ColumnResizeMode.onResizeEnd, // 推荐模式
                    controller: controller,
                    columns: [
                      if (selectable)
                        GridColumn(
                          columnName: 'select',
                          label: ValueListenableBuilder<bool>(
                            valueListenable: source.selectionNotifier,
                            builder: (context, _, __) => _buildCheckboxHeader(context, source),
                          ),
                          width: 60,
                        ),
                      ...columns,
                    ],
                    onCellTap: (details) {
                      if (details.column.columnName == 'select') {
                        final rowIndex = details.rowColumnIndex.rowIndex - 1;
                        if (rowIndex >= 0 && rowIndex < source.items.length) {
                          source.toggleRowSelection(rowIndex, !source.items[rowIndex].isSelected);
                          onSelectionChanged?.call(rowIndex, source.items[rowIndex].isSelected);
                        }
                      }
                    },
                    onColumnResizeUpdate: (ColumnResizeUpdateDetails details) {
                      print('列 ${details.column.columnName} 宽度变为 ${details.width}');
                      return true; // 返回true接受调整
                    },
                    onCellSecondaryTap: (details) {
                      final rowIndex = details.rowColumnIndex.rowIndex - 1;
                      final columnName = details.column.columnName;
                      final isHeader = rowIndex < 0;

                      final renderBox = context.findRenderObject() as RenderBox;
                      final position = details.globalPosition;

                      showMenu(
                        context: context,
                        position: RelativeRect.fromLTRB(
                          position.dx,
                          position.dy,
                          position.dx + renderBox.size.width - position.dx,
                          position.dy + renderBox.size.height - position.dy,
                        ),
                        items: [
                          const PopupMenuItem<String>(value: 'export', child: Text('导出表格(csv)')),
                          if (!isHeader)
                            PopupMenuItem<String>(
                              value: 'copyCell',
                              child: Text('复制单元格 ($columnName)'),
                            ),
                          if (!isHeader)
                            const PopupMenuItem<String>(value: 'copyRow', child: Text('复制当前行')),
                          PopupMenuItem<String>(
                            value: 'copyColumn',
                            child: Text('复制整列 ($columnName)'),
                          ),
                        ],
                      ).then((value) async {
                        if (value != null) {
                          switch (value) {
                            case 'copyCell':
                              final row = source.effectiveRows[rowIndex];
                              await source.copyCellValue(row, columnName);
                              break;
                            case 'copyRow':
                              final row = source.effectiveRows[rowIndex];
                              await source.copyRowValues(row);
                              break;
                            case 'copyColumn':
                              await source.copyColumnValues(source.effectiveRows, columnName);
                              break;
                            case 'export':
                              await source.exportToCsv(source.effectiveRows);
                              break;
                          }
                          if (value != 'export') {
                            ScaffoldMessenger.of(
                              // ignore: use_build_context_synchronously
                              context,
                            ).showSnackBar(const SnackBar(content: Text('已复制到剪贴板')));
                          }
                        }
                      });
                    },
                  ),
                );
              },
            ),
          ),
        ],
      ),
    );
  }
}
