import 'dart:io';

import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:syncfusion_flutter_datagrid/datagrid.dart';
import 'package:win_text_editor/shared/base/selectable_data_source.dart';
import 'package:win_text_editor/shared/base/selectable_item.dart';

abstract class MySfDataSource<T extends SelectableItem> extends SelectableDataSource<T> {
  MySfDataSource(
    List<T> items, {
    required Null Function(dynamic index, dynamic isSelected) onSelectionChanged,
  }) : super(items, onSelectionChanged: onSelectionChanged);

  @override
  List<DataGridRow> get rows;

  @override
  DataGridRowAdapter buildRow(DataGridRow row);

  Future<void> copyCellValue(DataGridRow row, String columnName) async {
    final cell = row.getCells().firstWhere(
      (cell) => cell.columnName == columnName,
      orElse: () => throw Exception('Column not found'),
    );

    String displayText;
    if (cell.value is Widget) {
      // 情况1：单元格是Widget（需手动提取文本，例如从Text widget）
      final widget = cell.value as Widget;
      displayText = _extractTextFromWidget(widget); // 自定义方法
    } else {
      // 情况2：普通单元格
      displayText = cell.value?.toString() ?? '';
    }

    await Clipboard.setData(ClipboardData(text: displayText));
  }

  /// 从Widget中提取文本（示例）
  String _extractTextFromWidget(Widget widget) {
    if (widget is Text) {
      return widget.data ?? '';
    } else if (widget is RichText) {
      return widget.text.toPlainText();
    }
    return widget.toString(); // 兜底
  }

  Future<void> copyRowValues(DataGridRow row) async {
    final values = row.getCells().map((cell) => cell.value.toString()).join('\t');
    await Clipboard.setData(ClipboardData(text: values));
  }

  Future<void> copyColumnValues(List<DataGridRow> rows, String columnName) async {
    final values = rows
        .map((row) {
          final cell = row.getCells().firstWhere(
            (cell) => cell.columnName == columnName,
            orElse: () => throw Exception('Column not found'),
          );
          return cell.value.toString();
        })
        .join('\n');
    await Clipboard.setData(ClipboardData(text: values));
  }

  Future<void> exportToCsv(List<DataGridRow> rows) async {
    if (rows.length <= 1) return;
    // 构建表头（不包含序号列）
    String csvData = "";
    for (int i = 1; i < rows[0].getCells().length; i++) {
      csvData += rows[0].getCells()[i].columnName.toString();
      if (i < rows[0].getCells().length - 1) csvData += '\t';
    }

    csvData += '\n';

    // 填充数据（跳过第一列的序号）
    for (final row in rows) {
      // 从第1列开始（跳过第0列的序号）
      for (int i = 1; i < row.getCells().length; i++) {
        csvData += row.getCells()[i].value.toString();
        if (i < row.getCells().length - 1) csvData += '\t';
      }
      csvData += '\n';
    }

    // 保存文件
    final filePath = await FilePicker.platform.saveFile(
      dialogTitle: '保存导出结果',
      fileName: 'template_results.csv',
      type: FileType.custom,
      allowedExtensions: ['csv'],
    );

    if (filePath != null) {
      await File(filePath).writeAsString(csvData);
    }
  }
}
