import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'editor_toolbar.dart';

class TextEditor extends StatefulWidget {
  final String tabId;
  final String? initialContent;
  final String title;
  final ValueChanged<String>? onContentChanged;
  final Widget Function(BuildContext, TextEditorState)? toolbarBuilder; // 新增：外部工具栏构建器

  const TextEditor({
    super.key,
    required this.tabId,
    this.initialContent,
    this.title = '未命名',
    this.onContentChanged,
    this.toolbarBuilder, // 新增：可选参数
  });

  @override
  State<TextEditor> createState() => TextEditorState();
}

class TextEditorState extends State<TextEditor> with AutomaticKeepAliveClientMixin {
  final TextEditingController _textController = TextEditingController();
  final FocusNode _focusNode = FocusNode();
  bool _isLoading = false;
  String _lastSyncedText = '';
  Timer? _debounceTimer;

  // 暴露给外部使用的公共方法
  String get currentText => _textController.text;
  set currentText(String text) {
    _textController.text = text;
    _lastSyncedText = text;
  }
  
  bool get isLoading => _isLoading;
  FocusNode get focusNode => _focusNode;

  // 暴露给外部使用的操作函数
  Future<void> openFile() => _openFile(context);
  Future<void> copyToClipboard() => _copyToClipboard(context);
  Future<void> saveFile() => _saveFile(context);

  @override
  bool get wantKeepAlive => true;

  @override
  void initState() {
    super.initState();
    _lastSyncedText = widget.initialContent ?? '';
    _textController.text = _lastSyncedText;
    _textController.addListener(_handleTextChanged);
  }

  void _handleTextChanged() {
    if (_debounceTimer?.isActive ?? false) {
      _debounceTimer?.cancel();
    }

    _debounceTimer = Timer(const Duration(milliseconds: 300), () {
      final currentText = _textController.text;
      if (widget.onContentChanged != null && currentText != _lastSyncedText) {
        _lastSyncedText = currentText;
        widget.onContentChanged!(currentText);
      }
    });
  }

  @override
  void didUpdateWidget(TextEditor oldWidget) {
    super.didUpdateWidget(oldWidget);
    final newText = widget.initialContent ?? '';
    if (newText != _textController.text && newText != _lastSyncedText) {
      _lastSyncedText = newText;
      _textController.text = newText;
    }
  }

  @override
  void dispose() {
    _debounceTimer?.cancel();
    _textController.removeListener(_handleTextChanged);
    _textController.dispose();
    _focusNode.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return Column(
      children: [
        // 使用外部传入的工具栏构建器或默认实现
        widget.toolbarBuilder?.call(context, this) ?? _buildDefaultToolbar(),
        Expanded(child: _buildEditorField(context)),
      ],
    );
  }

  // 默认工具栏实现
  Widget _buildDefaultToolbar() {
    return EditorToolbar(
          title: widget.title,
          text: _textController.text,
          isLoading: _isLoading,
          onOpenFile: () => _openFile(context),
          onCopyToClipboard: () => _copyToClipboard(context),
          onSaveFile: () => _saveFile(context),
    );
  }

  Widget _buildEditorField(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
        color: _focusNode.hasFocus ? Colors.blue[50] : Colors.white,
        border: Border.all(color: Colors.grey[300]!, width: 1.0),
      ),
      child: TextField(
        controller: _textController,
        focusNode: _focusNode,
        maxLines: null,
        expands: true,
        keyboardType: TextInputType.multiline,
        decoration: const InputDecoration(
          border: InputBorder.none,
          contentPadding: EdgeInsets.all(8),
        ),
        style: const TextStyle(fontFamily: 'Courier New', fontSize: 16, color: Colors.black),
      ),
    );
  }

  Future<void> _openFile(BuildContext context) async {
    final result = await FilePicker.platform.pickFiles(type: FileType.any, allowMultiple: false);

    if (result != null && result.files.single.path != null) {
      await _loadFile(context, result.files.single.path!);
    }
  }

  Future<void> _copyToClipboard(BuildContext context) async {
    await Clipboard.setData(ClipboardData(text: _textController.text));
    if (context.mounted) {
      ScaffoldMessenger.of(context).showSnackBar(const SnackBar(content: Text('已复制到剪贴板')));
    }
  }

  Future<void> _saveFile(BuildContext context) async {
    try {
      String? outputPath = await FilePicker.platform.saveFile(
        dialogTitle: '保存文件',
        fileName: 'untitled.txt',
        allowedExtensions: ['txt'],
        type: FileType.any,
      );

      if (outputPath == null) return;

      final file = File(outputPath);

      if (await file.exists()) {
        final shouldOverwrite = await showDialog<bool>(
          context: context,
          builder:
              (context) => AlertDialog(
                title: const Text('文件已存在'),
                content: const Text('要覆盖现有文件吗？'),
                actions: [
                  TextButton(
                    onPressed: () => Navigator.pop(context, false),
                    child: const Text('取消'),
                  ),
                  TextButton(
                    onPressed: () => Navigator.pop(context, true),
                    child: const Text('覆盖'),
                  ),
                ],
              ),
        );

        if (shouldOverwrite != true) return;
      }

      await file.writeAsString(_textController.text);

      if (context.mounted) {
        ScaffoldMessenger.of(context).showSnackBar(SnackBar(content: Text('已保存到: ${file.path}')));
      }
    } on FileSystemException catch (e) {
      if (context.mounted) {
        ScaffoldMessenger.of(context).showSnackBar(SnackBar(content: Text('保存失败: ${e.message}')));
      }
    } catch (e) {
      if (context.mounted) {
        ScaffoldMessenger.of(
          context,
        ).showSnackBar(SnackBar(content: Text('保存失败: ${e.toString()}')));
      }
    }
  }

  Future<void> _loadFile(BuildContext context, String filePath) async {
    try {
      _isLoading = true;
      final file = File(filePath);

      if (_textController.text.isNotEmpty) {
        final confirm = await showDialog<bool>(
          context: context,
          builder:
              (context) => AlertDialog(
                title: const Text('确认提示'),
                content: const Text('确认是否打开新的文件？打开后将覆盖当前内容'),
                actions: [
                  TextButton(
                    onPressed: () => Navigator.pop(context, false),
                    child: const Text('取消'),
                  ),
                  TextButton(
                    onPressed: () => Navigator.pop(context, true),
                    child: const Text('确认'),
                  ),
                ],
              ),
        );

        if (confirm != true) return;
      }

      _textController.text = '';

      final stream = file.openRead();
      final lines = stream.transform(utf8.decoder).transform(const LineSplitter());

      int lineCount = 0;

      await for (final line in lines) {
        _textController.text += '$line\n';

        lineCount++;
        if (lineCount >= 300) {
          return; //仅加载前300行
        }

        await Future.delayed(const Duration(milliseconds: 10));
      }

      if (context.mounted) {
        ScaffoldMessenger.of(context).showSnackBar(SnackBar(content: Text('已加载: ${file.path}')));
      }
    } on FormatException {
      if (context.mounted) {
        ScaffoldMessenger.of(context).showSnackBar(const SnackBar(content: Text('这不是可读的文本文件')));
      }
    } on FileSystemException catch (e) {
      if (context.mounted) {
        ScaffoldMessenger.of(context).showSnackBar(SnackBar(content: Text('文件访问错误: ${e.message}')));
      }
    } catch (e) {
      if (context.mounted) {
        ScaffoldMessenger.of(
          context,
        ).showSnackBar(SnackBar(content: Text('读取失败: ${e.toString()}')));
      }
    } finally {
      _isLoading = false;
    }
  }
}
