import 'dart:io';

import 'package:hive_flutter/hive_flutter.dart';
import 'package:win_text_editor/framework/controllers/logger.dart';
import 'package:win_text_editor/shared/models/std_filed.dart';
import 'package:xml/xml.dart' as xml;

class StdFieldsCache {
  static late final Box<StdField> _box; // 指定泛型类型
  static bool _loaded = false;

  // 初始化
  static Future<void> init() async {
    await Hive.initFlutter();
    Hive.registerAdapter(StdFieldAdapter()); // 在打开Box前注册适配器
    _box = await Hive.openBox<StdField>('uft_stdFieldsCache'); // 指定泛型类型
  }

  static Future loadForFile(String currFilePath) async {
    if (_loaded == false) {
      Logger().info("加载标准字段缓存");
      final metadataFile = await findMetadataFile(currFilePath);
      if (metadataFile != null) {
        await processStdFieldFile(metadataFile);
      }
      _loaded = true;
    }
  }

  static Future<int> getLength() async {
    return _box.length;
  }

  // 通用数据存取
  static getData(String key) {
    return _box.get(key);
  }

  static Future<void> setData(String key, StdField value) => _box.put(key, value);

  // 清空缓存
  static Future<void> clear() async {
    await _box.clear();
  }

  static Future<File?> findMetadataFile(String filePath) async {
    Directory currentDir = File(filePath).parent;
    const targetDirName = 'metadata';
    const targetFileName = 'stdfield.stdfield';

    // 向上查找 metadata 目录
    while (true) {
      final metadataDir = Directory('${currentDir.path}/$targetDirName');
      if (await metadataDir.exists()) {
        final stdFieldFile = File('${metadataDir.path}/$targetFileName');
        if (await stdFieldFile.exists()) {
          return stdFieldFile;
        } else {
          Logger().error("没找到标准字段文件 $targetFileName");
          return null;
        }
      }

      // 到达根目录时停止
      if (currentDir.path == currentDir.parent.path) {
        Logger().error("没有找到元数据目录 $targetDirName");
        return null;
      }

      currentDir = currentDir.parent;
    }
  }

  // 处理标准字段文件并缓存
  static Future<void> processStdFieldFile(File stdFieldFile) async {
    try {
      final content = await stdFieldFile.readAsString();
      final document = xml.XmlDocument.parse(content);
      final items = document.findAllElements('items');

      for (final item in items) {
        final name = item.getAttribute('name') ?? '';
        final chineseName = item.getAttribute('chineseName') ?? '';
        final dataType = item.getAttribute('dataType') ?? '';

        if (name.isNotEmpty) {
          final stdField = StdField(name: name, chineseName: chineseName, dateType: dataType);
          await StdFieldsCache.setData(name, stdField);
        }
      }
    } catch (e) {
      Logger().error("处理标准字段文件时出错: $e");
    }
  }
}

// 确保typeId在整个应用中唯一
class StdFieldAdapter extends TypeAdapter<StdField> {
  @override
  final int typeId = 7; // 改为一个唯一ID，确保不与其他适配器冲突

  @override
  StdField read(BinaryReader reader) {
    return StdField(name: reader.read(), chineseName: reader.read(), dateType: reader.read());
  }

  @override
  void write(BinaryWriter writer, StdField obj) {
    writer.write(obj.name);
    writer.write(obj.chineseName);
    writer.write(obj.dateType);
  }
}
